@@ -22,7 +22,7 @@ class HTMLListAccessible : public HyperTextAccessibleWrap
 {
 public:
   HTMLListAccessible(nsIContent* aContent, DocAccessible* aDoc) :
-    HyperTextAccessibleWrap(aContent, aDoc) { }
+    HyperTextAccessibleWrap(aContent, aDoc) { mGenericTypes |= eList; }
   virtual ~HTMLListAccessible() { }
 
   // nsISupports
@@ -57,7 +57,12 @@ public:
   virtual a11y::role NativeRole();
   virtual uint64_t NativeState();
 
-  // nsHTMLLIAccessible
+  // HyperTextAccessible
+  virtual int32_t FindOffset(int32_t aOffset, nsDirection aDirection,
+                             nsSelectionAmount aAmount,
+                             EWordMovementType aWordMovementType) MOZ_OVERRIDE;
+
+  // HTMLLIAccessible
   void UpdateBullet(bool aHasBullet);
 
 protected:
@@ -100,8 +105,7 @@ public:
 inline HTMLLIAccessible*
 Accessible::AsHTMLListItem()
 {
-  return mFlags & eHTMLListItemAccessible ?
-    static_cast<HTMLLIAccessible*>(this) : nullptr;
+  return IsHTMLListItem() ? static_cast<HTMLLIAccessible*>(this) : nullptr;
 }
 
 } // namespace a11y
