@@ -1209,6 +1209,14 @@ function synthUpKey(aNodeOrID, aCheckerOrEventSeq, aArgs)
                                 aCheckerOrEventSeq);
 }
 
+/**
+ * Left arrow key invoker.
+ */
+function synthLeftKey(aNodeOrID, aCheckerOrEventSeq)
+{
+  this.__proto__ = new synthKey(aNodeOrID, "VK_LEFT", null, aCheckerOrEventSeq);
+}
+
 /**
  * Right arrow key invoker.
  */
@@ -1541,10 +1549,12 @@ function moveToTextStart(aID)
 /**
  * Move the caret in text accessible.
  */
-function moveCaretToDOMPoint(aID, aNode, aOffset, aExpectedOffset,
-                             aFocusTargetID)
+function moveCaretToDOMPoint(aID, aDOMPointNodeID, aDOMPointOffset,
+                             aExpectedOffset, aFocusTargetID,
+                             aCheckFunc)
 {
   this.target = getAccessible(aID, [nsIAccessibleText]);
+  this.DOMPointNode = getNode(aDOMPointNodeID);
   this.focus = aFocusTargetID ? getAccessible(aFocusTargetID) : null;
   this.focusNode = this.focus ? this.focus.DOMNode : null;
 
@@ -1553,13 +1563,25 @@ function moveCaretToDOMPoint(aID, aNode, aOffset, aExpectedOffset,
     if (this.focusNode)
       this.focusNode.focus();
 
-    window.getSelection().getRangeAt(0).setStart(aNode, aOffset);
+    var selection = this.DOMPointNode.ownerDocument.defaultView.getSelection();
+    var selRange = selection.getRangeAt(0);
+    selRange.setStart(this.DOMPointNode, aDOMPointOffset);
+    selRange.collapse(true);
+
+    selection.removeRange(selRange);
+    selection.addRange(selRange);
   }
 
   this.getID = function moveCaretToDOMPoint_getID()
   {
    return "Set caret on " + prettyName(aID) + " at point: " +
-     prettyName(aNode) + " node with offset " + aOffset;
+     prettyName(aDOMPointNodeID) + " node with offset " + aDOMPointOffset;
+  }
+
+  this.finalCheck = function moveCaretToDOMPoint_finalCheck()
+  {
+    if (aCheckFunc)
+      aCheckFunc.call();
   }
 
   this.eventSeq = [
@@ -1643,6 +1665,17 @@ function invokerChecker(aEventType, aTargetOrFunc, aTargetFuncArg, aIsAsync)
   this.mTargetFuncArg = aTargetFuncArg;
 }
 
+/**
+ * Generic invoker checker for unexpected events.
+ */
+function unexpectedInvokerChecker(aEventType, aTargetOrFunc, aTargetFuncArg)
+{
+  this.__proto__ = new invokerChecker(aEventType, aTargetOrFunc,
+                                      aTargetFuncArg, true);
+
+  this.unexpected = true;
+}
+
 /**
  * Common invoker checker for async events.
  */
@@ -1722,6 +1755,19 @@ function caretMoveChecker(aCaretOffset, aTargetOrFunc, aTargetFuncArg)
   }
 }
 
+/**
+ * Text selection change checker.
+ */
+function textSelectionChecker(aID, aStartOffset, aEndOffset)
+{
+  this.__proto__ = new invokerChecker(EVENT_TEXT_SELECTION_CHANGED, aID);
+
+  this.check = function textSelectionChecker_check(aEvent)
+  {
+    testTextGetSelection(aID, aStartOffset, aEndOffset, 0);
+  }
+}
+
 /**
  * State change checker.
  */
