@@ -11,7 +11,6 @@ const ENSURE_SELECTION_VISIBLE_DELAY = 50; // ms
 
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 Cu.import("resource:///modules/devtools/DOMHelpers.jsm");
-Cu.import("resource:///modules/devtools/LayoutHelpers.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
 
 let promise = require("sdk/core/promise");
@@ -103,6 +102,7 @@ HTMLBreadcrumbs.prototype = {
     this.selection.on("new-node-front", this.update);
     this.selection.on("pseudoclass", this.updateSelectors);
     this.selection.on("attribute-changed", this.updateSelectors);
+    this.inspector.on("markupmutation", this.update);
     this.update();
   },
 
@@ -350,6 +350,7 @@ HTMLBreadcrumbs.prototype = {
     this.selection.off("new-node-front", this.update);
     this.selection.off("pseudoclass", this.updateSelectors);
     this.selection.off("attribute-changed", this.updateSelectors);
+    this.inspector.off("markupmutation", this.update);
 
     this.container.removeEventListener("underflow", this.onscrollboxreflow, false);
     this.container.removeEventListener("overflow", this.onscrollboxreflow, false);
@@ -612,7 +613,7 @@ HTMLBreadcrumbs.prototype = {
   /**
    * Update the breadcrumbs display when a new node is selected.
    */
-  update: function BC_update()
+  update: function BC_update(reason)
   {
     this.inspector.hideNodeMenu();
 
@@ -633,7 +634,8 @@ HTMLBreadcrumbs.prototype = {
     let idx = this.indexOf(this.selection.nodeFront);
 
     // Is the node already displayed in the breadcrumbs?
-    if (idx > -1) {
+    // (and there are no mutations that need re-display of the crumbs)
+    if (idx > -1 && reason !== "markupmutation") {
       // Yes. We select it.
       this.setCursor(idx);
     } else {
@@ -667,7 +669,7 @@ HTMLBreadcrumbs.prototype = {
       doneUpdating(this.selection.nodeFront);
       this.selectionGuardEnd(err);
     });
-  },
+  }
 }
 
 XPCOMUtils.defineLazyGetter(this, "DOMUtils", function () {
