@@ -74,21 +74,10 @@ function testPropertyPanel()
 {
   let jsterm = gHUD.jsterm;
   jsterm.clearOutput();
-  jsterm.execute("document");
-
-  waitForSuccess({
-    name: "jsterm document object output",
-    validatorFn: function()
-    {
-      return gHUD.outputNode.querySelector(".webconsole-msg-output");
-    },
-    successFn: function()
-    {
-      jsterm.once("variablesview-fetched", onVariablesViewReady);
-      let node = gHUD.outputNode.querySelector(".webconsole-msg-output");
-      EventUtils.synthesizeMouse(node, 2, 2, {}, gHUD.iframeWindow);
-    },
-    failureFn: finishTest,
+  jsterm.execute("document", (msg) => {
+    jsterm.once("variablesview-fetched", onVariablesViewReady);
+    let anchor = msg.querySelector(".body a");
+    EventUtils.synthesizeMouse(anchor, 2, 2, {}, gHUD.iframeWindow);
   });
 }
 
