@@ -15,6 +15,10 @@
 #include "sys/stat.h"
 #endif // defined(XP_UNIX)
 
+#if defined(XP_LINUX)
+#include <linux/fadvise.h>
+#endif // defined(XP_LINUX)
+
 #if defined(XP_MACOSX)
 #include "copyfile.h"
 #endif // defined(XP_MACOSX)
@@ -44,6 +48,7 @@
 
 #include "OSFileConstants.h"
 #include "nsIOSFileConstantsService.h"
+#include "nsZipArchive.h"
 
 #if defined(__DragonFly__) || defined(__FreeBSD__) \
   || defined(__NetBSD__) || defined(__OpenBSD__)
@@ -74,6 +79,38 @@ struct Paths {
   nsString tmpDir;
   nsString profileDir;
   nsString localProfileDir;
+  /**
+   * The user's home directory
+   */
+  nsString homeDir;
+  /**
+   * The user's desktop directory, if there is one. Otherwise this is
+   * the same as homeDir.
+   */
+  nsString desktopDir;
+
+#if defined(XP_WIN)
+  /**
+   * The user's application data directory.
+   */
+  nsString winAppDataDir;
+  /**
+   * The programs subdirectory in the user's start menu directory.
+   */
+  nsString winStartMenuProgsDir;
+#endif // defined(XP_WIN)
+
+#if defined(XP_MACOSX)
+  /**
+   * The user's Library directory.
+   */
+  nsString macUserLibDir;
+  /**
+   * The Application directory, that stores applications installed in the
+   * system.
+   */
+  nsString macLocalApplicationsDir;
+#endif // defined(XP_MACOSX)
 
   Paths()
   {
@@ -81,6 +118,18 @@ struct Paths {
     tmpDir.SetIsVoid(true);
     profileDir.SetIsVoid(true);
     localProfileDir.SetIsVoid(true);
+    homeDir.SetIsVoid(true);
+    desktopDir.SetIsVoid(true);
+
+#if defined(XP_WIN)
+    winAppDataDir.SetIsVoid(true);
+    winStartMenuProgsDir.SetIsVoid(true);
+#endif // defined(XP_WIN)
+
+#if defined(XP_MACOSX)
+    macUserLibDir.SetIsVoid(true);
+    macLocalApplicationsDir.SetIsVoid(true);
+#endif // defined(XP_MACOSX)
   }
 };
 
@@ -208,6 +257,18 @@ nsresult InitOSFileConstants()
   // some platforms or in non-Firefox embeddings of Gecko).
 
   GetPathToSpecialDir(NS_OS_TEMP_DIR, paths->tmpDir);
+  GetPathToSpecialDir(NS_OS_HOME_DIR, paths->homeDir);
+  GetPathToSpecialDir(NS_OS_DESKTOP_DIR, paths->desktopDir);
+
+#if defined(XP_WIN)
+  GetPathToSpecialDir(NS_WIN_APPDATA_DIR, paths->winAppDataDir);
+  GetPathToSpecialDir(NS_WIN_PROGRAMS_DIR, paths->winStartMenuProgsDir);
+#endif // defined(XP_WIN)
+
+#if defined(XP_MACOSX)
+  GetPathToSpecialDir(NS_MAC_USER_LIB_DIR, paths->macUserLibDir);
+  GetPathToSpecialDir(NS_OSX_LOCAL_APPLICATIONS_DIR, paths->macLocalApplicationsDir);
+#endif // defined(XP_MACOSX)
 
   gPaths = paths.forget();
   return NS_OK;
@@ -242,7 +303,7 @@ void CleanupOSFileConstants()
 /**
  * End marker for ConstantSpec
  */
-#define PROP_END { NULL, JSVAL_VOID }
+#define PROP_END { NULL, JS::UndefinedValue() }
 
 
 // Define missing constants for Android
@@ -269,7 +330,7 @@ void CleanupOSFileConstants()
  * keep properties organized by alphabetical order
  * and #ifdef-away properties that are not portable.
  */
-static dom::ConstantSpec gLibcProperties[] =
+static const dom::ConstantSpec gLibcProperties[] =
 {
   // Arguments for open
   INT_CONSTANT(O_APPEND),
@@ -320,6 +381,10 @@ static dom::ConstantSpec gLibcProperties[] =
   INT_CONSTANT(AT_SYMLINK_NOFOLLOW),
 #endif //defined(AT_SYMLINK_NOFOLLOW)
 
+#if defined(POSIX_FADV_SEQUENTIAL)
+  INT_CONSTANT(POSIX_FADV_SEQUENTIAL),
+#endif //defined(POSIX_FADV_SEQUENTIAL)
+
   // access
 #if defined(F_OK)
   INT_CONSTANT(F_OK),
@@ -522,7 +587,7 @@ static dom::ConstantSpec gLibcProperties[] =
  * keep properties organized by alphabetical order
  * and #ifdef-away properties that are not portable.
  */
-static dom::ConstantSpec gWinProperties[] =
+static const dom::ConstantSpec gWinProperties[] =
 {
   // FormatMessage flags
   INT_CONSTANT(FORMAT_MESSAGE_FROM_SYSTEM),
@@ -605,7 +670,7 @@ JSObject *GetOrCreateObjectProperty(JSContext *cx, JS::Handle<JSObject*> aObject
                                     const char *aProperty)
 {
   JS::Rooted<JS::Value> val(cx);
-  if (!JS_GetProperty(cx, aObject, aProperty, val.address())) {
+  if (!JS_GetProperty(cx, aObject, aProperty, &val)) {
     return NULL;
   }
   if (!val.isUndefined()) {
@@ -633,8 +698,8 @@ bool SetStringProperty(JSContext *cx, JS::Handle<JSObject*> aObject, const char
   }
   JSString* strValue = JS_NewUCStringCopyZ(cx, aValue.get());
   NS_ENSURE_TRUE(strValue, false);
-  JS::Value valValue = STRING_TO_JSVAL(strValue);
-  return JS_SetProperty(cx, aObject, aProperty, &valValue);
+  JS::Rooted<JS::Value> valValue(cx, STRING_TO_JSVAL(strValue));
+  return JS_SetProperty(cx, aObject, aProperty, valValue);
 }
 
 /**
@@ -706,15 +771,15 @@ bool DefineOSFileConstants(JSContext *cx, JS::Handle<JSObject*> global)
       return false;
     }
 
-    JS::Value valVersion = STRING_TO_JSVAL(strVersion);
-    if (!JS_SetProperty(cx, objSys, "Name", &valVersion)) {
+    JS::Rooted<JS::Value> valVersion(cx, STRING_TO_JSVAL(strVersion));
+    if (!JS_SetProperty(cx, objSys, "Name", valVersion)) {
       return false;
     }
   }
 
 #if defined(DEBUG)
-  JS::Value valDebug = JSVAL_TRUE;
-  if (!JS_SetProperty(cx, objSys, "DEBUG", &valDebug)) {
+  JS::Rooted<JS::Value> valDebug(cx, JSVAL_TRUE);
+  if (!JS_SetProperty(cx, objSys, "DEBUG", valDebug)) {
     return false;
   }
 #endif
@@ -768,6 +833,34 @@ bool DefineOSFileConstants(JSContext *cx, JS::Handle<JSObject*> global)
     return false;
   }
 
+  if (!SetStringProperty(cx, objPath, "homeDir", gPaths->homeDir)) {
+    return false;
+  }
+
+  if (!SetStringProperty(cx, objPath, "desktopDir", gPaths->desktopDir)) {
+    return false;
+  }
+
+#if defined(XP_WIN)
+  if (!SetStringProperty(cx, objPath, "winAppDataDir", gPaths->winAppDataDir)) {
+    return false;
+  }
+
+  if (!SetStringProperty(cx, objPath, "winStartMenuProgsDir", gPaths->winStartMenuProgsDir)) {
+    return false;
+  }
+#endif // defined(XP_WIN)
+
+#if defined(XP_MACOSX)
+  if (!SetStringProperty(cx, objPath, "macUserLibDir", gPaths->macUserLibDir)) {
+    return false;
+  }
+
+  if (!SetStringProperty(cx, objPath, "macLocalApplicationsDir", gPaths->macLocalApplicationsDir)) {
+    return false;
+  }
+#endif // defined(XP_MACOSX)
+
   return true;
 }
 
