@@ -8,14 +8,14 @@
 //----------------------------------------------------------------------
 // Global functions and data [declaration]
 
-static const PRUint16 g_ufMappingTable[] = {
-#include "cp869.uf"
-};
-
 nsresult
 nsUnicodeToCP869Constructor(nsISupports *aOuter, REFNSIID aIID,
                             void **aResult) 
 {
+  static const uint16_t g_ufMappingTable[] = {
+#include "cp869.uf"
+  };
+
   return CreateTableEncoder(u1ByteCharset,
                             (uMappingTable*) &g_ufMappingTable, 1,
                             aOuter, aIID, aResult);
