@@ -4,7 +4,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 #ifndef nsISO2022KRToUnicode_h__
 #define nsISO2022KRToUnicode_h__
-#include "nsISupports.h"
 #include "nsUCSupport.h"
 
 
@@ -17,7 +16,7 @@ public:
     mState = mState_Init;
     mLastLegalState = mState_ASCII;
     mData = 0;
-    mEUCKRDecoder = nsnull;
+    mEUCKRDecoder = nullptr;
     mRunLength = 0;
   }
 
@@ -26,11 +25,11 @@ public:
     NS_IF_RELEASE(mEUCKRDecoder);
   }
 
-  NS_IMETHOD Convert(const char * aSrc, PRInt32 * aSrcLength,
-     PRUnichar * aDest, PRInt32 * aDestLength) ;
+  NS_IMETHOD Convert(const char * aSrc, int32_t * aSrcLength,
+     PRUnichar * aDest, int32_t * aDestLength) ;
   
-  NS_IMETHOD GetMaxLength(const char * aSrc, PRInt32 aSrcLength,
-     PRInt32 * aDestLength) 
+  NS_IMETHOD GetMaxLength(const char * aSrc, int32_t aSrcLength,
+     int32_t * aDestLength) 
   {
     *aDestLength = aSrcLength;
     return NS_OK;
@@ -56,10 +55,10 @@ private:
     mState_ERROR
   } mState, mLastLegalState;
 
-  PRUint8 mData;
+  uint8_t mData;
 
   // Length of non-ASCII run
-  PRUint32 mRunLength;
+  uint32_t mRunLength;
 
   nsIUnicodeDecoder *mEUCKRDecoder;
 };
