@@ -9,14 +9,14 @@
 //----------------------------------------------------------------------
 // Global functions and data [declaration]
 
-static const PRUint16 g_utMappingTable[] = {
-#include "armscii.ut"
-};
-
 nsresult
 nsARMSCII8ToUnicodeConstructor(nsISupports *aOuter, REFNSIID aIID,
                                void **aResult) 
 {
+  static const uint16_t g_utMappingTable[] = {
+#include "armscii.ut"
+  };
+
   return CreateOneByteDecoder((uMappingTable*) &g_utMappingTable,
                               aOuter, aIID, aResult);
 }
