@@ -32,7 +32,7 @@ def _error(loc, fmt, *args):
     raise ParseError(loc, fmt, *args)
 
 class Parser:
-    # when we reach an |include protocol "foo.ipdl";| statement, we need to
+    # when we reach an |include [protocol] foo;| statement, we need to
     # save the current parser state and create a new one.  this "stack" is
     # where that state is saved
     #
@@ -41,14 +41,16 @@ class Parser:
     parseStack = [ ]
     parsed = { }
 
-    def __init__(self, debug=0):
+    def __init__(self, type, name, debug=0):
+        assert type and name
+        self.type = type
         self.debug = debug
         self.filename = None
         self.includedirs = None
         self.loc = None         # not always up to date
         self.lexer = None
         self.parser = None
-        self.tu = TranslationUnit()
+        self.tu = TranslationUnit(type, name)
         self.direction = None
         self.errout = None
 
@@ -119,10 +121,12 @@ reserved = set((
         'bridges',
         'call',
         'child',
+        'compress',
         '__delete__',
         'delete',                       # reserve 'delete' to prevent its use
         'goto',
         'include',
+        'intr',
         'manager',
         'manages',
         'namespace',
@@ -141,6 +145,7 @@ reserved = set((
         'struct',
         'sync',
         'union',
+        'urgent',
         'using'))
 tokens = [
     'COLONCOLON', 'ID', 'STRING'
@@ -182,11 +187,12 @@ def t_error(t):
 def p_TranslationUnit(p):
     """TranslationUnit : Preamble NamespacedStuff"""
     tu = Parser.current.tu
+    tu.loc = Loc(tu.filename)
     for stmt in p[1]:
         if isinstance(stmt, CxxInclude):
             tu.addCxxInclude(stmt)
-        elif isinstance(stmt, ProtocolInclude):
-            tu.addProtocolInclude(stmt)
+        elif isinstance(stmt, Include):
+            tu.addInclude(stmt)
         elif isinstance(stmt, UsingStmt):
             tu.addUsingStmt(stmt)
         else:
@@ -204,6 +210,22 @@ def p_TranslationUnit(p):
         else:
             assert(0)
 
+    # The "canonical" namespace of the tu, what it's considered to be
+    # in for the purposes of C++: |#include "foo/bar/TU.h"|
+    if tu.protocol:
+        assert tu.filetype == 'protocol'
+        tu.namespaces = tu.protocol.namespaces
+        tu.name = tu.protocol.name
+    else:
+        print tu.filetype
+        assert tu.filetype == 'header'
+        # There's not really a canonical "thing" in headers.  So
+        # somewhat arbitrarily use the namespace of the last
+        # interesting thing that was declared.
+        for thing in reversed(tu.structsAndUnions):
+            tu.namespaces = thing.namespaces
+            break
+
     p[0] = tu
 
 ##--------------------
@@ -219,7 +241,7 @@ def p_Preamble(p):
 
 def p_PreambleStmt(p):
     """PreambleStmt : CxxIncludeStmt
-                    | ProtocolIncludeStmt
+                    | IncludeStmt
                     | UsingStmt"""
     p[0] = p[1]
 
@@ -227,23 +249,26 @@ def p_CxxIncludeStmt(p):
     """CxxIncludeStmt : INCLUDE STRING"""
     p[0] = CxxInclude(locFromTok(p, 1), p[2])
 
-def p_ProtocolIncludeStmt(p):
-    """ProtocolIncludeStmt : INCLUDE PROTOCOL ID
-                           | INCLUDE PROTOCOL STRING"""
+def p_IncludeStmt(p):
+    """IncludeStmt : INCLUDE PROTOCOL ID
+                   | INCLUDE ID"""
     loc = locFromTok(p, 1)
-    
-    if 0 <= p[3].rfind('.ipdl'):
-        _error(loc, "`include protocol \"P.ipdl\"' syntax is obsolete.  Use `include protocol P' instead.")
-    
+ 
     Parser.current.loc = loc
-    inc = ProtocolInclude(loc, p[3])
+    if 4 == len(p):
+        id = p[3]
+        type = 'protocol'
+    else:
+        id = p[2]
+        type = 'header'
+    inc = Include(loc, type, id)
 
     path = Parser.current.resolveIncludePath(inc.file)
     if path is None:
-        raise ParseError(loc, "can't locate protocol include file `%s'"% (
+        raise ParseError(loc, "can't locate include file `%s'"% (
                 inc.file))
     
-    inc.tu = Parser().parse(open(path).read(), path, Parser.current.includedirs, Parser.current.errout)
+    inc.tu = Parser(type, id).parse(open(path).read(), path, Parser.current.includedirs, Parser.current.errout)
     p[0] = inc
 
 def p_UsingStmt(p):
@@ -315,6 +340,10 @@ def p_ProtocolDefn(p):
     protocol.sendSemantics = p[1]
     p[0] = protocol
 
+    if Parser.current.type == 'header':
+        _error(protocol.loc, 'can\'t define a protocol in a header.  Do it in a protocol spec instead.')
+
+
 def p_ProtocolBody(p):
     """ProtocolBody : SpawnsStmtsOpt"""
     p[0] = p[1]
@@ -459,13 +488,15 @@ def p_MessageDecl(p):
     p[0] = msg
 
 def p_MessageBody(p):
-    """MessageBody : MessageId MessageInParams MessageOutParams"""
+    """MessageBody : MessageId MessageInParams MessageOutParams OptionalMessageCompress"""
     # FIXME/cjones: need better loc info: use one of the quals
     loc, name = p[1]
     msg = MessageDecl(loc)
     msg.name = name
     msg.addInParams(p[2])
     msg.addOutParams(p[3])
+    msg.compress = p[4]
+
     p[0] = msg
 
 def p_MessageId(p):
@@ -492,6 +523,14 @@ def p_MessageOutParams(p):
     else:
         p[0] = p[3]
 
+def p_OptionalMessageCompress(p):
+    """OptionalMessageCompress : COMPRESS
+                               | """
+    if 1 == len(p):
+        p[0] = ''
+    else:
+        p[0] = 'compress'
+
 ##--------------------
 ## State machine
 
@@ -568,12 +607,16 @@ def p_OptionalSendSemanticsQual(p):
 
 def p_SendSemanticsQual(p):
     """SendSemanticsQual : ASYNC
+                         | INTR
                          | RPC
+                         | URGENT
                          | SYNC"""
     s = p[1]
-    if 'async' == s: p[0] = ASYNC
-    elif 'rpc' == s: p[0] = RPC
-    elif 'sync'== s: p[0] = SYNC
+    if 'async' == s: p[0] =    ASYNC
+    elif 'intr' == s: p[0] =   INTR
+    elif 'sync' == s: p[0] =   SYNC
+    elif 'urgent' == s: p[0] = URGENT
+    elif 'rpc' == s: p[0] =    RPC
     else:
         assert 0
 
