@@ -1,4 +1,5 @@
-/* This Source Code Form is subject to the terms of the Mozilla Public
+/* vim: set ts=8 sts=4 et sw=4 tw=80:
+ * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
@@ -14,8 +15,7 @@
 #include "base/basictypes.h"
 
 #include "jsapi.h"
-#include "jsdbgapi.h"
-#include "jsprf.h"
+#include "js/OldDebugAPI.h"
 
 #include "xpcpublic.h"
 
@@ -33,7 +33,7 @@
 #include "nsIXPConnect.h"
 #include "nsIXPCScriptable.h"
 
-#include "nsContentUtils.h"
+#include "nsCxPusher.h"
 #include "nsJSUtils.h"
 #include "nsJSPrincipals.h"
 #include "nsThreadUtils.h"
@@ -44,35 +44,16 @@
 #include "TestShellChild.h"
 #include "TestShellParent.h"
 
-#define EXITCODE_RUNTIME_ERROR 3
-#define EXITCODE_FILE_NOT_FOUND 4
-
 using mozilla::ipc::XPCShellEnvironment;
 using mozilla::ipc::TestShellChild;
 using mozilla::ipc::TestShellParent;
+using mozilla::AutoSafeJSContext;
+using namespace JS;
 
 namespace {
 
 static const char kDefaultRuntimeScriptFilename[] = "xpcshell.js";
 
-class FullTrustSecMan : public nsIScriptSecurityManager
-{
-public:
-    NS_DECL_ISUPPORTS
-    NS_DECL_NSIXPCSECURITYMANAGER
-    NS_DECL_NSISCRIPTSECURITYMANAGER
-
-    FullTrustSecMan() { }
-    virtual ~FullTrustSecMan() { }
-
-    void SetSystemPrincipal(nsIPrincipal *aPrincipal) {
-        mSystemPrincipal = aPrincipal;
-    }
-
-private:
-    nsCOMPtr<nsIPrincipal> mSystemPrincipal;
-};
-
 class XPCShellDirProvider : public nsIDirectoryServiceProvider
 {
 public:
@@ -90,127 +71,20 @@ private:
 };
 
 inline XPCShellEnvironment*
-Environment(JSContext* cx)
+Environment(Handle<JSObject*> global)
 {
-    XPCShellEnvironment* env = 
-        static_cast<XPCShellEnvironment*>(JS_GetContextPrivate(cx));
-    NS_ASSERTION(env, "Should never be null!");
-    return env;
+    AutoSafeJSContext cx;
+    JSAutoCompartment ac(cx, global);
+    Rooted<Value> v(cx);
+    if (!JS_GetProperty(cx, global, "__XPCShellEnvironment", &v) ||
+        !v.get().isDouble())
+    {
+        return nullptr;
+    }
+    return static_cast<XPCShellEnvironment*>(v.get().toPrivate());
 }
 
-static void
-ScriptErrorReporter(JSContext *cx,
-                    const char *message,
-                    JSErrorReport *report)
-{
-    int i, j, k, n;
-    char *prefix = NULL, *tmp;
-    const char *ctmp;
-    nsCOMPtr<nsIXPConnect> xpc;
-
-    // Don't report an exception from inner JS frames as the callers may intend
-    // to handle it.
-    if (JS_DescribeScriptedCaller(cx, nullptr, nullptr)) {
-        return;
-    }
-
-    // In some cases cx->fp is null here so use XPConnect to tell us about inner
-    // frames.
-    if ((xpc = do_GetService(nsIXPConnect::GetCID()))) {
-        nsAXPCNativeCallContext *cc = nullptr;
-        xpc->GetCurrentNativeCallContext(&cc);
-        if (cc) {
-            nsAXPCNativeCallContext *prev = cc;
-            while (NS_SUCCEEDED(prev->GetPreviousCallContext(&prev)) && prev) {
-                uint16_t lang;
-                if (NS_SUCCEEDED(prev->GetLanguage(&lang)) &&
-                    lang == nsAXPCNativeCallContext::LANG_JS) {
-                    return;
-                }
-            }
-        }
-    }
-
-    if (!report) {
-        fprintf(stderr, "%s\n", message);
-        return;
-    }
-
-    /* Conditionally ignore reported warnings. */
-    if (JSREPORT_IS_WARNING(report->flags) &&
-        !Environment(cx)->ShouldReportWarnings()) {
-        return;
-    }
-
-    if (report->filename)
-        prefix = JS_smprintf("%s:", report->filename);
-    if (report->lineno) {
-        tmp = prefix;
-        prefix = JS_smprintf("%s%u: ", tmp ? tmp : "", report->lineno);
-        JS_free(cx, tmp);
-    }
-    if (JSREPORT_IS_WARNING(report->flags)) {
-        tmp = prefix;
-        prefix = JS_smprintf("%s%swarning: ",
-                             tmp ? tmp : "",
-                             JSREPORT_IS_STRICT(report->flags) ? "strict " : "");
-        JS_free(cx, tmp);
-    }
-
-    /* embedded newlines -- argh! */
-    while ((ctmp = strchr(message, '\n')) != 0) {
-        ctmp++;
-        if (prefix) fputs(prefix, stderr);
-        fwrite(message, 1, ctmp - message, stderr);
-        message = ctmp;
-    }
-    /* If there were no filename or lineno, the prefix might be empty */
-    if (prefix)
-        fputs(prefix, stderr);
-    fputs(message, stderr);
-
-    if (!report->linebuf) {
-        fputc('\n', stderr);
-        goto out;
-    }
-
-    fprintf(stderr, ":\n%s%s\n%s", prefix, report->linebuf, prefix);
-    n = report->tokenptr - report->linebuf;
-    for (i = j = 0; i < n; i++) {
-        if (report->linebuf[i] == '\t') {
-            for (k = (j + 8) & ~7; j < k; j++) {
-                fputc('.', stderr);
-            }
-            continue;
-        }
-        fputc('.', stderr);
-        j++;
-    }
-    fputs("^\n", stderr);
- out:
-    if (!JSREPORT_IS_WARNING(report->flags)) {
-        Environment(cx)->SetExitCode(EXITCODE_RUNTIME_ERROR);
-    }
-    JS_free(cx, prefix);
-}
-
-JSContextCallback gOldContextCallback = NULL;
-
-static JSBool
-ContextCallback(JSContext *cx,
-                unsigned contextOp)
-{
-    if (gOldContextCallback && !gOldContextCallback(cx, contextOp))
-        return JS_FALSE;
-
-    if (contextOp == JSCONTEXT_NEW) {
-        JS_SetErrorReporter(cx, ScriptErrorReporter);
-        JS_SetVersion(cx, JSVERSION_LATEST);
-    }
-    return JS_TRUE;
-}
-
-static JSBool
+static bool
 Print(JSContext *cx,
       unsigned argc,
       JS::Value *vp)
@@ -222,10 +96,10 @@ Print(JSContext *cx,
     for (i = n = 0; i < argc; i++) {
         str = JS_ValueToString(cx, argv[i]);
         if (!str)
-            return JS_FALSE;
+            return false;
         JSAutoByteString bytes(cx, str);
         if (!bytes)
-            return JS_FALSE;
+            return false;
         fprintf(stdout, "%s%s", i ? " " : "", bytes.ptr());
         fflush(stdout);
     }
@@ -233,10 +107,10 @@ Print(JSContext *cx,
     if (n)
         fputc('\n', stdout);
     JS_SET_RVAL(cx, vp, JSVAL_VOID);
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
+static bool
 GetLine(char *bufp,
         FILE *file,
         const char *prompt)
@@ -245,12 +119,12 @@ GetLine(char *bufp,
     fputs(prompt, stdout);
     fflush(stdout);
     if (!fgets(line, sizeof line, file))
-        return JS_FALSE;
+        return false;
     strcpy(bufp, line);
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
+static bool
 Dump(JSContext *cx,
      unsigned argc,
      JS::Value *vp)
@@ -259,21 +133,21 @@ Dump(JSContext *cx,
 
     JSString *str;
     if (!argc)
-        return JS_TRUE;
+        return true;
 
     str = JS_ValueToString(cx, JS_ARGV(cx, vp)[0]);
     if (!str)
-        return JS_FALSE;
+        return false;
     JSAutoByteString bytes(cx, str);
     if (!bytes)
-      return JS_FALSE;
+      return false;
 
     fputs(bytes.ptr(), stdout);
     fflush(stdout);
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
+static bool
 Load(JSContext *cx,
      unsigned argc,
      JS::Value *vp)
@@ -282,77 +156,74 @@ Load(JSContext *cx,
 
     JS::Rooted<JSObject*> obj(cx, JS_THIS_OBJECT(cx, vp));
     if (!obj)
-        return JS_FALSE;
+        return false;
 
     JS::Value *argv = JS_ARGV(cx, vp);
     for (unsigned i = 0; i < argc; i++) {
         JSString *str = JS_ValueToString(cx, argv[i]);
         if (!str)
-            return JS_FALSE;
+            return false;
         argv[i] = STRING_TO_JSVAL(str);
         JSAutoByteString filename(cx, str);
         if (!filename)
-            return JS_FALSE;
+            return false;
         FILE *file = fopen(filename.ptr(), "r");
         if (!file) {
             JS_ReportError(cx, "cannot open file '%s' for reading", filename.ptr());
-            return JS_FALSE;
+            return false;
         }
+        Rooted<JSObject*> global(cx, JS::CurrentGlobalOrNull(cx));
         JS::CompileOptions options(cx);
         options.setUTF8(true)
                .setFileAndLine(filename.ptr(), 1)
-               .setPrincipals(Environment(cx)->GetPrincipal());
+               .setPrincipals(Environment(global)->GetPrincipal());
         JS::RootedObject rootedObj(cx, obj);
         JSScript *script = JS::Compile(cx, rootedObj, options, file);
         fclose(file);
         if (!script)
-            return JS_FALSE;
+            return false;
 
-        if (!Environment(cx)->ShouldCompileOnly() &&
-            !JS_ExecuteScript(cx, obj, script, result.address())) {
-            return JS_FALSE;
+        if (!JS_ExecuteScript(cx, obj, script, result.address())) {
+            return false;
         }
     }
     JS_SET_RVAL(cx, vp, JSVAL_VOID);
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
+static bool
 Version(JSContext *cx,
         unsigned argc,
         JS::Value *vp)
 {
     JS::Value *argv = JS_ARGV(cx, vp);
+    JS_SET_RVAL(cx, vp, INT_TO_JSVAL(JS_GetVersion(cx)));
     if (argc > 0 && JSVAL_IS_INT(argv[0]))
-        JS_SET_RVAL(cx, vp, INT_TO_JSVAL(JS_SetVersion(cx, JSVersion(JSVAL_TO_INT(argv[0])))));
-    else
-        JS_SET_RVAL(cx, vp, INT_TO_JSVAL(JS_GetVersion(cx)));
-    return JS_TRUE;
+        JS_SetVersionForCompartment(js::GetContextCompartment(cx),
+                                    JSVersion(JSVAL_TO_INT(argv[0])));
+    return true;
 }
 
-static JSBool
+static bool
 BuildDate(JSContext *cx, unsigned argc, JS::Value *vp)
 {
     fprintf(stdout, "built on %s at %s\n", __DATE__, __TIME__);
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
+static bool
 Quit(JSContext *cx,
      unsigned argc,
      JS::Value *vp)
 {
-    int exitCode = 0;
-    JS_ConvertArguments(cx, argc, JS_ARGV(cx, vp), "/ i", &exitCode);
-
-    XPCShellEnvironment* env = Environment(cx);
-    env->SetExitCode(exitCode);
+    Rooted<JSObject*> global(cx, JS::CurrentGlobalOrNull(cx));
+    XPCShellEnvironment* env = Environment(global);
     env->SetIsQuitting();
 
-    return JS_FALSE;
+    return false;
 }
 
-static JSBool
+static bool
 DumpXPC(JSContext *cx,
         unsigned argc,
         JS::Value *vp)
@@ -361,17 +232,17 @@ DumpXPC(JSContext *cx,
 
     if (argc > 0) {
         if (!JS_ValueToInt32(cx, JS_ARGV(cx, vp)[0], &depth))
-            return JS_FALSE;
+            return false;
     }
 
     nsCOMPtr<nsIXPConnect> xpc = do_GetService(nsIXPConnect::GetCID());
     if(xpc)
         xpc->DebugDump(int16_t(depth));
     JS_SET_RVAL(cx, vp, JSVAL_VOID);
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
+static bool
 GC(JSContext *cx,
    unsigned argc,
    JS::Value *vp)
@@ -382,11 +253,11 @@ GC(JSContext *cx,
     js_DumpGCStats(rt, stdout);
 #endif
     JS_SET_RVAL(cx, vp, JSVAL_VOID);
-    return JS_TRUE;
+    return true;
 }
 
 #ifdef JS_GC_ZEAL
-static JSBool
+static bool
 GCZeal(JSContext *cx, 
        unsigned argc,
        JS::Value *vp)
@@ -395,28 +266,28 @@ GCZeal(JSContext *cx,
 
   uint32_t zeal;
   if (!JS_ValueToECMAUint32(cx, argv[0], &zeal))
-    return JS_FALSE;
+    return false;
 
   JS_SetGCZeal(cx, uint8_t(zeal), JS_DEFAULT_ZEAL_FREQ);
-  return JS_TRUE;
+  return true;
 }
 #endif
 
 #ifdef DEBUG
 
-static JSBool
+static bool
 DumpHeap(JSContext *cx,
          unsigned argc,
          JS::Value *vp)
 {
     JSAutoByteString fileName;
-    void* startThing = NULL;
+    void* startThing = nullptr;
     JSGCTraceKind startTraceKind = JSTRACE_OBJECT;
-    void *thingToFind = NULL;
+    void *thingToFind = nullptr;
     size_t maxDepth = (size_t)-1;
-    void *thingToIgnore = NULL;
+    void *thingToIgnore = nullptr;
     FILE *dumpFile;
-    JSBool ok;
+    bool ok;
 
     JS::Value *argv = JS_ARGV(cx, vp);
     JS_SET_RVAL(cx, vp, JSVAL_VOID);
@@ -427,10 +298,10 @@ DumpHeap(JSContext *cx,
 
         str = JS_ValueToString(cx, *vp);
         if (!str)
-            return JS_FALSE;
+            return false;
         *vp = STRING_TO_JSVAL(str);
         if (!fileName.encodeLatin1(cx, str))
-            return JS_FALSE;
+            return false;
     }
 
     vp = argv + 1;
@@ -453,7 +324,7 @@ DumpHeap(JSContext *cx,
         uint32_t depth;
 
         if (!JS_ValueToECMAUint32(cx, *vp, &depth))
-            return JS_FALSE;
+            return false;
         maxDepth = depth;
     }
 
@@ -471,7 +342,7 @@ DumpHeap(JSContext *cx,
         if (!dumpFile) {
             fprintf(stderr, "dumpHeap: can't open %s: %s\n",
                     fileName.ptr(), strerror(errno));
-            return JS_FALSE;
+            return false;
         }
     }
 
@@ -487,7 +358,7 @@ DumpHeap(JSContext *cx,
     fprintf(stderr,
             "dumpHeap: argument %u is not null or a heap-allocated thing\n",
             (unsigned)(vp - argv));
-    return JS_FALSE;
+    return false;
 }
 
 #endif /* DEBUG */
@@ -521,21 +392,21 @@ typedef enum JSShellErrNum
 #undef MSGDEF
 } JSShellErrNum;
 
-static void
-ProcessFile(JSContext *cx,
-            JS::Handle<JSObject*> obj,
-            const char *filename,
-            FILE *file,
-            JSBool forceTTY)
+} /* anonymous namespace */
+
+void
+XPCShellEnvironment::ProcessFile(JSContext *cx,
+                                 JS::Handle<JSObject*> obj,
+                                 const char *filename,
+                                 FILE *file,
+                                 bool forceTTY)
 {
-    XPCShellEnvironment* env = Environment(cx);
-    nsCxPusher pusher;
-    pusher.Push(env->GetContext());
+    XPCShellEnvironment* env = this;
 
     JSScript *script;
     JS::Rooted<JS::Value> result(cx);
     int lineno, startline;
-    JSBool ok, hitEOF;
+    bool ok, hitEOF;
     char *bufp, buffer[4096];
     JSString *str;
 
@@ -572,7 +443,7 @@ ProcessFile(JSContext *cx,
                .setFileAndLine(filename, 1)
                .setPrincipals(env->GetPrincipal());
         JSScript* script = JS::Compile(cx, obj, options, file);
-        if (script && !env->ShouldCompileOnly())
+        if (script)
             (void)JS_ExecuteScript(cx, obj, script, result.address());
 
         return;
@@ -580,7 +451,7 @@ ProcessFile(JSContext *cx,
 
     /* It's an interactive filehandle; drop into read-eval-print loop. */
     lineno = 1;
-    hitEOF = JS_FALSE;
+    hitEOF = false;
     do {
         bufp = buffer;
         *bufp = '\0';
@@ -597,7 +468,7 @@ ProcessFile(JSContext *cx,
         startline = lineno;
         do {
             if (!GetLine(bufp, file, startline == lineno ? "js> " : "")) {
-                hitEOF = JS_TRUE;
+                hitEOF = true;
                 break;
             }
             bufp += strlen(bufp);
@@ -612,22 +483,20 @@ ProcessFile(JSContext *cx,
         if (script) {
             JSErrorReporter older;
 
-            if (!env->ShouldCompileOnly()) {
-                ok = JS_ExecuteScript(cx, obj, script, result.address());
-                if (ok && result != JSVAL_VOID) {
-                    /* Suppress error reports from JS_ValueToString(). */
-                    older = JS_SetErrorReporter(cx, NULL);
-                    str = JS_ValueToString(cx, result);
-                    JSAutoByteString bytes;
-                    if (str)
-                        bytes.encodeLatin1(cx, str);
-                    JS_SetErrorReporter(cx, older);
+            ok = JS_ExecuteScript(cx, obj, script, result.address());
+            if (ok && result != JSVAL_VOID) {
+                /* Suppress error reports from JS_ValueToString(). */
+                older = JS_SetErrorReporter(cx, nullptr);
+                str = JS_ValueToString(cx, result);
+                JSAutoByteString bytes;
+                if (str)
+                    bytes.encodeLatin1(cx, str);
+                JS_SetErrorReporter(cx, older);
 
-                    if (!!bytes)
-                        fprintf(stdout, "%s\n", bytes.ptr());
-                    else
-                        ok = JS_FALSE;
-                }
+                if (!!bytes)
+                    fprintf(stdout, "%s\n", bytes.ptr());
+                else
+                    ok = false;
             }
         }
     } while (!hitEOF && !env->IsQuitting());
@@ -635,218 +504,6 @@ ProcessFile(JSContext *cx,
     fprintf(stdout, "\n");
 }
 
-} /* anonymous namespace */
-
-NS_INTERFACE_MAP_BEGIN(FullTrustSecMan)
-    NS_INTERFACE_MAP_ENTRY(nsIXPCSecurityManager)
-    NS_INTERFACE_MAP_ENTRY(nsIScriptSecurityManager)
-    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIXPCSecurityManager)
-NS_INTERFACE_MAP_END
-
-NS_IMPL_ADDREF(FullTrustSecMan)
-NS_IMPL_RELEASE(FullTrustSecMan)
-
-NS_IMETHODIMP
-FullTrustSecMan::CanCreateWrapper(JSContext * aJSContext,
-                                  const nsIID & aIID,
-                                  nsISupports *aObj,
-                                  nsIClassInfo *aClassInfo,
-                                  void * *aPolicy)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CanCreateInstance(JSContext * aJSContext,
-                                   const nsCID & aCID)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CanGetService(JSContext * aJSContext,
-                               const nsCID & aCID)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CanAccess(uint32_t aAction,
-                           nsAXPCNativeCallContext *aCallContext,
-                           JSContext * aJSContext,
-                           JSObject * aJSObject,
-                           nsISupports *aObj,
-                           nsIClassInfo *aClassInfo,
-                           jsid aName,
-                           void * *aPolicy)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CheckPropertyAccess(JSContext * aJSContext,
-                                     JSObject * aJSObject,
-                                     const char *aClassName,
-                                     jsid aProperty,
-                                     uint32_t aAction)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CheckLoadURIFromScript(JSContext * cx,
-                                        nsIURI *uri)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CheckLoadURIWithPrincipal(nsIPrincipal *aPrincipal,
-                                           nsIURI *uri,
-                                           uint32_t flags)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CheckLoadURIStrWithPrincipal(nsIPrincipal *aPrincipal,
-                                              const nsACString & uri,
-                                              uint32_t flags)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CheckFunctionAccess(JSContext * cx,
-                                     void * funObj,
-                                     void * targetObj)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CanExecuteScripts(JSContext * cx,
-                                   nsIPrincipal *principal,
-                                   bool *_retval)
-{
-    *_retval = true;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetSubjectPrincipal(nsIPrincipal **_retval)
-{
-    NS_IF_ADDREF(*_retval = mSystemPrincipal);
-    return *_retval ? NS_OK : NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetSystemPrincipal(nsIPrincipal **_retval)
-{
-    NS_IF_ADDREF(*_retval = mSystemPrincipal);
-    return *_retval ? NS_OK : NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetSimpleCodebasePrincipal(nsIURI *aURI,
-                                            nsIPrincipal **_retval)
-{
-    NS_IF_ADDREF(*_retval = mSystemPrincipal);
-    return *_retval ? NS_OK : NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetNoAppCodebasePrincipal(nsIURI *aURI,
-                                           nsIPrincipal **_retval)
-{
-    return GetSimpleCodebasePrincipal(aURI, _retval);
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetCodebasePrincipal(nsIURI *aURI, nsIPrincipal **_retval)
-{
-    return GetSimpleCodebasePrincipal(aURI, _retval);
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetAppCodebasePrincipal(nsIURI *aURI,
-                                         uint32_t aAppId,
-                                         bool aInMozBrowser,
-                                         nsIPrincipal **_retval)
-{
-    return GetSimpleCodebasePrincipal(aURI, _retval);
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetDocShellCodebasePrincipal(nsIURI *aURI,
-                                              nsIDocShell* aDocShell,
-                                              nsIPrincipal **_retval)
-{
-    return GetSimpleCodebasePrincipal(aURI, _retval);
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetObjectPrincipal(JSContext * cx,
-                                    JSObject * obj,
-                                    nsIPrincipal **_retval)
-{
-    NS_IF_ADDREF(*_retval = mSystemPrincipal);
-    return *_retval ? NS_OK : NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::SubjectPrincipalIsSystem(bool *_retval)
-{
-    *_retval = true;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CheckSameOrigin(JSContext * aJSContext,
-                                 nsIURI *aTargetURI)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::CheckSameOriginURI(nsIURI *aSourceURI,
-                                    nsIURI *aTargetURI,
-                                    bool reportError)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetChannelPrincipal(nsIChannel *aChannel,
-                                     nsIPrincipal **_retval)
-{
-    NS_IF_ADDREF(*_retval = mSystemPrincipal);
-    return *_retval ? NS_OK : NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::IsSystemPrincipal(nsIPrincipal *aPrincipal,
-                                   bool *_retval)
-{
-    *_retval = aPrincipal == mSystemPrincipal;
-    return NS_OK;
-}
-
-NS_IMETHODIMP_(nsIPrincipal *)
-FullTrustSecMan::GetCxSubjectPrincipal(JSContext *cx)
-{
-    return mSystemPrincipal;
-}
-
-NS_IMETHODIMP
-FullTrustSecMan::GetExtendedOrigin(nsIURI* aURI, uint32_t aAppId,
-                                   bool aInMozBrowser,
-                                   nsACString& aExtendedOrigin)
-{
-  aExtendedOrigin.Truncate();
-  return NS_OK;
-}
-
 NS_IMETHODIMP_(nsrefcnt)
 XPCShellDirProvider::AddRef()
 {
@@ -895,41 +552,24 @@ XPCShellEnvironment::CreateEnvironment()
 }
 
 XPCShellEnvironment::XPCShellEnvironment()
-:   mCx(NULL),
-    mJSPrincipals(NULL),
-    mExitCode(0),
-    mQuitting(JS_FALSE),
-    mReportWarnings(JS_TRUE),
-    mCompileOnly(JS_FALSE)
+:   mQuitting(false)
 {
 }
 
 XPCShellEnvironment::~XPCShellEnvironment()
 {
-    if (mCx) {
-        JS_BeginRequest(mCx);
 
-        JSObject* global = GetGlobalObject();
-        if (global) {
-            JS_SetAllNonReservedSlotsToUndefined(mCx, global);
+    AutoSafeJSContext cx;
+    Rooted<JSObject*> global(cx, GetGlobalObject());
+    if (global) {
+        {
+            JSAutoCompartment ac(cx, global);
+            JS_SetAllNonReservedSlotsToUndefined(cx, global);
         }
         mGlobalHolder.Release();
 
-        JSRuntime *rt = JS_GetRuntime(mCx);
+        JSRuntime *rt = JS_GetRuntime(cx);
         JS_GC(rt);
-
-        if (mJSPrincipals) {
-            JS_DropPrincipals(rt, mJSPrincipals);
-        }
-
-        JS_EndRequest(mCx);
-        JS_DestroyContext(mCx);
-
-        if (gOldContextCallback) {
-            NS_ASSERTION(rt, "Should never be null!");
-            JS_SetContextCallback(rt, gOldContextCallback);
-            gOldContextCallback = NULL;
-        }
     }
 }
 
@@ -962,18 +602,7 @@ XPCShellEnvironment::Init()
         return false;
     }
 
-    gOldContextCallback = JS_SetContextCallback(rt, ContextCallback);
-
-    JSContext *cx = JS_NewContext(rt, 8192);
-    if (!cx) {
-        NS_ERROR("JS_NewContext failed!");
-
-        JS_SetContextCallback(rt, gOldContextCallback);
-        gOldContextCallback = NULL;
-
-        return false;
-    }
-    mCx = cx;
+    AutoSafeJSContext cx;
 
     JS_SetContextPrivate(cx, this);
 
@@ -984,30 +613,18 @@ XPCShellEnvironment::Init()
         return false;
     }
 
-    nsRefPtr<FullTrustSecMan> secman(new FullTrustSecMan());
-    xpc->SetSecurityManagerForJSContext(cx, secman, 0xFFFF);
-
     nsCOMPtr<nsIPrincipal> principal;
-
     nsCOMPtr<nsIScriptSecurityManager> securityManager =
         do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv) && securityManager) {
         rv = securityManager->GetSystemPrincipal(getter_AddRefs(principal));
         if (NS_FAILED(rv)) {
             fprintf(stderr, "+++ Failed to obtain SystemPrincipal from ScriptSecurityManager service.\n");
-        } else {
-            // fetch the JS principals and stick in a global
-            mJSPrincipals = nsJSPrincipals::get(principal);
-            JS_HoldPrincipals(mJSPrincipals);
-            secman->SetSystemPrincipal(principal);
         }
     } else {
         fprintf(stderr, "+++ Failed to get ScriptSecurityManager service, running without principals");
     }
 
-    nsCxPusher pusher;
-    pusher.Push(mCx);
-
     nsRefPtr<BackstagePass> backstagePass;
     rv = NS_NewBackstagePass(getter_AddRefs(backstagePass));
     if (NS_FAILED(rv)) {
@@ -1015,35 +632,38 @@ XPCShellEnvironment::Init()
         return false;
     }
 
+    JS::CompartmentOptions options;
+    options.setZone(JS::SystemZone)
+           .setVersion(JSVERSION_LATEST);
     nsCOMPtr<nsIXPConnectJSObjectHolder> holder;
     rv = xpc->InitClassesWithNewWrappedGlobal(cx,
                                               static_cast<nsIGlobalObject *>(backstagePass),
                                               principal, 0,
-                                              JS::SystemZone,
+                                              options,
                                               getter_AddRefs(holder));
     if (NS_FAILED(rv)) {
         NS_ERROR("InitClassesWithNewWrappedGlobal failed!");
         return false;
     }
 
-    JS::Rooted<JSObject*> globalObj(cx);
-    rv = holder->GetJSObject(globalObj.address());
-    if (NS_FAILED(rv)) {
+    JS::Rooted<JSObject*> globalObj(cx, holder->GetJSObject());
+    if (!globalObj) {
         NS_ERROR("Failed to get global JSObject!");
         return false;
     }
+    JSAutoCompartment ac(cx, globalObj);
 
     backstagePass->SetGlobalObject(globalObj);
 
+    if (!JS_DefineProperty(cx, globalObj, "__XPCShellEnvironment",
+                           PRIVATE_TO_JSVAL(this), JS_PropertyStub,
+                           JS_StrictPropertyStub,
+                           JSPROP_READONLY | JSPROP_PERMANENT) ||
+        !JS_DefineFunctions(cx, globalObj, gGlobalFunctions) ||
+        !JS_DefineProfilingFunctions(cx, globalObj))
     {
-        JSAutoRequest ar(cx);
-        JSAutoCompartment ac(cx, globalObj);
-
-        if (!JS_DefineFunctions(cx, globalObj, gGlobalFunctions) ||
-	    !JS_DefineProfilingFunctions(cx, globalObj)) {
-            NS_ERROR("JS_DefineFunctions failed!");
-            return false;
-        }
+        NS_ERROR("JS_DefineFunctions failed!");
+        return false;
     }
 
     mGlobalHolder = globalObj;
@@ -1052,7 +672,7 @@ XPCShellEnvironment::Init()
     if (runtimeScriptFile) {
         fprintf(stdout, "[loading '%s'...]\n", kDefaultRuntimeScriptFilename);
         ProcessFile(cx, globalObj, kDefaultRuntimeScriptFilename,
-                    runtimeScriptFile, JS_FALSE);
+                    runtimeScriptFile, false);
         fclose(runtimeScriptFile);
     }
 
@@ -1063,43 +683,34 @@ bool
 XPCShellEnvironment::EvaluateString(const nsString& aString,
                                     nsString* aResult)
 {
-  XPCShellEnvironment* env = Environment(mCx);
-  nsCxPusher pusher;
-  pusher.Push(env->GetContext());
-
-  JSAutoRequest ar(mCx);
-
-  JS_ClearPendingException(mCx);
-
-  JS::Rooted<JSObject*> global(mCx, GetGlobalObject());
-  JSAutoCompartment ac(mCx, global);
+  AutoSafeJSContext cx;
+  JS::Rooted<JSObject*> global(cx, GetGlobalObject());
+  JSAutoCompartment ac(cx, global);
 
   JSScript* script =
-      JS_CompileUCScriptForPrincipals(mCx, global, GetPrincipal(),
+      JS_CompileUCScriptForPrincipals(cx, global, GetPrincipal(),
                                       aString.get(), aString.Length(),
                                       "typein", 0);
   if (!script) {
      return false;
   }
 
-  if (!ShouldCompileOnly()) {
-      if (aResult) {
-          aResult->Truncate();
-      }
+  if (aResult) {
+      aResult->Truncate();
+  }
 
-      JS::Rooted<JS::Value> result(mCx);
-      JSBool ok = JS_ExecuteScript(mCx, global, script, result.address());
-      if (ok && result != JSVAL_VOID) {
-          JSErrorReporter old = JS_SetErrorReporter(mCx, NULL);
-          JSString* str = JS_ValueToString(mCx, result);
-          nsDependentJSString depStr;
-          if (str)
-              depStr.init(mCx, str);
-          JS_SetErrorReporter(mCx, old);
+  JS::Rooted<JS::Value> result(cx);
+  bool ok = JS_ExecuteScript(cx, global, script, result.address());
+  if (ok && result != JSVAL_VOID) {
+      JSErrorReporter old = JS_SetErrorReporter(cx, nullptr);
+      JSString* str = JS_ValueToString(cx, result);
+      nsDependentJSString depStr;
+      if (str)
+          depStr.init(cx, str);
+      JS_SetErrorReporter(cx, old);
 
-          if (!depStr.IsEmpty() && aResult) {
-              aResult->Assign(depStr);
-          }
+      if (!depStr.IsEmpty() && aResult) {
+          aResult->Assign(depStr);
       }
   }
 
