@@ -57,61 +57,61 @@ void JSD_ASSERT_VALID_PROPERTY(JSDProperty* jsdprop)
 #endif
 
 
-JSBool
+bool
 jsd_IsValueObject(JSDContext* jsdc, JSDValue* jsdval)
 {
     return !JSVAL_IS_PRIMITIVE(jsdval->val) || JSVAL_IS_NULL(jsdval->val);
 }
 
-JSBool
+bool
 jsd_IsValueNumber(JSDContext* jsdc, JSDValue* jsdval)
 {
     return JSVAL_IS_NUMBER(jsdval->val);
 }
 
-JSBool
+bool
 jsd_IsValueInt(JSDContext* jsdc, JSDValue* jsdval)
 {
     return JSVAL_IS_INT(jsdval->val);
 }
 
-JSBool
+bool
 jsd_IsValueDouble(JSDContext* jsdc, JSDValue* jsdval)
 {
     return JSVAL_IS_DOUBLE(jsdval->val);
 }
 
-JSBool
+bool
 jsd_IsValueString(JSDContext* jsdc, JSDValue* jsdval)
 {
     return JSVAL_IS_STRING(jsdval->val);
 }
 
-JSBool
+bool
 jsd_IsValueBoolean(JSDContext* jsdc, JSDValue* jsdval)
 {
     return JSVAL_IS_BOOLEAN(jsdval->val);
 }
 
-JSBool
+bool
 jsd_IsValueNull(JSDContext* jsdc, JSDValue* jsdval)
 {
     return JSVAL_IS_NULL(jsdval->val);
 }
 
-JSBool
+bool
 jsd_IsValueVoid(JSDContext* jsdc, JSDValue* jsdval)
 {
     return JSVAL_IS_VOID(jsdval->val);
 }
 
-JSBool
+bool
 jsd_IsValuePrimitive(JSDContext* jsdc, JSDValue* jsdval)
 {
     return JSVAL_IS_PRIMITIVE(jsdval->val);
 }
 
-JSBool
+bool
 jsd_IsValueFunction(JSDContext* jsdc, JSDValue* jsdval)
 {
     AutoSafeJSContext cx; // NB: Actually unused.
@@ -119,7 +119,7 @@ jsd_IsValueFunction(JSDContext* jsdc, JSDValue* jsdval)
            JS_ObjectIsCallable(cx, JSVAL_TO_OBJECT(jsdval->val));
 }
 
-JSBool
+bool
 jsd_IsValueNative(JSDContext* jsdc, JSDValue* jsdval)
 {
     AutoSafeJSContext cx;
@@ -129,10 +129,10 @@ jsd_IsValueNative(JSDContext* jsdc, JSDValue* jsdval)
     {
         JSAutoCompartment ac(cx, JSVAL_TO_OBJECT(jsdval->val));
         AutoSaveExceptionState as(cx);
-        JSBool ok = JS_FALSE;
+        bool ok = false;
         fun = JSD_GetValueFunction(jsdc, jsdval);
         if(fun)
-            ok = JS_GetFunctionScript(cx, fun) ? JS_FALSE : JS_TRUE;
+            ok = JS_GetFunctionScript(cx, fun) ? false : true;
         JS_ASSERT(fun);
         return ok;
     }
@@ -141,12 +141,12 @@ jsd_IsValueNative(JSDContext* jsdc, JSDValue* jsdval)
 
 /***************************************************************************/
 
-JSBool
+bool
 jsd_GetValueBoolean(JSDContext* jsdc, JSDValue* jsdval)
 {
     jsval val = jsdval->val;
     if(!JSVAL_IS_BOOLEAN(val))
-        return JS_FALSE;
+        return false;
     return JSVAL_TO_BOOLEAN(val);
 }
 
@@ -196,13 +196,13 @@ jsd_GetValueString(JSDContext* jsdc, JSDValue* jsdval)
     if(string) {
         stringval = STRING_TO_JSVAL(string);
     }
-    if(!string || !JS_WrapValue(cx, stringval.address())) {
-        return NULL;
+    if(!string || !JS_WrapValue(cx, &stringval)) {
+        return nullptr;
     }
 
     jsdval->string = JSVAL_TO_STRING(stringval);
     if(!JS_AddNamedStringRoot(cx, &jsdval->string, "ValueString"))
-        jsdval->string = NULL;
+        jsdval->string = nullptr;
 
     return jsdval->string;
 }
@@ -219,7 +219,7 @@ jsd_GetValueFunctionId(JSDContext* jsdc, JSDValue* jsdval)
         AutoSaveExceptionState as(cx);
         fun = JSD_GetValueFunction(jsdc, jsdval);
         if(!fun)
-            return NULL;
+            return nullptr;
         jsdval->funName = JS_GetFunctionId(fun);
 
         /* For compatibility we return "anonymous", not an empty string here. */
@@ -239,29 +239,29 @@ jsd_GetValueFunctionId(JSDContext* jsdc, JSDValue* jsdval)
 JSDValue*
 jsd_NewValue(JSDContext* jsdc, jsval value)
 {
+    JS::RootedValue val(jsdc->jsrt, value);
     AutoSafeJSContext cx;
-    JS::RootedValue val(cx, value);
     JSDValue* jsdval;
 
     if(!(jsdval = (JSDValue*) calloc(1, sizeof(JSDValue))))
-        return NULL;
+        return nullptr;
 
     if(JSVAL_IS_GCTHING(val))
     {
-        JSBool ok;
+        bool ok;
         JSAutoCompartment ac(cx, jsdc->glob);
 
         ok = JS_AddNamedValueRoot(cx, &jsdval->val, "JSDValue");
         if(ok && JSVAL_IS_STRING(val)) {
-            if(!JS_WrapValue(cx, val.address())) {
-                ok = JS_FALSE;
+            if(!JS_WrapValue(cx, &val)) {
+                ok = false;
             }
         }
 
         if(!ok)
         {
             free(jsdval);
-            return NULL;
+            return nullptr;
         }
     }
     jsdval->val  = val;
@@ -309,32 +309,33 @@ jsd_GetValueWrappedJSVal(JSDContext* jsdc, JSDValue* jsdval)
     return val;
 }
 
-static JSDProperty* _newProperty(JSDContext* jsdc, JSPropertyDesc* pd,
-                                 unsigned additionalFlags)
+static JSDProperty* _newProperty(JSDContext* jsdc, JS::HandleValue propId,
+                                 JS::HandleValue propValue, JS::HandleValue propAlias,
+                                 uint8_t propFlags, unsigned additionalFlags)
 {
     JSDProperty* jsdprop;
 
     if(!(jsdprop = (JSDProperty*) calloc(1, sizeof(JSDProperty))))
-        return NULL;
+        return nullptr;
 
     JS_INIT_CLIST(&jsdprop->links);
     jsdprop->nref = 1;
-    jsdprop->flags = pd->flags | additionalFlags;
+    jsdprop->flags = propFlags | additionalFlags;
 
-    if(!(jsdprop->name = jsd_NewValue(jsdc, pd->id)))
+    if(!(jsdprop->name = jsd_NewValue(jsdc, propId)))
         goto new_prop_fail;
 
-    if(!(jsdprop->val = jsd_NewValue(jsdc, pd->value)))
+    if(!(jsdprop->val = jsd_NewValue(jsdc, propValue)))
         goto new_prop_fail;
 
     if((jsdprop->flags & JSDPD_ALIAS) &&
-       !(jsdprop->alias = jsd_NewValue(jsdc, pd->alias)))
+       !(jsdprop->alias = jsd_NewValue(jsdc, propAlias)))
         goto new_prop_fail;
 
     return jsdprop;
 new_prop_fail:
     jsd_DropProperty(jsdc, jsdprop);
-    return NULL;
+    return nullptr;
 }
 
 static void _freeProps(JSDContext* jsdc, JSDValue* jsdval)
@@ -351,7 +352,7 @@ static void _freeProps(JSDContext* jsdc, JSDValue* jsdval)
     CLEAR_BIT_FLAG(jsdval->flags, GOT_PROPS);
 }
 
-static JSBool _buildProps(JSDContext* jsdc, JSDValue* jsdval)
+static bool _buildProps(JSDContext* jsdc, JSDValue* jsdval)
 {
     AutoSafeJSContext cx;
     JS::RootedObject obj(cx);
@@ -363,7 +364,7 @@ static JSBool _buildProps(JSDContext* jsdc, JSDValue* jsdval)
     JS_ASSERT(!JSVAL_IS_PRIMITIVE(jsdval->val));
 
     if(JSVAL_IS_PRIMITIVE(jsdval->val))
-        return JS_FALSE;
+        return false;
 
     obj = JSVAL_TO_OBJECT(jsdval->val);
 
@@ -371,12 +372,20 @@ static JSBool _buildProps(JSDContext* jsdc, JSDValue* jsdval)
 
     if(!JS_GetPropertyDescArray(cx, obj, &pda))
     {
-        return JS_FALSE;
+        return false;
     }
 
+    JS::RootedValue propId(cx);
+    JS::RootedValue propValue(cx);
+    JS::RootedValue propAlias(cx);
+    uint8_t propFlags;
     for(i = 0; i < pda.length; i++)
     {
-        JSDProperty* prop = _newProperty(jsdc, &pda.array[i], 0);
+        propId = pda.array[i].id;
+        propValue = pda.array[i].value;
+        propAlias = pda.array[i].alias;
+        propFlags = pda.array[i].flags;
+        JSDProperty* prop = _newProperty(jsdc, propId, propValue, propAlias, propFlags, 0);
         if(!prop)
         {
             _freeProps(jsdc, jsdval);
@@ -390,7 +399,7 @@ static JSBool _buildProps(JSDContext* jsdc, JSDValue* jsdval)
 }
 
 #undef  DROP_CLEAR_VALUE
-#define DROP_CLEAR_VALUE(jsdc, x) if(x){jsd_DropValue(jsdc,x); x = NULL;}
+#define DROP_CLEAR_VALUE(jsdc, x) if(x){jsd_DropValue(jsdc,x); x = nullptr;}
 
 void
 jsd_RefreshValue(JSDContext* jsdc, JSDValue* jsdval)
@@ -404,11 +413,11 @@ jsd_RefreshValue(JSDContext* jsdc, JSDValue* jsdval)
             JSAutoCompartment ac(cx, jsdc->glob);
             JS_RemoveStringRoot(cx, &jsdval->string);
         }
-        jsdval->string = NULL;
+        jsdval->string = nullptr;
     }
 
-    jsdval->funName = NULL;
-    jsdval->className = NULL;
+    jsdval->funName = nullptr;
+    jsdval->className = nullptr;
     DROP_CLEAR_VALUE(jsdc, jsdval->proto);
     DROP_CLEAR_VALUE(jsdc, jsdval->parent);
     DROP_CLEAR_VALUE(jsdc, jsdval->ctor);
@@ -445,13 +454,13 @@ jsd_IterateProperties(JSDContext* jsdc, JSDValue* jsdval, JSDProperty **iterp)
     {
         JS_ASSERT(!jsdprop);
         if(!_buildProps(jsdc, jsdval))
-            return NULL;
+            return nullptr;
     }
 
     if(!jsdprop)
         jsdprop = (JSDProperty*)jsdval->props.next;
     if(jsdprop == (JSDProperty*)&jsdval->props)
-        return NULL;
+        return nullptr;
     *iterp = (JSDProperty*)jsdprop->links.next;
 
     JS_ASSERT(jsdprop);
@@ -462,25 +471,24 @@ jsd_IterateProperties(JSDContext* jsdc, JSDValue* jsdval, JSDProperty **iterp)
 JSDProperty*
 jsd_GetValueProperty(JSDContext* jsdc, JSDValue* jsdval, JSString* nameStr)
 {
+    JS::RootedString name(jsdc->jsrt, nameStr);
     AutoSafeJSContext cx;
     JSAutoCompartment acBase(cx, jsdc->glob);
     JSDProperty* jsdprop;
-    JSDProperty* iter = NULL;
+    JSDProperty* iter = nullptr;
     JS::RootedObject obj(cx);
-    JS::RootedString name(cx, nameStr);
-    unsigned  attrs = 0;
-    JSBool found;
-    JSPropertyDesc pd;
-    const jschar * nameChars;
-    size_t nameLen;
     JS::RootedValue val(cx), nameval(cx);
     JS::RootedId nameid(cx);
+    JS::RootedValue propId(cx);
+    JS::RootedValue propValue(cx);
+    JS::RootedValue propAlias(cx);
+    uint8_t propFlags;
 
     if(!jsd_IsValueObject(jsdc, jsdval))
-        return NULL;
+        return nullptr;
 
     /* If we already have the prop, then return it */
-    while(NULL != (jsdprop = jsd_IterateProperties(jsdc, jsdval, &iter)))
+    while(nullptr != (jsdprop = jsd_IterateProperties(jsdc, jsdval, &iter)))
     {
         JSString* propName = jsd_GetValueString(jsdc, jsdprop->name);
         if(propName) {
@@ -492,58 +500,58 @@ jsd_GetValueProperty(JSDContext* jsdc, JSDValue* jsdval, JSString* nameStr)
     }
     /* Not found in property list, look it up explicitly */
 
+    nameval = STRING_TO_JSVAL(name);
+    if(!JS_ValueToId(cx, nameval, nameid.address()))
+        return nullptr;
+
     if(!(obj = JSVAL_TO_OBJECT(jsdval->val)))
-        return NULL;
-
-    if (!(nameChars = JS_GetStringCharsZAndLength(cx, name, &nameLen)))
-        return NULL;
+        return nullptr;
 
+    JS::Rooted<JSPropertyDescriptor> desc(cx);
     {
         JSAutoCompartment ac(cx, obj);
+        JS::RootedId id(cx, nameid);
 
-        JS_GetUCPropertyAttributes(cx, obj, nameChars, nameLen, &attrs, &found);
-        if (!found)
-        {
-            return NULL;
-        }
+        if(!JS_WrapId(cx, id.address()))
+            return nullptr;
+        if(!JS_GetOwnPropertyDescriptorById(cx, obj, id, 0, &desc))
+            return nullptr;
+        if(!desc.object())
+            return nullptr;
 
         JS_ClearPendingException(cx);
 
-        if(!JS_GetUCProperty(cx, obj, nameChars, nameLen, &val))
+        if(!JS_GetPropertyById(cx, obj, id, &val))
         {
             if (JS_IsExceptionPending(cx))
             {
-                if (!JS_GetPendingException(cx, &pd.value))
+                if (!JS_GetPendingException(cx, &propValue))
                 {
-                    return NULL;
+                    return nullptr;
                 }
-                pd.flags = JSPD_EXCEPTION;
+                propFlags = JSPD_EXCEPTION;
             }
             else
             {
-                pd.flags = JSPD_ERROR;
-                pd.value = JSVAL_VOID;
+                propFlags = JSPD_ERROR;
+                propValue = JSVAL_VOID;
             }
         }
         else
         {
-            pd.value = val;
+            propValue = val;
         }
     }
 
-    nameval = STRING_TO_JSVAL(name);
-    if (!JS_ValueToId(cx, nameval, nameid.address()) ||
-        !JS_IdToValue(cx, nameid, &pd.id)) {
-        return NULL;
-    }
+    if (!JS_IdToValue(cx, nameid, propId.address()))
+        return nullptr;
 
-    pd.spare = 0;
-    pd.alias = JSVAL_NULL;
-    pd.flags |= (attrs & JSPROP_ENUMERATE) ? JSPD_ENUMERATE : 0
-        | (attrs & JSPROP_READONLY)  ? JSPD_READONLY  : 0
-        | (attrs & JSPROP_PERMANENT) ? JSPD_PERMANENT : 0;
+    propAlias = JSVAL_NULL;
+    propFlags |= desc.isEnumerable() ? JSPD_ENUMERATE : 0
+        | desc.isReadonly() ? JSPD_READONLY  : 0
+        | desc.isPermanent() ? JSPD_PERMANENT : 0;
 
-    return _newProperty(jsdc, &pd, JSDPD_HINTED);
+    return _newProperty(jsdc, propId, propValue, propAlias, propFlags, JSDPD_HINTED);
 }
 
 /*
@@ -559,11 +567,12 @@ jsd_GetValueFunction(JSDContext* jsdc, JSDValue* jsdval)
     JS::RootedFunction fun(cx);
 
     if (JSVAL_IS_PRIMITIVE(jsdval->val))
-        return NULL;
+        return nullptr;
 
     obj = js::UncheckedUnwrap(JSVAL_TO_OBJECT(jsdval->val));
     JSAutoCompartment ac(cx, obj);
-    fun = JS_ValueToFunction(cx, OBJECT_TO_JSVAL(obj));
+    JS::RootedValue funval(cx, JS::ObjectValue(*obj));
+    fun = JS_ValueToFunction(cx, funval);
 
     return fun;
 }
@@ -579,12 +588,12 @@ jsd_GetValuePrototype(JSDContext* jsdc, JSDValue* jsdval)
         JS_ASSERT(!jsdval->proto);
         SET_BIT_FLAG(jsdval->flags, GOT_PROTO);
         if(JSVAL_IS_PRIMITIVE(jsdval->val))
-            return NULL;
+            return nullptr;
         obj = JSVAL_TO_OBJECT(jsdval->val);
         if(!JS_GetPrototype(cx, obj, &proto))
-            return NULL;
+            return nullptr;
         if(!proto)
-            return NULL;
+            return nullptr;
         jsdval->proto = jsd_NewValue(jsdc, OBJECT_TO_JSVAL(proto));
     }
     if(jsdval->proto)
@@ -603,14 +612,14 @@ jsd_GetValueParent(JSDContext* jsdc, JSDValue* jsdval)
         JS_ASSERT(!jsdval->parent);
         SET_BIT_FLAG(jsdval->flags, GOT_PARENT);
         if(JSVAL_IS_PRIMITIVE(jsdval->val))
-            return NULL;
+            return nullptr;
         obj = JSVAL_TO_OBJECT(jsdval->val);
         {
             JSAutoCompartment ac(cx, obj);
             parent = JS_GetParentOrScopeChain(cx, obj);
         }
         if(!parent)
-            return NULL;
+            return nullptr;
         jsdval->parent = jsd_NewValue(jsdc, OBJECT_TO_JSVAL(parent));
     }
     if(jsdval->parent)
@@ -630,18 +639,18 @@ jsd_GetValueConstructor(JSDContext* jsdc, JSDValue* jsdval)
         JS_ASSERT(!jsdval->ctor);
         SET_BIT_FLAG(jsdval->flags, GOT_CTOR);
         if(JSVAL_IS_PRIMITIVE(jsdval->val))
-            return NULL;
+            return nullptr;
         obj = JSVAL_TO_OBJECT(jsdval->val);
         if(!JS_GetPrototype(cx, obj, &proto))
-            return NULL;
+            return nullptr;
         if(!proto)
-            return NULL;
+            return nullptr;
         {
             JSAutoCompartment ac(cx, obj);
             ctor = JS_GetConstructor(cx, proto);
         }
         if(!ctor)
-            return NULL;
+            return nullptr;
         jsdval->ctor = jsd_NewValue(jsdc, OBJECT_TO_JSVAL(ctor));
     }
     if(jsdval->ctor)
@@ -655,8 +664,8 @@ jsd_GetValueClassName(JSDContext* jsdc, JSDValue* jsdval)
     jsval val = jsdval->val;
     if(!jsdval->className && !JSVAL_IS_PRIMITIVE(val))
     {
+        JS::RootedObject obj(jsdc->jsrt, JSVAL_TO_OBJECT(val));
         AutoSafeJSContext cx;
-        JS::RootedObject obj(cx, JSVAL_TO_OBJECT(val));
         JSAutoCompartment ac(cx, obj);
         jsdval->className = JS_GetDebugClassName(obj);
     }
@@ -668,12 +677,12 @@ jsd_GetScriptForValue(JSDContext* jsdc, JSDValue* jsdval)
 {
     AutoSafeJSContext cx;
     JS::RootedValue val(cx, jsdval->val);
-    JSFunction* fun = NULL;
+    JSFunction* fun = nullptr;
     JS::RootedScript script(cx);
     JSDScript* jsdscript;
 
     if (!jsd_IsValueFunction(jsdc, jsdval))
-        return NULL;
+        return nullptr;
 
     {
         JSAutoCompartment ac(cx, JSVAL_TO_OBJECT(val));
@@ -684,7 +693,7 @@ jsd_GetScriptForValue(JSDContext* jsdc, JSDValue* jsdval)
     }
 
     if (!script)
-        return NULL;
+        return nullptr;
 
     JSD_LOCK_SCRIPTS(jsdc);
     jsdscript = jsd_FindJSDScript(jsdc, script);
