@@ -4,22 +4,16 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#ifndef ParallelArray_h__
-#define ParallelArray_h__
+#ifndef builtin_ParallelArray_h
+#define builtin_ParallelArray_h
 
-#include "jsapi.h"
-#include "jscntxt.h"
 #include "jsobj.h"
 
-#include "ion/Ion.h"
-#include "vm/ForkJoin.h"
-#include "vm/ThreadPool.h"
-
 namespace js {
 
 class ParallelArrayObject : public JSObject
 {
-    static Class protoClass;
+    static const Class protoClass;
     static const JSFunctionSpec methods[];
     static const uint32_t NumFixedSlots = 4;
     static const uint32_t NumCtors = 4;
@@ -28,10 +22,10 @@ class ParallelArrayObject : public JSObject
     static bool initProps(JSContext *cx, HandleObject obj);
 
   public:
-    static Class class_;
+    static const Class class_;
 
-    static JSBool construct(JSContext *cx, unsigned argc, Value *vp);
-    static JSBool constructHelper(JSContext *cx, MutableHandleFunction ctor, CallArgs &args);
+    static bool construct(JSContext *cx, unsigned argc, Value *vp);
+    static bool constructHelper(JSContext *cx, MutableHandleFunction ctor, CallArgs &args);
 
     // Creates a new ParallelArray instance with the correct number of slots
     // and so forth.
@@ -57,4 +51,4 @@ class ParallelArrayObject : public JSObject
 extern JSObject *
 js_InitParallelArrayClass(JSContext *cx, js::HandleObject obj);
 
-#endif // ParallelArray_h__
+#endif /* builtin_ParallelArray_h */
