@@ -41,7 +41,7 @@ struct CGObjectList {
     uint32_t            length;     /* number of emitted so far objects */
     ObjectBox           *lastbox;   /* last emitted object */
 
-    CGObjectList() : length(0), lastbox(NULL) {}
+    CGObjectList() : length(0), lastbox(nullptr) {}
 
     unsigned add(ObjectBox *objbox);
     unsigned indexOf(JSObject *obj);
@@ -124,6 +124,10 @@ struct BytecodeEmitter
 
     bool            emittingRunOnceLambda:1; /* true while emitting a lambda which is only
                                                 expected to run once. */
+    bool            lazyRunOnceLambda:1; /* true while lazily emitting a script for
+                                          * a lambda which is only expected to run once. */
+
+    bool isRunOnceLambda();
 
     bool            insideEval:1;       /* True if compiling an eval-expression or a function
                                            nested inside an eval. */
