@@ -4,48 +4,49 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "jsapi.h"
-#include "jscntxt.h"
-#include "jsgc.h"
-#include "jsprf.h"
-#include "jsutil.h"
-#include "jswatchpoint.h"
-
-#include "mozilla/Util.h"
-
-#include "js/HashTable.h"
-#include "gc/GCInternals.h"
-
-#include "jsobjinlines.h"
-#include "jsgcinlines.h"
-
 #ifdef MOZ_VALGRIND
 # include <valgrind/memcheck.h>
 #endif
 
+#include "jscntxt.h"
+#include "jsgc.h"
+#include "jsprf.h"
+
+#include "gc/GCInternals.h"
+#include "gc/Zone.h"
+#include "js/HashTable.h"
+
+#include "jsgcinlines.h"
+
 using namespace js;
 using namespace js::gc;
 using namespace mozilla;
 
 #if defined(DEBUG) && defined(JS_GC_ZEAL) && defined(JSGC_ROOT_ANALYSIS) && !defined(JS_THREADSAFE)
+#  if JS_STACK_GROWTH_DIRECTION > 0
+#    error "Root analysis is only supported on a descending stack."
+#  endif
 
 template <typename T>
 bool
-CheckNonAddressThing(uintptr_t *w, T *t)
+CheckNonAddressThing(uintptr_t *w, Rooted<T> *rootp)
 {
-    return w >= (uintptr_t*)t && w < (uintptr_t*)(t + 1);
+    return w >= (uintptr_t*)rootp->address() && w < (uintptr_t*)(rootp->address() + 1);
 }
 
-JS_ALWAYS_INLINE bool
-CheckStackRootThing(uintptr_t *w, void *address, ThingRootKind kind)
+static JS_ALWAYS_INLINE bool
+CheckStackRootThing(uintptr_t *w, Rooted<void *> *rootp, ThingRootKind kind)
 {
     if (kind == THING_ROOT_BINDINGS)
-        return CheckNonAddressThing(w, static_cast<Bindings*>(address));
+        return CheckNonAddressThing(w, reinterpret_cast<Rooted<Bindings> *>(rootp));
 
     if (kind == THING_ROOT_PROPERTY_DESCRIPTOR)
-        return CheckNonAddressThing(w, static_cast<PropertyDescriptor*>(address));
+        return CheckNonAddressThing(w, reinterpret_cast<Rooted<PropertyDescriptor> *>(rootp));
 
-    return address == static_cast<void*>(w);
+    if (kind == THING_ROOT_VALUE)
+        return CheckNonAddressThing(w, reinterpret_cast<Rooted<Value> *>(rootp));
+
+    return rootp->address() == static_cast<void*>(w);
 }
 
 struct Rooter {
@@ -66,7 +67,7 @@ CheckStackRoot(JSRuntime *rt, uintptr_t *w, Rooter *begin, Rooter *end)
         return;
 
     /* Don't check atoms as these will never be subject to generational collection. */
-    if (static_cast<Cell *>(thing)->tenuredZone() == rt->atomsCompartment->zone())
+    if (rt->isAtomsZone(static_cast<Cell *>(thing)->tenuredZone()))
         return;
 
     /*
@@ -76,7 +77,7 @@ CheckStackRoot(JSRuntime *rt, uintptr_t *w, Rooter *begin, Rooter *end)
      */
 
     for (Rooter *p = begin; p != end; p++) {
-        if (CheckStackRootThing(w, p->rooter->address(), p->kind))
+        if (CheckStackRootThing(w, p->rooter, p->kind))
             return;
     }
 
@@ -113,7 +114,7 @@ CheckStackRootsRange(JSRuntime *rt, uintptr_t *begin, uintptr_t *end, Rooter *rb
 }
 
 static void
-CheckStackRootsRangeAndSkipIon(JSRuntime *rt, uintptr_t *begin, uintptr_t *end, Rooter *rbegin, Rooter *rend)
+CheckStackRootsRangeAndSkipJit(JSRuntime *rt, uintptr_t *begin, uintptr_t *end, Rooter *rbegin, Rooter *rend)
 {
     /*
      * Regions of the stack between Ion activiations are marked exactly through
@@ -122,17 +123,17 @@ CheckStackRootsRangeAndSkipIon(JSRuntime *rt, uintptr_t *begin, uintptr_t *end,
      */
     uintptr_t *i = begin;
 
-#if JS_STACK_GROWTH_DIRECTION < 0 && defined(JS_ION)
-    for (ion::IonActivationIterator ion(rt); ion.more(); ++ion) {
-        uintptr_t *ionMin, *ionEnd;
-        ion.ionStackRange(ionMin, ionEnd);
+#if defined(JS_ION)
+    for (jit::JitActivationIterator iter(rt); !iter.done(); ++iter) {
+        uintptr_t *jitMin, *jitEnd;
+        iter.jitStackRange(jitMin, jitEnd);
 
-        uintptr_t *upto = Min(ionMin, end);
+        uintptr_t *upto = Min(jitMin, end);
         if (upto > i)
             CheckStackRootsRange(rt, i, upto, rbegin, rend);
         else
             break;
-        i = ionEnd;
+        i = jitEnd;
     }
 #endif
 
@@ -161,7 +162,7 @@ CompareRooters(const void *vpA, const void *vpB)
  * over and over, so we need more than a depth-1 memory.
  */
 static bool
-SuppressCheckRoots(Vector<Rooter, 0, SystemAllocPolicy> &rooters)
+SuppressCheckRoots(js::Vector<Rooter, 0, SystemAllocPolicy> &rooters)
 {
     static const unsigned int NumStackMemories = 6;
     static const size_t StackCheckDepth = 10;
@@ -177,15 +178,15 @@ SuppressCheckRoots(Vector<Rooter, 0, SystemAllocPolicy> &rooters)
     qsort(rooters.begin(), rooters.length(), sizeof(Rooter), CompareRooters);
 
     // Forward-declare a variable so its address can be used to mark the
-    // current top of the stack
+    // current top of the stack.
     unsigned int pos;
 
-    // Compute the hash of the current stack
+    // Compute the hash of the current stack.
     uint32_t hash = HashGeneric(&pos);
     for (unsigned int i = 0; i < Min(StackCheckDepth, rooters.length()); i++)
         hash = AddToHash(hash, rooters[rooters.length() - i - 1].rooter);
 
-    // Scan through the remembered stacks to find the current stack
+    // Scan through the remembered stacks to find the current stack.
     for (pos = 0; pos < numMemories; pos++) {
         if (stacks[pos] == hash) {
             // Skip this check. Technically, it is incorrect to not update the
@@ -195,7 +196,7 @@ SuppressCheckRoots(Vector<Rooter, 0, SystemAllocPolicy> &rooters)
         }
     }
 
-    // Replace the oldest remembered stack with our current stack
+    // Replace the oldest remembered stack with our current stack.
     stacks[oldestMemory] = hash;
     oldestMemory = (oldestMemory + 1) % NumStackMemories;
     if (numMemories < NumStackMemories)
@@ -205,7 +206,7 @@ SuppressCheckRoots(Vector<Rooter, 0, SystemAllocPolicy> &rooters)
 }
 
 static void
-GatherRooters(Vector<Rooter, 0, SystemAllocPolicy> &rooters,
+GatherRooters(js::Vector<Rooter, 0, SystemAllocPolicy> &rooters,
               Rooted<void*> **thingGCRooters,
               unsigned thingRootKind)
 {
@@ -220,20 +221,20 @@ GatherRooters(Vector<Rooter, 0, SystemAllocPolicy> &rooters,
 void
 JS::CheckStackRoots(JSContext *cx)
 {
-    JSRuntime *rt = cx->runtime;
+    JSRuntime *rt = cx->runtime();
 
     if (rt->gcZeal_ != ZealStackRootingValue)
         return;
 
     // GCs can't happen when analysis/inference/compilation are active.
-    if (cx->compartment->activeAnalysis)
+    if (cx->compartment()->activeAnalysis)
         return;
 
     if (rt->mainThread.suppressGC)
         return;
 
     // Can switch to the atoms compartment during analysis.
-    if (IsAtomsCompartment(cx->compartment)) {
+    if (IsAtomsCompartment(cx->compartment())) {
         for (CompartmentsIter c(rt); !c.done(); c.next()) {
             if (c.get()->activeAnalysis)
                 return;
@@ -247,16 +248,12 @@ JS::CheckStackRoots(JSContext *cx)
 
     JS_ASSERT(cgcd->hasStackToScan());
     uintptr_t *stackMin, *stackEnd;
-#if JS_STACK_GROWTH_DIRECTION > 0
-    stackMin = rt->nativeStackBase;
-    stackEnd = cgcd->nativeStackTop;
-#else
     stackMin = cgcd->nativeStackTop + 1;
     stackEnd = reinterpret_cast<uintptr_t *>(rt->nativeStackBase);
-#endif
+    JS_ASSERT(stackMin <= stackEnd);
 
     // Gather up all of the rooters
-    Vector<Rooter, 0, SystemAllocPolicy> rooters;
+    js::Vector<Rooter, 0, SystemAllocPolicy> rooters;
     for (unsigned i = 0; i < THING_ROOT_LIMIT; i++) {
         for (ContextIter cx(rt); !cx.done(); cx.next()) {
             GatherRooters(rooters, cx->thingGCRooters, i);
@@ -271,16 +268,11 @@ JS::CheckStackRoots(JSContext *cx)
     // Truncate stackEnd to just after the address of the youngest
     // already-scanned rooter on the stack, to avoid re-scanning the rest of
     // the stack.
-    void *firstScanned = NULL;
+    void *firstScanned = nullptr;
     for (Rooter *p = rooters.begin(); p != rooters.end(); p++) {
         if (p->rooter->scanned) {
             uintptr_t *addr = reinterpret_cast<uintptr_t*>(p->rooter);
-#if JS_STACK_GROWTH_DIRECTION < 0
-            if (stackEnd > addr)
-#else
-            if (stackEnd < addr)
-#endif
-            {
+            if (stackEnd > addr) {
                 stackEnd = addr;
                 firstScanned = p->rooter;
             }
@@ -292,25 +284,19 @@ JS::CheckStackRoots(JSContext *cx)
     Rooter *firstToScan = rooters.begin();
     if (firstScanned) {
         for (Rooter *p = rooters.begin(); p != rooters.end(); p++) {
-#if JS_STACK_GROWTH_DIRECTION < 0
-            if (p->rooter >= firstScanned)
-#else
-            if (p->rooter <= firstScanned)
-#endif
-            {
+            if (p->rooter >= firstScanned) {
                 Swap(*firstToScan, *p);
                 ++firstToScan;
             }
         }
     }
 
-    JS_ASSERT(stackMin <= stackEnd);
-    CheckStackRootsRangeAndSkipIon(rt, stackMin, stackEnd, firstToScan, rooters.end());
+    CheckStackRootsRangeAndSkipJit(rt, stackMin, stackEnd, firstToScan, rooters.end());
     CheckStackRootsRange(rt, cgcd->registerSnapshot.words,
                          ArrayEnd(cgcd->registerSnapshot.words),
                          firstToScan, rooters.end());
 
-    // Mark all rooters as scanned
+    // Mark all rooters as scanned.
     for (Rooter *p = rooters.begin(); p != rooters.end(); p++)
         p->rooter->scanned = true;
 }
@@ -319,37 +305,6 @@ JS::CheckStackRoots(JSContext *cx)
 
 #ifdef JS_GC_ZEAL
 
-static void
-DisableGGCForVerification(JSRuntime *rt)
-{
-#ifdef JSGC_GENERATIONAL
-    if (rt->gcVerifyPreData || rt->gcVerifyPostData)
-        return;
-
-    if (rt->gcNursery.isEnabled()) {
-        MinorGC(rt, JS::gcreason::API);
-        rt->gcNursery.disable();
-    }
-
-    if (rt->gcStoreBuffer.isEnabled())
-        rt->gcStoreBuffer.disable();
-#endif
-}
-
-static void
-EnableGGCAfterVerification(JSRuntime *rt)
-{
-#ifdef JSGC_GENERATIONAL
-    if (rt->gcVerifyPreData || rt->gcVerifyPostData)
-        return;
-
-    if (rt->gcGenerationalEnabled) {
-        rt->gcNursery.enable();
-        rt->gcStoreBuffer.enable();
-    }
-#endif
-}
-
 /*
  * Write barrier verification
  *
@@ -423,7 +378,7 @@ struct VerifyPreTracer : JSTracer {
     char *term;
     NodeMap nodemap;
 
-    VerifyPreTracer() : root(NULL) {}
+    VerifyPreTracer() : root(nullptr) {}
     ~VerifyPreTracer() { js_free(root); }
 };
 
@@ -436,6 +391,8 @@ AccumulateEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
 {
     VerifyPreTracer *trc = (VerifyPreTracer *)jstrc;
 
+    JS_ASSERT(!IsInsideNursery(trc->runtime, *(uintptr_t **)thingp));
+
     trc->edgeptr += sizeof(EdgeValue);
     if (trc->edgeptr >= trc->term) {
         trc->edgeptr = trc->term;
@@ -447,7 +404,7 @@ AccumulateEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
 
     node->edges[i].thing = *thingp;
     node->edges[i].kind = kind;
-    node->edges[i].label = trc->debugPrinter ? NULL : (char *)trc->debugPrintArg;
+    node->edges[i].label = trc->debugPrinter ? nullptr : (char *)trc->debugPrintArg;
     node->count++;
 }
 
@@ -460,7 +417,7 @@ MakeNode(VerifyPreTracer *trc, void *thing, JSGCTraceKind kind)
         trc->edgeptr += sizeof(VerifyNode) - sizeof(EdgeValue);
         if (trc->edgeptr >= trc->term) {
             trc->edgeptr = trc->term;
-            return NULL;
+            return nullptr;
         }
 
         node->thing = thing;
@@ -469,7 +426,7 @@ MakeNode(VerifyPreTracer *trc, void *thing, JSGCTraceKind kind)
         trc->nodemap.add(p, thing, node);
         return node;
     }
-    return NULL;
+    return nullptr;
 }
 
 static VerifyNode *
@@ -485,21 +442,22 @@ NextNode(VerifyNode *node)
 void
 gc::StartVerifyPreBarriers(JSRuntime *rt)
 {
-    if (rt->gcVerifyPreData ||
-        rt->gcIncrementalState != NO_INCREMENTAL ||
-        !IsIncrementalGCSafe(rt))
-    {
+    if (rt->gcVerifyPreData || rt->gcIncrementalState != NO_INCREMENTAL)
         return;
-    }
 
-    DisableGGCForVerification(rt);
+    MinorGC(rt, JS::gcreason::API);
 
     AutoPrepareForTracing prep(rt);
 
+    if (!IsIncrementalGCSafe(rt))
+        return;
+
     for (GCChunkSet::Range r(rt->gcChunkSet.all()); !r.empty(); r.popFront())
         r.front()->bitmap.clear();
 
     VerifyPreTracer *trc = js_new<VerifyPreTracer>();
+    if (!trc)
+        return;
 
     rt->gcNumber++;
     trc->number = rt->gcNumber;
@@ -518,7 +476,7 @@ gc::StartVerifyPreBarriers(JSRuntime *rt)
         goto oom;
 
     /* Create the root node. */
-    trc->curnode = MakeNode(trc, NULL, JSGCTraceKind(0));
+    trc->curnode = MakeNode(trc, nullptr, JSGCTraceKind(0));
 
     /* We want MarkRuntime to save the roots to gcSavedRoots. */
     rt->gcIncrementalState = MARK_ROOTS;
@@ -563,8 +521,7 @@ gc::StartVerifyPreBarriers(JSRuntime *rt)
 oom:
     rt->gcIncrementalState = NO_INCREMENTAL;
     js_delete(trc);
-    rt->gcVerifyPreData = NULL;
-    EnableGGCAfterVerification(rt);
+    rt->gcVerifyPreData = nullptr;
 }
 
 static bool
@@ -573,14 +530,14 @@ IsMarkedOrAllocated(Cell *cell)
     return cell->isMarked() || cell->arenaHeader()->allocatedDuringIncremental;
 }
 
-const static uint32_t MAX_VERIFIER_EDGES = 1000;
+static const uint32_t MAX_VERIFIER_EDGES = 1000;
 
 /*
  * This function is called by EndVerifyBarriers for every heap edge. If the edge
  * already existed in the original snapshot, we "cancel it out" by overwriting
- * it with NULL. EndVerifyBarriers later asserts that the remaining non-NULL
- * edges (i.e., the ones from the original snapshot that must have been
- * modified) must point to marked objects.
+ * it with nullptr. EndVerifyBarriers later asserts that the remaining
+ * non-nullptr edges (i.e., the ones from the original snapshot that must have
+ * been modified) must point to marked objects.
  */
 static void
 CheckEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
@@ -595,7 +552,7 @@ CheckEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
     for (uint32_t i = 0; i < node->count; i++) {
         if (node->edges[i].thing == *thingp) {
             JS_ASSERT(node->edges[i].kind == kind);
-            node->edges[i].thing = NULL;
+            node->edges[i].thing = nullptr;
             return;
         }
     }
@@ -644,7 +601,7 @@ gc::EndVerifyPreBarriers(JSRuntime *rt)
     JS_ASSERT(trc->number == rt->gcNumber);
     rt->gcNumber++;
 
-    rt->gcVerifyPreData = NULL;
+    rt->gcVerifyPreData = nullptr;
     rt->gcIncrementalState = NO_INCREMENTAL;
 
     if (!compartmentCreated && IsIncrementalGCSafe(rt)) {
@@ -669,8 +626,6 @@ gc::EndVerifyPreBarriers(JSRuntime *rt)
     rt->gcMarker.stop();
 
     js_delete(trc);
-
-    EnableGGCAfterVerification(rt);
 }
 
 /*** Post-Barrier Verifyier ***/
@@ -681,6 +636,10 @@ struct VerifyPostTracer : JSTracer {
 
     /* This counts up to gcZealFrequency to decide whether to verify. */
     int count;
+
+    /* The set of edges in the StoreBuffer at the end of verification. */
+    typedef HashSet<void **, PointerHasher<void **, 3>, SystemAllocPolicy> EdgeSet;
+    EdgeSet *edges;
 };
 
 /*
@@ -698,38 +657,53 @@ gc::StartVerifyPostBarriers(JSRuntime *rt)
         return;
     }
 
-    DisableGGCForVerification(rt);
+    MinorGC(rt, JS::gcreason::API);
 
     VerifyPostTracer *trc = js_new<VerifyPostTracer>();
+    if (!trc)
+        return;
+
     rt->gcVerifyPostData = trc;
     rt->gcNumber++;
     trc->number = rt->gcNumber;
     trc->count = 0;
-
-    if (!rt->gcVerifierNursery.enable())
-        goto oom;
-
-    if (!rt->gcStoreBuffer.enable())
-        goto oom;
-
-    return;
-oom:
-    js_delete(trc);
-    rt->gcVerifyPostData = NULL;
-    EnableGGCAfterVerification(rt);
 #endif
 }
 
 #ifdef JSGC_GENERATIONAL
-static void
-AssertStoreBufferContainsEdge(StoreBuffer *storebuf, void *loc, void *dst)
+void
+PostVerifierCollectStoreBufferEdges(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
 {
-    if (storebuf->containsEdgeAt(loc))
+    VerifyPostTracer *trc = (VerifyPostTracer *)jstrc;
+
+    /* The nursery only stores objects. */
+    if (kind != JSTRACE_OBJECT)
+        return;
+
+    /* The store buffer may store extra, non-cross-generational edges. */
+    JSObject *dst = *reinterpret_cast<JSObject **>(thingp);
+    if (trc->runtime->gcNursery.isInside(thingp) || !trc->runtime->gcNursery.isInside(dst))
+        return;
+
+    /*
+     * Values will be unpacked to the stack before getting here. However, the
+     * only things that enter this callback are marked by the store buffer. The
+     * store buffer ensures that the real tracing location is set correctly.
+     */
+    void **loc = trc->realLocation != nullptr ? (void **)trc->realLocation : thingp;
+
+    trc->edges->put(loc);
+}
+
+static void
+AssertStoreBufferContainsEdge(VerifyPostTracer::EdgeSet *edges, void **loc, JSObject *dst)
+{
+    if (edges->has(loc))
         return;
 
     char msgbuf[1024];
     JS_snprintf(msgbuf, sizeof(msgbuf), "[post-barrier verifier] Missing edge @ %p to %p",
-                loc, dst);
+                (void *)loc, (void *)dst);
     MOZ_ReportAssertionFailure(msgbuf, __FILE__, __LINE__);
     MOZ_CRASH();
 }
@@ -738,21 +712,26 @@ void
 PostVerifierVisitEdge(JSTracer *jstrc, void **thingp, JSGCTraceKind kind)
 {
     VerifyPostTracer *trc = (VerifyPostTracer *)jstrc;
-    Cell *dst = (Cell *)*thingp;
-    JSRuntime *rt = dst->runtime();
+
+    /* The nursery only stores objects. */
+    if (kind != JSTRACE_OBJECT)
+        return;
 
     /* Filter out non cross-generational edges. */
-    if (!rt->gcVerifierNursery.isInside(dst))
+    JS_ASSERT(!trc->runtime->gcNursery.isInside(thingp));
+    JSObject *dst = *reinterpret_cast<JSObject **>(thingp);
+    if (!trc->runtime->gcNursery.isInside(dst))
         return;
 
     /*
-     * Note: since Value travels through the stack to get Cell**, we need to use
-     * the annotated location in the tracer instead of the indirect location for
-     * these edges.
+     * Values will be unpacked to the stack before getting here. However, the
+     * only things that enter this callback are marked by the JS_TraceChildren
+     * below. Since ObjectImpl::markChildren handles this, the real trace
+     * location will be set correctly in these cases.
      */
-    Cell *loc = (Cell *)(trc->realLocation != NULL ? trc->realLocation : thingp);
+    void **loc = trc->realLocation != nullptr ? (void **)trc->realLocation : thingp;
 
-    AssertStoreBufferContainsEdge(&rt->gcStoreBuffer, loc, dst);
+    AssertStoreBufferContainsEdge(trc->edges, loc, dst);
 }
 #endif
 
@@ -760,36 +739,32 @@ void
 js::gc::EndVerifyPostBarriers(JSRuntime *rt)
 {
 #ifdef JSGC_GENERATIONAL
+    VerifyPostTracer::EdgeSet edges;
     AutoPrepareForTracing prep(rt);
 
     VerifyPostTracer *trc = (VerifyPostTracer *)rt->gcVerifyPostData;
+
+    /* Visit every entry in the store buffer and put the edges in a hash set. */
+    JS_TracerInit(trc, rt, PostVerifierCollectStoreBufferEdges);
+    if (!edges.init())
+        goto oom;
+    trc->edges = &edges;
+    rt->gcStoreBuffer.mark(trc);
+
+    /* Walk the heap to find any edges not the the |edges| set. */
     JS_TracerInit(trc, rt, PostVerifierVisitEdge);
-    trc->count = 0;
-
-    if (rt->gcStoreBuffer.hasOverflowed())
-        goto oom;
-
-    if (!rt->gcStoreBuffer.coalesceForVerification())
-        goto oom;
-
-    /* Walk the heap. */
     for (GCZoneGroupIter zone(rt); !zone.done(); zone.next()) {
         for (size_t kind = 0; kind < FINALIZE_LIMIT; ++kind) {
             for (CellIterUnderGC cells(zone, AllocKind(kind)); !cells.done(); cells.next()) {
                 Cell *src = cells.getCell();
-                if (!rt->gcVerifierNursery.isInside(src))
-                    JS_TraceChildren(trc, src, MapAllocToTraceKind(AllocKind(kind)));
+                JS_TraceChildren(trc, src, MapAllocToTraceKind(AllocKind(kind)));
             }
         }
     }
 
 oom:
     js_delete(trc);
-    rt->gcVerifyPostData = NULL;
-    rt->gcVerifierNursery.disable();
-    rt->gcStoreBuffer.disable();
-    rt->gcStoreBuffer.releaseVerificationData();
-    EnableGGCAfterVerification(rt);
+    rt->gcVerifyPostData = nullptr;
 #endif
 }
 
@@ -861,8 +836,8 @@ MaybeVerifyPostBarriers(JSRuntime *rt, bool always)
 void
 js::gc::MaybeVerifyBarriers(JSContext *cx, bool always)
 {
-    MaybeVerifyPreBarriers(cx->runtime, always);
-    MaybeVerifyPostBarriers(cx->runtime, always);
+    MaybeVerifyPreBarriers(cx->runtime(), always);
+    MaybeVerifyPostBarriers(cx->runtime(), always);
 }
 
 void
@@ -870,18 +845,14 @@ js::gc::FinishVerifier(JSRuntime *rt)
 {
     if (VerifyPreTracer *trc = (VerifyPreTracer *)rt->gcVerifyPreData) {
         js_delete(trc);
-        rt->gcVerifyPreData = NULL;
+        rt->gcVerifyPreData = nullptr;
     }
 #ifdef JSGC_GENERATIONAL
     if (VerifyPostTracer *trc = (VerifyPostTracer *)rt->gcVerifyPostData) {
         js_delete(trc);
-        rt->gcVerifierNursery.disable();
-        rt->gcStoreBuffer.disable();
-        rt->gcStoreBuffer.releaseVerificationData();
-        rt->gcVerifyPostData = NULL;
+        rt->gcVerifyPostData = nullptr;
     }
 #endif
-    EnableGGCAfterVerification(rt);
 }
 
 #endif /* JS_GC_ZEAL */
