@@ -2,10 +2,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "tests.h"
 #include "jsatom.h"
 
 #include "gc/Marking.h"
+#include "jsapi-tests/tests.h"
 #include "vm/String.h"
 
 using mozilla::ArrayLength;
@@ -22,14 +22,14 @@ BEGIN_TEST(testAtomizedIsNotInterned)
 }
 END_TEST(testAtomizedIsNotInterned)
 
-struct StringWrapper
+struct StringWrapperStruct
 {
     JSString *str;
     bool     strOk;
 } sw;
 
 void
-FinalizeCallback(JSFreeOp *fop, JSFinalizeStatus status, JSBool isCompartmentGC)
+FinalizeCallback(JSFreeOp *fop, JSFinalizeStatus status, bool isCompartmentGC)
 {
     if (status == JSFINALIZE_GROUP_START)
         sw.strOk = js::gc::IsStringMarked(&sw.str);
