@@ -2,15 +2,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "tests.h"
+#include "mozilla/DebugOnly.h"
 
-#include "mozilla/Util.h"
+#include "jsapi-tests/tests.h"
 
 BEGIN_TEST(testOOM)
 {
-    JSString *jsstr = JS_ValueToString(cx, INT_TO_JSVAL(9));
-    jsval tmp = STRING_TO_JSVAL(jsstr);
-    JS_SetProperty(cx, global, "rootme", &tmp);
+    JS::RootedString jsstr(cx, JS_ValueToString(cx, INT_TO_JSVAL(9)));
     mozilla::DebugOnly<const jschar *> s = JS_GetStringCharsZ(cx, jsstr);
     JS_ASSERT(s[0] == '9' && s[1] == '\0');
     return true;
@@ -19,7 +17,10 @@ BEGIN_TEST(testOOM)
 virtual JSRuntime * createRuntime()
 {
     JSRuntime *rt = JS_NewRuntime(0, JS_USE_HELPER_THREADS);
+    if (!rt)
+        return nullptr;
     JS_SetGCParameter(rt, JSGC_MAX_BYTES, (uint32_t)-1);
+    setNativeStackQuota(rt);
     return rt;
 }
 END_TEST(testOOM)
