@@ -16,8 +16,8 @@ BEGIN_TEST(testBug795104)
     s[0] = '"';
     memset(s + 1, 'x', strLen - 2);
     s[strLen - 1] = '"';
-    CHECK(JS::Evaluate(cx, global, opts, s, strLen, NULL));
-    CHECK(JS::CompileFunction(cx, global, opts, "f", 0, NULL, s, strLen));
+    CHECK(JS::Evaluate(cx, global, opts, s, strLen, nullptr));
+    CHECK(JS::CompileFunction(cx, global, opts, "f", 0, nullptr, s, strLen));
     JS_free(cx, s);
 
     return true;
@@ -39,9 +39,9 @@ BEGIN_TEST(testScriptSourceReentrant)
     JS::CompileOptions opts(cx);
     bool match = false;
     JS_SetNewScriptHook(rt, newScriptHook, &match);
-    CHECK(JS::Evaluate(cx, global, opts, simpleSource, strlen(simpleSource), NULL));
+    CHECK(JS::Evaluate(cx, global, opts, simpleSource, strlen(simpleSource), nullptr));
     CHECK(match);
-    JS_SetNewScriptHook(rt, NULL, NULL);
+    JS_SetNewScriptHook(rt, nullptr, nullptr);
 
     return true;
 }
