@@ -51,6 +51,7 @@ using mozilla::PositiveInfinity;
 using mozilla::NegativeInfinity;
 using mozilla::SpecificNaN;
 using JS::ToNumber;
+using JS::GenericNaN;
 
 #ifndef M_E
 #define M_E             2.7182818284590452354
@@ -101,7 +102,7 @@ MathCache::sizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf)
     return mallocSizeOf(this);
 }
 
-Class js::MathClass = {
+const Class js::MathClass = {
     js_Math_str,
     JSCLASS_HAS_CACHED_PROTO(JSProto_Math),
     JS_PropertyStub,         /* addProperty */
@@ -113,13 +114,13 @@ Class js::MathClass = {
     JS_ConvertStub
 };
 
-JSBool
+bool
 js_math_abs(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -132,23 +133,35 @@ js_math_abs(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+#if defined(SOLARIS) && defined(__GNUC__)
+#define ACOS_IF_OUT_OF_RANGE(x) if (x < -1 || 1 < x) return GenericNaN();
+#else
+#define ACOS_IF_OUT_OF_RANGE(x)
+#endif
+
 double
 js::math_acos_impl(MathCache *cache, double x)
 {
-#if defined(SOLARIS) && defined(__GNUC__)
-    if (x < -1 || 1 < x)
-        return js_NaN;
-#endif
+    ACOS_IF_OUT_OF_RANGE(x);
     return cache->lookup(acos, x);
 }
 
-JSBool
+double
+js::math_acos_uncached(double x)
+{
+    ACOS_IF_OUT_OF_RANGE(x);
+    return acos(x);
+}
+
+#undef ACOS_IF_OUT_OF_RANGE
+
+bool
 js::math_acos(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -165,23 +178,35 @@ js::math_acos(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+#if defined(SOLARIS) && defined(__GNUC__)
+#define ASIN_IF_OUT_OF_RANGE(x) if (x < -1 || 1 < x) return GenericNaN();
+#else
+#define ASIN_IF_OUT_OF_RANGE(x)
+#endif
+
 double
 js::math_asin_impl(MathCache *cache, double x)
 {
-#if defined(SOLARIS) && defined(__GNUC__)
-    if (x < -1 || 1 < x)
-        return js_NaN;
-#endif
+    ASIN_IF_OUT_OF_RANGE(x);
     return cache->lookup(asin, x);
 }
 
-JSBool
+double
+js::math_asin_uncached(double x)
+{
+    ASIN_IF_OUT_OF_RANGE(x);
+    return asin(x);
+}
+
+#undef ASIN_IF_OUT_OF_RANGE
+
+bool
 js::math_asin(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -204,13 +229,19 @@ js::math_atan_impl(MathCache *cache, double x)
     return cache->lookup(atan, x);
 }
 
-JSBool
+double
+js::math_atan_uncached(double x)
+{
+    return atan(x);
+}
+
+bool
 js::math_atan(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -257,13 +288,13 @@ js::ecmaAtan2(double y, double x)
     return atan2(y, x);
 }
 
-JSBool
+bool
 js::math_atan2(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() <= 1) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -277,7 +308,7 @@ js::math_atan2(JSContext *cx, unsigned argc, Value *vp)
 }
 
 double
-js_math_ceil_impl(double x)
+js::math_ceil_impl(double x)
 {
 #ifdef __APPLE__
     if (x < 0 && x > -1.0)
@@ -286,13 +317,13 @@ js_math_ceil_impl(double x)
     return ceil(x);
 }
 
-JSBool
-js_math_ceil(JSContext *cx, unsigned argc, Value *vp)
+bool
+js::math_ceil(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -300,7 +331,7 @@ js_math_ceil(JSContext *cx, unsigned argc, Value *vp)
     if (!ToNumber(cx, args[0], &x))
         return false;
 
-    double z = js_math_ceil_impl(x);
+    double z = math_ceil_impl(x);
     args.rval().setNumber(z);
     return true;
 }
@@ -311,13 +342,19 @@ js::math_cos_impl(MathCache *cache, double x)
     return cache->lookup(cos, x);
 }
 
-JSBool
+double
+js::math_cos_uncached(double x)
+{
+    return cos(x);
+}
+
+bool
 js::math_cos(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -334,27 +371,41 @@ js::math_cos(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+#ifdef _WIN32
+#define EXP_IF_OUT_OF_RANGE(x)                  \
+    if (!IsNaN(x)) {                            \
+        if (x == PositiveInfinity())            \
+            return PositiveInfinity();          \
+        if (x == NegativeInfinity())            \
+            return 0.0;                         \
+    }
+#else
+#define EXP_IF_OUT_OF_RANGE(x)
+#endif
+
 double
 js::math_exp_impl(MathCache *cache, double x)
 {
-#ifdef _WIN32
-    if (!IsNaN(x)) {
-        if (x == js_PositiveInfinity)
-            return js_PositiveInfinity;
-        if (x == js_NegativeInfinity)
-            return 0.0;
-    }
-#endif
+    EXP_IF_OUT_OF_RANGE(x);
     return cache->lookup(exp, x);
 }
 
-JSBool
+double
+js::math_exp_uncached(double x)
+{
+    EXP_IF_OUT_OF_RANGE(x);
+    return exp(x);
+}
+
+#undef EXP_IF_OUT_OF_RANGE
+
+bool
 js::math_exp(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -372,18 +423,18 @@ js::math_exp(JSContext *cx, unsigned argc, Value *vp)
 }
 
 double
-js_math_floor_impl(double x)
+js::math_floor_impl(double x)
 {
     return floor(x);
 }
 
-JSBool
-js_math_floor(JSContext *cx, unsigned argc, Value *vp)
+bool
+js::math_floor(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -391,12 +442,12 @@ js_math_floor(JSContext *cx, unsigned argc, Value *vp)
     if (!ToNumber(cx, args[0], &x))
         return false;
 
-    double z = js_math_floor_impl(x);
+    double z = math_floor_impl(x);
     args.rval().setNumber(z);
     return true;
 }
 
-JSBool
+bool
 js::math_imul(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
@@ -414,23 +465,54 @@ js::math_imul(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+bool
+js::math_fround(JSContext *cx, unsigned argc, Value *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+
+    if (args.length() == 0) {
+        args.rval().setNaN();
+        return true;
+    }
+
+    double x;
+    if (!ToNumber(cx, args[0], &x))
+        return false;
+
+    float f = x;
+    args.rval().setDouble(static_cast<double>(f));
+    return true;
+}
+
+#if defined(SOLARIS) && defined(__GNUC__)
+#define LOG_IF_OUT_OF_RANGE(x) if (x < 0) return GenericNaN();
+#else
+#define LOG_IF_OUT_OF_RANGE(x)
+#endif
+
 double
 js::math_log_impl(MathCache *cache, double x)
 {
-#if defined(SOLARIS) && defined(__GNUC__)
-    if (x < 0)
-        return js_NaN;
-#endif
+    LOG_IF_OUT_OF_RANGE(x);
     return cache->lookup(log, x);
 }
 
-JSBool
+double
+js::math_log_uncached(double x)
+{
+    LOG_IF_OUT_OF_RANGE(x);
+    return log(x);
+}
+
+#undef LOG_IF_OUT_OF_RANGE
+
+bool
 js::math_log(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -447,7 +529,7 @@ js::math_log(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
-JSBool
+bool
 js_math_max(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
@@ -465,7 +547,7 @@ js_math_max(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
-JSBool
+bool
 js_math_min(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
@@ -537,7 +619,7 @@ js::ecmaPow(double x, double y)
      * we need to wrap the libm call to make it ECMA compliant.
      */
     if (!IsFinite(y) && (x == 1.0 || x == -1.0))
-        return js_NaN;
+        return GenericNaN();
     /* pow(x, +-0) is always 1, even for x = NaN (MSVC gets this wrong). */
     if (y == 0)
         return 1;
@@ -551,13 +633,13 @@ js::ecmaPow(double x, double y)
 #if defined(_MSC_VER)
 # pragma optimize("g", off)
 #endif
-JSBool
+bool
 js_math_pow(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() <= 1) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -679,7 +761,7 @@ math_random_no_outparam(JSContext *cx)
     return random_nextDouble(cx);
 }
 
-JSBool
+bool
 js_math_random(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
@@ -688,13 +770,27 @@ js_math_random(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
-JSBool /* ES5 15.8.2.15. */
-js_math_round(JSContext *cx, unsigned argc, Value *vp)
+double
+js::math_round_impl(double x)
+{
+    int32_t i;
+    if (DoubleIsInt32(x, &i))
+        return double(i);
+
+    /* Some numbers are so big that adding 0.5 would give the wrong number. */
+    if (ExponentComponent(x) >= 52)
+        return x;
+
+    return js_copysign(floor(x + 0.5), x);
+}
+
+bool /* ES5 15.8.2.15. */
+js::math_round(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -702,19 +798,8 @@ js_math_round(JSContext *cx, unsigned argc, Value *vp)
     if (!ToNumber(cx, args[0], &x))
         return false;
 
-    int32_t i;
-    if (DoubleIsInt32(x, &i)) {
-        args.rval().setInt32(i);
-        return true;
-    }
-
-    /* Some numbers are so big that adding 0.5 would give the wrong number. */
-    if (ExponentComponent(x) >= 52) {
-        args.rval().setNumber(x);
-        return true;
-    }
-
-    args.rval().setNumber(js_copysign(floor(x + 0.5), x));
+    double z = math_round_impl(x);
+    args.rval().setNumber(z);
     return true;
 }
 
@@ -724,13 +809,19 @@ js::math_sin_impl(MathCache *cache, double x)
     return cache->lookup(sin, x);
 }
 
-JSBool
+double
+js::math_sin_uncached(double x)
+{
+    return sin(x);
+}
+
+bool
 js::math_sin(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -747,13 +838,13 @@ js::math_sin(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
-JSBool
+bool
 js_math_sqrt(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -776,13 +867,19 @@ js::math_tan_impl(MathCache *cache, double x)
     return cache->lookup(tan, x);
 }
 
-JSBool
+double
+js::math_tan_uncached(double x)
+{
+    return tan(x);
+}
+
+bool
 js::math_tan(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
     if (args.length() == 0) {
-        args.rval().setDouble(js_NaN);
+        args.rval().setNaN();
         return true;
     }
 
@@ -803,11 +900,11 @@ js::math_tan(JSContext *cx, unsigned argc, Value *vp)
 typedef double (*UnaryMathFunctionType)(MathCache *cache, double);
 
 template <UnaryMathFunctionType F>
-JSBool math_function(JSContext *cx, unsigned argc, Value *vp)
+static bool math_function(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
     if (args.length() == 0) {
-        args.rval().setNumber(js_NaN);
+        args.rval().setNumber(GenericNaN());
         return true;
     }
 
@@ -832,7 +929,13 @@ js::math_log10_impl(MathCache *cache, double x)
     return cache->lookup(log10, x);
 }
 
-JSBool
+double
+js::math_log10_uncached(double x)
+{
+    return log10(x);
+}
+
+bool
 js::math_log10(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_log10_impl>(cx, argc, vp);
@@ -851,7 +954,13 @@ js::math_log2_impl(MathCache *cache, double x)
     return cache->lookup(log2, x);
 }
 
-JSBool
+double
+js::math_log2_uncached(double x)
+{
+    return log2(x);
+}
+
+bool
 js::math_log2(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_log2_impl>(cx, argc, vp);
@@ -874,19 +983,30 @@ double log1p(double x)
 }
 #endif
 
+#ifdef __APPLE__
+// Ensure that log1p(-0) is -0.
+#define LOG1P_IF_OUT_OF_RANGE(x) if (x == 0) return x;
+#else
+#define LOG1P_IF_OUT_OF_RANGE(x)
+#endif
+
 double
 js::math_log1p_impl(MathCache *cache, double x)
 {
-#ifdef __APPLE__
-    // Ensure that log1p(-0) is -0.
-    if (x == 0)
-        return x;
-#endif
-
+    LOG1P_IF_OUT_OF_RANGE(x);
     return cache->lookup(log1p, x);
 }
 
-JSBool
+double
+js::math_log1p_uncached(double x)
+{
+    LOG1P_IF_OUT_OF_RANGE(x);
+    return log1p(x);
+}
+
+#undef LOG1P_IF_OUT_OF_RANGE
+
+bool
 js::math_log1p(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_log1p_impl>(cx, argc, vp);
@@ -919,7 +1039,13 @@ js::math_expm1_impl(MathCache *cache, double x)
     return cache->lookup(expm1, x);
 }
 
-JSBool
+double
+js::math_expm1_uncached(double x)
+{
+    return expm1(x);
+}
+
+bool
 js::math_expm1(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_expm1_impl>(cx, argc, vp);
@@ -943,7 +1069,13 @@ js::math_cosh_impl(MathCache *cache, double x)
     return cache->lookup(cosh, x);
 }
 
-JSBool
+double
+js::math_cosh_uncached(double x)
+{
+    return cosh(x);
+}
+
+bool
 js::math_cosh(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_cosh_impl>(cx, argc, vp);
@@ -955,7 +1087,13 @@ js::math_sinh_impl(MathCache *cache, double x)
     return cache->lookup(sinh, x);
 }
 
-JSBool
+double
+js::math_sinh_uncached(double x)
+{
+    return sinh(x);
+}
+
+bool
 js::math_sinh(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_sinh_impl>(cx, argc, vp);
@@ -967,7 +1105,13 @@ js::math_tanh_impl(MathCache *cache, double x)
     return cache->lookup(tanh, x);
 }
 
-JSBool
+double
+js::math_tanh_uncached(double x)
+{
+    return tanh(x);
+}
+
+bool
 js::math_tanh(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_tanh_impl>(cx, argc, vp);
@@ -1010,14 +1154,22 @@ js::math_acosh_impl(MathCache *cache, double x)
     return cache->lookup(acosh, x);
 }
 
-JSBool
+double
+js::math_acosh_uncached(double x)
+{
+    return acosh(x);
+}
+
+bool
 js::math_acosh(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_acosh_impl>(cx, argc, vp);
 }
 
 #if !HAVE_ASINH
-double asinh(double x)
+// Bug 899712 - gcc incorrectly rewrites -asinh(-x) to asinh(x) when overriding
+// asinh.
+static double my_asinh(double x)
 {
     const double SQUARE_ROOT_EPSILON = sqrt(std::numeric_limits<double>::epsilon());
     const double FOURTH_ROOT_EPSILON = sqrt(SQUARE_ROOT_EPSILON);
@@ -1032,7 +1184,7 @@ double asinh(double x)
         else
             return log(x + sqrt(x * x + 1));
     } else if (x <= -FOURTH_ROOT_EPSILON) {
-        return -asinh(-x);
+        return -my_asinh(-x);
     } else {
         // http://functions.wolfram.com/ElementaryFunctions/ArcSinh/06/01/03/01/0001/
         // approximation by taylor series in x at 0 up to order 2
@@ -1052,10 +1204,24 @@ double asinh(double x)
 double
 js::math_asinh_impl(MathCache *cache, double x)
 {
+#ifdef HAVE_ASINH
     return cache->lookup(asinh, x);
+#else
+    return cache->lookup(my_asinh, x);
+#endif
 }
 
-JSBool
+double
+js::math_asinh_uncached(double x)
+{
+#ifdef HAVE_ASINH
+    return asinh(x);
+#else
+    return my_asinh(x);
+#endif
+}
+
+bool
 js::math_asinh(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_asinh_impl>(cx, argc, vp);
@@ -1095,80 +1261,53 @@ js::math_atanh_impl(MathCache *cache, double x)
     return cache->lookup(atanh, x);
 }
 
-JSBool
+double
+js::math_atanh_uncached(double x)
+{
+    return atanh(x);
+}
+
+bool
 js::math_atanh(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_atanh_impl>(cx, argc, vp);
 }
 
-// Math.hypot is disabled pending the resolution of spec issues (bug 896264).
-#if 0
-#if !HAVE_HYPOT
-double hypot(double x, double y)
-{
-    if (mozilla::IsInfinite(x) || mozilla::IsInfinite(y))
-        return js_PositiveInfinity;
-
-    if (mozilla::IsNaN(x) || mozilla::IsNaN(y))
-        return js_NaN;
-
-    double xabs = mozilla::Abs(x);
-    double yabs = mozilla::Abs(y);
-
-    double min = std::min(xabs, yabs);
-    double max = std::max(xabs, yabs);
-
-    if (min == 0) {
-        return max;
-    } else {
-        double u = min / max;
-        return max * sqrt(1 + u * u);
-    }
-}
-#endif
-
-double
-js::math_hypot_impl(double x, double y)
-{
-#ifdef XP_WIN
-    // On Windows, hypot(NaN, Infinity) is NaN. ES6 requires Infinity.
-    if (mozilla::IsInfinite(x) || mozilla::IsInfinite(y))
-        return js_PositiveInfinity;
-#endif
-    return hypot(x, y);
-}
-
-JSBool
+bool
 js::math_hypot(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
-    if (args.length() < 2) {
-        args.rval().setNumber(js_NaN);
-        return true;
+
+    bool isInfinite = false;
+    bool isNaN = false;
+
+    double scale = 0;
+    double sumsq = 1;
+
+    for (unsigned i = 0; i < args.length(); i++) {
+        double x;
+        if (!ToNumber(cx, args[i], &x))
+            return false;
+
+        isInfinite |= mozilla::IsInfinite(x);
+        isNaN |= mozilla::IsNaN(x);
+
+        double xabs = mozilla::Abs(x);
+
+        if (scale < xabs) {
+            sumsq = 1 + sumsq * (scale / xabs) * (scale / xabs);
+            scale = xabs;
+        } else if (scale != 0) {
+            sumsq += (xabs / scale) * (xabs / scale);
+        }
     }
 
-    double x, y;
-    if (!ToNumber(cx, args[0], &x))
-        return false;
-
-    if (!ToNumber(cx, args[1], &y))
-        return false;
-
-    if (args.length() == 2) {
-        args.rval().setNumber(math_hypot_impl(x, y));
-        return true;
-    }
-
-    /* args.length() > 2 */
-    double z;
-    if (!ToNumber(cx, args[2], &z)) {
-        return false;
-    }
-
-    args.rval().setNumber(math_hypot_impl(math_hypot_impl(x, y), z));
+    double result = isInfinite ? PositiveInfinity() :
+                    isNaN ? GenericNaN() :
+                    scale * sqrt(sumsq);
+    args.rval().setNumber(result);
     return true;
 }
-#endif
 
 #if !HAVE_TRUNC
 double trunc(double x)
@@ -1183,16 +1322,22 @@ js::math_trunc_impl(MathCache *cache, double x)
     return cache->lookup(trunc, x);
 }
 
-JSBool
+double
+js::math_trunc_uncached(double x)
+{
+    return trunc(x);
+}
+
+bool
 js::math_trunc(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_trunc_impl>(cx, argc, vp);
 }
 
-double sign(double x)
+static double sign(double x)
 {
     if (mozilla::IsNaN(x))
-        return js_NaN;
+        return GenericNaN();
 
     return x == 0 ? x : x < 0 ? -1 : 1;
 }
@@ -1203,7 +1348,13 @@ js::math_sign_impl(MathCache *cache, double x)
     return cache->lookup(sign, x);
 }
 
-JSBool
+double
+js::math_sign_uncached(double x)
+{
+    return sign(x);
+}
+
+bool
 js::math_sign(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_sign_impl>(cx, argc, vp);
@@ -1228,14 +1379,20 @@ js::math_cbrt_impl(MathCache *cache, double x)
     return cache->lookup(cbrt, x);
 }
 
-JSBool
+double
+js::math_cbrt_uncached(double x)
+{
+    return cbrt(x);
+}
+
+bool
 js::math_cbrt(JSContext *cx, unsigned argc, Value *vp)
 {
     return math_function<math_cbrt_impl>(cx, argc, vp);
 }
 
 #if JS_HAS_TOSOURCE
-static JSBool
+static bool
 math_toSource(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
@@ -1253,17 +1410,18 @@ static const JSFunctionSpec math_static_methods[] = {
     JS_FN("asin",           math_asin,            1, 0),
     JS_FN("atan",           math_atan,            1, 0),
     JS_FN("atan2",          math_atan2,           2, 0),
-    JS_FN("ceil",           js_math_ceil,         1, 0),
+    JS_FN("ceil",           math_ceil,            1, 0),
     JS_FN("cos",            math_cos,             1, 0),
     JS_FN("exp",            math_exp,             1, 0),
-    JS_FN("floor",          js_math_floor,        1, 0),
+    JS_FN("floor",          math_floor,           1, 0),
     JS_FN("imul",           math_imul,            2, 0),
+    JS_FN("fround",         math_fround,          1, 0),
     JS_FN("log",            math_log,             1, 0),
     JS_FN("max",            js_math_max,          2, 0),
     JS_FN("min",            js_math_min,          2, 0),
     JS_FN("pow",            js_math_pow,          2, 0),
     JS_FN("random",         js_math_random,       0, 0),
-    JS_FN("round",          js_math_round,        1, 0),
+    JS_FN("round",          math_round,           1, 0),
     JS_FN("sin",            math_sin,             1, 0),
     JS_FN("sqrt",           js_math_sqrt,         1, 0),
     JS_FN("tan",            math_tan,             1, 0),
@@ -1277,10 +1435,7 @@ static const JSFunctionSpec math_static_methods[] = {
     JS_FN("acosh",          math_acosh,           1, 0),
     JS_FN("asinh",          math_asinh,           1, 0),
     JS_FN("atanh",          math_atanh,           1, 0),
-// Math.hypot is disabled pending the resolution of spec issues (bug 896264).
-#if 0
     JS_FN("hypot",          math_hypot,           2, 0),
-#endif
     JS_FN("trunc",          math_trunc,           1, 0),
     JS_FN("sign",           math_sign,            1, 0),
     JS_FN("cbrt",           math_cbrt,            1, 0),
@@ -1290,21 +1445,21 @@ static const JSFunctionSpec math_static_methods[] = {
 JSObject *
 js_InitMathClass(JSContext *cx, HandleObject obj)
 {
-    RootedObject Math(cx, NewObjectWithClassProto(cx, &MathClass, NULL, obj, SingletonObject));
+    RootedObject Math(cx, NewObjectWithClassProto(cx, &MathClass, nullptr, obj, SingletonObject));
     if (!Math)
-        return NULL;
+        return nullptr;
 
     if (!JS_DefineProperty(cx, obj, js_Math_str, OBJECT_TO_JSVAL(Math),
                            JS_PropertyStub, JS_StrictPropertyStub, 0)) {
-        return NULL;
+        return nullptr;
     }
 
     if (!JS_DefineFunctions(cx, Math, math_static_methods))
-        return NULL;
+        return nullptr;
     if (!JS_DefineConstDoubles(cx, Math, math_constants))
-        return NULL;
+        return nullptr;
 
-    MarkStandardClassInitializedNoProto(obj, &MathClass);
+    obj->as<GlobalObject>().markStandardClassInitializedNoProto(&MathClass);
 
     return Math;
 }
