@@ -9,13 +9,12 @@
 
 #include "mozilla/DebugOnly.h"
 #include "mozilla/GuardObjects.h"
-#include "mozilla/HashFunctions.h"
 
 #include <stddef.h>
 
 #include "jsscript.h"
 
-#include "js/Utility.h"
+#include "js/ProfilingStack.h"
 
 /*
  * SPS Profiler integration with the JS Engine
@@ -70,18 +69,18 @@
  *
  * = Native Stack Pointer
  *
- * The actual value pushed as the native pointer is NULL for most JS functions.
- * The reason for this is that there's actually very little correlation between
- * the JS stack and the C++ stack because many JS functions all run in the same
- * C++ frame, or can even go backwards in C++ when going from the JIT back to
- * the interpreter.
+ * The actual value pushed as the native pointer is nullptr for most JS
+ * functions. The reason for this is that there's actually very little
+ * correlation between the JS stack and the C++ stack because many JS functions
+ * all run in the same C++ frame, or can even go backwards in C++ when going
+ * from the JIT back to the interpreter.
  *
- * To alleviate this problem, all JS functions push NULL as their "native stack
- * pointer" to indicate that it's a JS function call. The function RunScript(),
- * however, pushes an actual C++ stack pointer onto the SPS stack. This way when
- * interleaving C++ and JS, if SPS sees a NULL native stack pointer on the SPS
- * stack, it looks backwards for the first non-NULL pointer and uses that for
- * all subsequent NULL native stack pointers.
+ * To alleviate this problem, all JS functions push nullptr as their "native
+ * stack pointer" to indicate that it's a JS function call. The function
+ * RunScript(), however, pushes an actual C++ stack pointer onto the SPS stack.
+ * This way when interleaving C++ and JS, if SPS sees a nullptr native stack
+ * pointer on the SPS stack, it looks backwards for the first non-nullptr
+ * pointer and uses that for all subsequent nullptr native stack pointers.
  *
  * = Line Numbers
  *
@@ -96,16 +95,14 @@
  * JS was far too expensive, so that is why the pc instead of the translated
  * line number is stored.
  *
- * As an invariant, if the pc is NULL, then the JIT is currently executing
+ * As an invariant, if the pc is nullptr, then the JIT is currently executing
  * generated code. Otherwise execution is in another JS function or in C++. With
- * this in place, only the top entry of the stack can ever have NULL as its pc.
- * Additionally with this invariant, it is possible to maintain mappings of JIT
- * code to pc which can be accessed safely because they will only be accessed
- * from a signal handler when the JIT code is executing.
+ * this in place, only the top entry of the stack can ever have nullptr as its
+ * pc. Additionally with this invariant, it is possible to maintain mappings of
+ * JIT code to pc which can be accessed safely because they will only be
+ * accessed from a signal handler when the JIT code is executing.
  */
 
-class JSFunction;
-
 namespace js {
 
 class ProfileEntry;
@@ -154,7 +151,7 @@ class SPSProfiler
 
     /* management of whether instrumentation is on or off */
     bool enabled() { JS_ASSERT_IF(enabled_, installed()); return enabled_; }
-    bool installed() { return stack_ != NULL && size_ != NULL; }
+    bool installed() { return stack_ != nullptr && size_ != nullptr; }
     void enable(bool enabled);
     void enableSlowAssertions(bool enabled) { slowAssertions = enabled; }
     bool slowAssertionsEnabled() { return slowAssertions; }
@@ -182,7 +179,7 @@ class SPSProfiler
     void enterNative(const char *string, void *sp);
     void exitNative() { pop(); }
 
-    jsbytecode *ipToPC(JSScript *script, size_t ip) { return NULL; }
+    jsbytecode *ipToPC(JSScript *script, size_t ip) { return nullptr; }
 
     void setProfilingStack(ProfileEntry *stack, uint32_t *size, uint32_t max);
     const char *profileString(JSContext *cx, JSScript *script, JSFunction *maybeFun);
@@ -233,7 +230,7 @@ class SPSInstrumentation
 {
     /* Because of inline frames, this is a nested structure in a vector */
     struct FrameState {
-        JSScript *script; // script for this frame, NULL if not pushed yet
+        JSScript *script; // script for this frame, nullptr if not pushed yet
         bool skipNext;    // should the next call to reenter be skipped?
         int  left;        // number of leave() calls made without a matching reenter()
     };
@@ -249,7 +246,7 @@ class SPSInstrumentation
      * profiler's stack and constituent fields.
      */
     SPSInstrumentation(SPSProfiler *profiler)
-      : profiler_(profiler), frame(NULL)
+      : profiler_(profiler), frame(nullptr)
     {
         enterInlineFrame();
     }
@@ -264,7 +261,7 @@ class SPSInstrumentation
         if (!enabled())
             return;
         JS_ASSERT(frame->left == 0);
-        JS_ASSERT(frame->script != NULL);
+        JS_ASSERT(frame->script != nullptr);
         frames.shrinkBy(1);
         JS_ASSERT(frames.length() > 0);
         frame = &frames[frames.length() - 1];
@@ -274,12 +271,12 @@ class SPSInstrumentation
     bool enterInlineFrame() {
         if (!enabled())
             return true;
-        JS_ASSERT_IF(frame != NULL, frame->script != NULL);
-        JS_ASSERT_IF(frame != NULL, frame->left == 1);
+        JS_ASSERT_IF(frame != nullptr, frame->script != nullptr);
+        JS_ASSERT_IF(frame != nullptr, frame->left == 1);
         if (!frames.growBy(1))
             return false;
         frame = &frames[frames.length() - 1];
-        frame->script = NULL;
+        frame->script = nullptr;
         frame->skipNext = false;
         frame->left = 0;
         return true;
@@ -293,10 +290,10 @@ class SPSInstrumentation
     /*
      * When debugging or with slow assertions, sometimes a C++ method will be
      * invoked to perform the pop operation from the SPS stack. When we leave
-     * JIT code, we need to record the current PC, but upon reentering JIT code,
-     * no update back to NULL should happen. This method exists to flag this
-     * behavior. The next leave() will emit instrumentation, but the following
-     * reenter() will be a no-op.
+     * JIT code, we need to record the current PC, but upon reentering JIT
+     * code, no update back to nullptr should happen. This method exists to
+     * flag this behavior. The next leave() will emit instrumentation, but the
+     * following reenter() will be a no-op.
      */
     void skipNextReenter() {
         /* If we've left the frame, the reenter will be skipped anyway */
@@ -328,7 +325,7 @@ class SPSInstrumentation
             return true;
         const char *string = profiler_->profileString(cx, script,
                                                       script->function());
-        if (string == NULL)
+        if (string == nullptr)
             return false;
         masm.spsPushFrame(profiler_, string, script, scratch);
         setPushed(script);
@@ -338,7 +335,7 @@ class SPSInstrumentation
     /*
      * Signifies that C++ performed the push() for this function. C++ always
      * sets the current PC to something non-null, however, so as soon as JIT
-     * code is reentered this updates the current pc to NULL.
+     * code is reentered this updates the current pc to nullptr.
      */
     void pushManual(JSScript *script, Assembler &masm, Register scratch) {
         if (!enabled())
