@@ -4,16 +4,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#ifndef Xdr_h___
-#define Xdr_h___
+#ifndef vm_Xdr_h
+#define vm_Xdr_h
 
 #include "mozilla/Endian.h"
 
-#include "jsapi.h"
-#include "jsprvtd.h"
-#include "jsnum.h"
-
-#include "vm/NumericConversions.h"
+#include "jsatom.h"
 
 namespace js {
 
@@ -26,12 +22,12 @@ namespace js {
  * and saved versions. If deserialization fails, the data should be
  * invalidated if possible.
  */
-static const uint32_t XDR_BYTECODE_VERSION = uint32_t(0xb973c0de - 143);
+static const uint32_t XDR_BYTECODE_VERSION = uint32_t(0xb973c0de - 153);
 
 class XDRBuffer {
   public:
     XDRBuffer(JSContext *cx)
-      : context(cx), base(NULL), cursor(NULL), limit(NULL) { }
+      : context(cx), base(nullptr), cursor(nullptr), limit(nullptr) { }
 
     JSContext *cx() const {
         return context;
@@ -67,7 +63,7 @@ class XDRBuffer {
     uint8_t *write(size_t n) {
         if (n > size_t(limit - cursor)) {
             if (!grow(n))
-                return NULL;
+                return nullptr;
         }
         uint8_t *ptr = cursor;
         cursor += n;
@@ -98,11 +94,11 @@ class XDRState {
     XDRBuffer buf;
 
   protected:
-    JSPrincipals *principals;
-    JSPrincipals *originPrincipals;
+    JSPrincipals *principals_;
+    JSPrincipals *originPrincipals_;
 
     XDRState(JSContext *cx)
-      : buf(cx), principals(NULL), originPrincipals(NULL) {
+      : buf(cx), principals_(nullptr), originPrincipals_(nullptr) {
     }
 
   public:
@@ -110,6 +106,10 @@ class XDRState {
         return buf.cx();
     }
 
+    JSPrincipals *originPrincipals() const {
+        return originPrincipals_;
+    }
+
     bool codeUint8(uint8_t *n) {
         if (mode == XDR_ENCODE) {
             uint8_t *ptr = buf.write(sizeof *n);
@@ -210,21 +210,6 @@ class XDRState {
 
     bool codeFunction(JS::MutableHandleObject objp);
     bool codeScript(MutableHandleScript scriptp);
-
-    void initScriptPrincipals(JSScript *script) {
-        JS_ASSERT(mode == XDR_DECODE);
-
-        /* The origin principals must be normalized at this point. */
-        JS_ASSERT_IF(principals, originPrincipals);
-        JS_ASSERT(!script->originPrincipals);
-        if (principals)
-            JS_ASSERT(script->principals() == principals);
-
-        if (originPrincipals) {
-            script->originPrincipals = originPrincipals;
-            JS_HoldPrincipals(originPrincipals);
-        }
-    }
 };
 
 class XDREncoder : public XDRState<XDR_ENCODE> {
@@ -243,7 +228,7 @@ class XDREncoder : public XDRState<XDR_ENCODE> {
 
     void *forgetData(uint32_t *lengthp) {
         void *data = buf.getData(lengthp);
-        buf.setData(NULL, 0);
+        buf.setData(nullptr, 0);
         return data;
     }
 };
@@ -257,4 +242,4 @@ class XDRDecoder : public XDRState<XDR_DECODE> {
 
 } /* namespace js */
 
-#endif /* Xdr_h___ */
+#endif /* vm_Xdr_h */
