@@ -7,10 +7,14 @@
 #ifndef xpcquickstubs_h___
 #define xpcquickstubs_h___
 
-#include "xpcpublic.h"
-#include "xpcprivate.h"
-#include "qsObjectHelper.h"
-#include "mozilla/dom/BindingUtils.h"
+#include "XPCForwards.h"
+
+class qsObjectHelper;
+namespace mozilla {
+namespace dom {
+class NativeProperties;
+}
+}
 
 /* XPCQuickStubs.h - Support functions used only by quick stubs. */
 
@@ -44,7 +48,7 @@ struct xpc_qsHashEntry {
     uint16_t chain;
 };
 
-JSBool
+bool
 xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, unsigned extraFlags,
                        uint32_t ifacec, const nsIID **interfaces,
                        uint32_t tableSize, const xpc_qsHashEntry *table,
@@ -53,7 +57,7 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, unsigned extraFlags,
                        const char *stringTable);
 
 /** Raise an exception on @a cx and return false. */
-JSBool
+bool
 xpc_qsThrow(JSContext *cx, nsresult rv);
 
 /**
@@ -69,14 +73,14 @@ xpc_qsThrow(JSContext *cx, nsresult rv);
  * receives the wrapper JSObject.  (The other reason is to help the caller keep
  * that JSObject GC-reachable.)
  */
-JSBool
+bool
 xpc_qsThrowGetterSetterFailed(JSContext *cx, nsresult rv,
                               JSObject *obj, jsid memberId);
 // And variants using strings and string tables
-JSBool
+bool
 xpc_qsThrowGetterSetterFailed(JSContext *cx, nsresult rv,
                               JSObject *obj, const char* memberName);
-JSBool
+bool
 xpc_qsThrowGetterSetterFailed(JSContext *cx, nsresult rv,
                               JSObject *obj, uint16_t memberIndex);
 
@@ -85,10 +89,10 @@ xpc_qsThrowGetterSetterFailed(JSContext *cx, nsresult rv,
  *
  * See NOTE at xpc_qsThrowGetterSetterFailed.
  */
-JSBool
+bool
 xpc_qsThrowMethodFailed(JSContext *cx, nsresult rv, jsval *vp);
 
-JSBool
+bool
 xpc_qsThrowMethodFailedWithCcx(XPCCallContext &ccx, nsresult rv);
 
 bool
@@ -128,25 +132,26 @@ xpc_qsThrowBadSetterValue(JSContext *cx, nsresult rv, JSObject *obj,
                           uint16_t name_index);
 
 
-JSBool
-xpc_qsGetterOnlyPropertyStub(JSContext *cx, JSHandleObject obj, JSHandleId id, JSBool strict, JSMutableHandleValue vp);
+bool
+xpc_qsGetterOnlyPropertyStub(JSContext *cx, JS::HandleObject obj, JS::HandleId id,
+                             bool strict, JS::MutableHandleValue vp);
 
-JSBool
+bool
 xpc_qsGetterOnlyNativeStub(JSContext *cx, unsigned argc, jsval *vp);
 
 /* Functions for converting values between COM and JS. */
 
-inline JSBool
-xpc_qsInt64ToJsval(JSContext *cx, int64_t i, jsval *rv)
+inline bool
+xpc_qsInt64ToJsval(JSContext *cx, int64_t i, JS::MutableHandleValue rv)
 {
-    *rv = JS_NumberValue(static_cast<double>(i));
+    rv.setNumber(static_cast<double>(i));
     return true;
 }
 
-inline JSBool
-xpc_qsUint64ToJsval(JSContext *cx, uint64_t u, jsval *rv)
+inline bool
+xpc_qsUint64ToJsval(JSContext *cx, uint64_t u, JS::MutableHandleValue rv)
 {
-    *rv = JS_NumberValue(static_cast<double>(u));
+    rv.setNumber(static_cast<double>(u));
     return true;
 }
 
@@ -166,7 +171,7 @@ public:
             Ptr()->~implementation_type();
     }
 
-    JSBool IsValid() const { return mValid; }
+    bool IsValid() const { return mValid; }
 
     implementation_type *Ptr()
     {
@@ -219,7 +224,7 @@ protected:
      * stored in mBuf, if mValid is true.
      */
     void *mBuf[JS_HOWMANY(sizeof(implementation_type), sizeof(void *))];
-    JSBool mValid;
+    bool mValid;
 
     /*
      * If null is returned, then we either failed or fully initialized
@@ -230,7 +235,9 @@ protected:
      * when |v| is JSVAL_IS_NULL and JSVAL_IS_VOID respectively.
      */
     template<class traits>
-    JSString* InitOrStringify(JSContext* cx, jsval v, jsval* pval,
+    JSString* InitOrStringify(JSContext* cx, jsval v,
+                              JS::MutableHandleValue pval,
+                              bool notpassed,
                               StringificationBehavior nullBehavior,
                               StringificationBehavior undefinedBehavior) {
         JSString *s;
@@ -247,8 +254,8 @@ protected:
             // If pval is null, that means the argument was optional and
             // not passed; turn those into void strings if they're
             // supposed to be stringified.
-            if (behavior != eStringify || !pval) {
-                // Here behavior == eStringify implies !pval, so both eNull and
+            if (behavior != eStringify || notpassed) {
+                // Here behavior == eStringify implies notpassed, so both eNull and
                 // eStringify should end up with void strings.
                 (new(mBuf) implementation_type(traits::sEmptyBuffer, uint32_t(0)))->
                     SetIsVoid(behavior != eEmpty);
@@ -261,7 +268,7 @@ protected:
                 mValid = false;
                 return nullptr;
             }
-            *pval = STRING_TO_JSVAL(s);  // Root the new string.
+            pval.setString(s);  // Root the new string.
         }
 
         return s;
@@ -285,7 +292,8 @@ protected:
 class xpc_qsDOMString : public xpc_qsBasicString<nsAString, nsDependentString>
 {
 public:
-    xpc_qsDOMString(JSContext *cx, jsval v, jsval *pval,
+    xpc_qsDOMString(JSContext *cx, JS::HandleValue v,
+                    JS::MutableHandleValue pval, bool notpassed,
                     StringificationBehavior nullBehavior,
                     StringificationBehavior undefinedBehavior);
 };
@@ -297,8 +305,9 @@ public:
 class xpc_qsAString : public xpc_qsDOMString
 {
 public:
-    xpc_qsAString(JSContext *cx, jsval v, jsval *pval)
-        : xpc_qsDOMString(cx, v, pval, eNull, eNull)
+    xpc_qsAString(JSContext *cx, JS::HandleValue v,
+                  JS::MutableHandleValue pval, bool notpassed)
+        : xpc_qsDOMString(cx, v, pval, notpassed, eNull, eNull)
     {}
 };
 
@@ -309,9 +318,10 @@ public:
 class xpc_qsACString : public xpc_qsBasicString<nsACString, nsCString>
 {
 public:
-    xpc_qsACString(JSContext *cx, jsval v, jsval *pval,
-                   StringificationBehavior nullBehavior = eNull,
-                   StringificationBehavior undefinedBehavior = eNull);
+    xpc_qsACString(JSContext *cx, JS::HandleValue v,
+                   JS::MutableHandleValue pval, bool notpassed,
+                   StringificationBehavior nullBehavior,
+                   StringificationBehavior undefinedBehavior);
 };
 
 /**
@@ -321,7 +331,8 @@ class xpc_qsAUTF8String :
   public xpc_qsBasicString<nsACString, NS_ConvertUTF16toUTF8>
 {
 public:
-  xpc_qsAUTF8String(JSContext* cx, jsval v, jsval *pval);
+  xpc_qsAUTF8String(JSContext* cx, JS::HandleValue v,
+                    JS::MutableHandleValue pval, bool notpassed);
 };
 
 struct xpc_qsSelfRef
@@ -344,15 +355,15 @@ struct xpc_qsSelfRef
  *     Out. On success it receives the converted string unless v is null or
  *     undefinedin which case bytes->ptr() remains null.
  */
-JSBool
+bool
 xpc_qsJsvalToCharStr(JSContext *cx, jsval v, JSAutoByteString *bytes);
 
-JSBool
-xpc_qsJsvalToWcharStr(JSContext *cx, jsval v, jsval *pval, const PRUnichar **pstr);
+bool
+xpc_qsJsvalToWcharStr(JSContext *cx, jsval v, JS::MutableHandleValue pval, const PRUnichar **pstr);
 
 
 /** Convert an nsString to JSString, returning true on success. This will sometimes modify |str| to be empty. */
-JSBool
+bool
 xpc_qsStringToJsstring(JSContext *cx, nsString &str, JSString **rval);
 
 nsresult
@@ -370,8 +381,7 @@ castNative(JSContext *cx,
            const nsIID &iid,
            void **ppThis,
            nsISupports **ppThisRef,
-           jsval *vp,
-           XPCLazyCallContext *lccx);
+           JS::MutableHandleValue vp);
 
 /**
  * Search @a obj and its prototype chain for an XPCOM object that implements
@@ -390,13 +400,12 @@ castNative(JSContext *cx,
  * Requires a request on @a cx.
  */
 template <class T>
-inline JSBool
+inline bool
 xpc_qsUnwrapThis(JSContext *cx,
                  JS::HandleObject obj,
                  T **ppThis,
                  nsISupports **pThisRef,
-                 jsval *pThisVal,
-                 XPCLazyCallContext *lccx,
+                 JS::MutableHandleValue pThisVal,
                  bool failureFatal = true)
 {
     XPCWrappedNative *wrapper;
@@ -405,8 +414,7 @@ xpc_qsUnwrapThis(JSContext *cx,
     nsresult rv = getWrapper(cx, obj, &wrapper, current.address(), &tearoff);
     if (NS_SUCCEEDED(rv))
         rv = castNative(cx, wrapper, current, tearoff, NS_GET_TEMPLATE_IID(T),
-                        reinterpret_cast<void **>(ppThis), pThisRef, pThisVal,
-                        lccx);
+                        reinterpret_cast<void **>(ppThis), pThisRef, pThisVal);
 
     if (failureFatal)
         return NS_SUCCEEDED(rv) || xpc_qsThrow(cx, rv);
@@ -416,99 +424,33 @@ xpc_qsUnwrapThis(JSContext *cx,
     return true;
 }
 
-MOZ_ALWAYS_INLINE bool
-HasBitInInterfacesBitmap(JSObject *obj, uint32_t interfaceBit)
-{
-    NS_ASSERTION(IS_WRAPPER_CLASS(js::GetObjectClass(obj)), "Not a wrapper?");
-
-    XPCWrappedNativeJSClass *clasp =
-      (XPCWrappedNativeJSClass*)js::GetObjectClass(obj);
-    return (clasp->interfacesBitmap & (1 << interfaceBit)) != 0;
-}
-
-MOZ_ALWAYS_INLINE nsISupports*
+nsISupports*
 castNativeFromWrapper(JSContext *cx,
                       JSObject *obj,
                       uint32_t interfaceBit,
                       uint32_t protoID,
                       int32_t protoDepth,
                       nsISupports **pRef,
-                      jsval *pVal,
-                      XPCLazyCallContext *lccx,
-                      nsresult *rv)
-{
-    XPCWrappedNative *wrapper;
-    XPCWrappedNativeTearOff *tearoff;
-    JSObject *cur;
+                      JS::MutableHandleValue pVal,
+                      nsresult *rv);
 
-    if (IS_WRAPPER_CLASS(js::GetObjectClass(obj))) {
-        cur = obj;
-        wrapper = IS_WN_WRAPPER_OBJECT(cur) ?
-                  (XPCWrappedNative*)xpc_GetJSPrivate(obj) :
-                  nullptr;
-        tearoff = nullptr;
-    } else {
-        *rv = getWrapper(cx, obj, &wrapper, &cur, &tearoff);
-        if (NS_FAILED(*rv))
-            return nullptr;
-    }
-
-    nsISupports *native;
-    if (wrapper) {
-        native = wrapper->GetIdentityObject();
-        cur = wrapper->GetFlatJSObject();
-        if (!native || !HasBitInInterfacesBitmap(cur, interfaceBit)) {
-            native = nullptr;
-        } else if (lccx) {
-            lccx->SetWrapper(wrapper, tearoff);
-        }
-    } else if (cur && IS_SLIM_WRAPPER(cur)) {
-        native = static_cast<nsISupports*>(xpc_GetJSPrivate(cur));
-        if (!native || !HasBitInInterfacesBitmap(cur, interfaceBit)) {
-            native = nullptr;
-        } else if (lccx) {
-            lccx->SetWrapper(cur);
-        }
-    } else if (cur && protoDepth >= 0) {
-        const mozilla::dom::DOMClass* domClass =
-            mozilla::dom::GetDOMClass(cur);
-        native = mozilla::dom::UnwrapDOMObject<nsISupports>(cur);
-        if (native &&
-            (uint32_t)domClass->mInterfaceChain[protoDepth] != protoID) {
-            native = nullptr;
-        }
-    } else {
-        native = nullptr;
-    }
-
-    if (native) {
-        *pRef = nullptr;
-        *pVal = OBJECT_TO_JSVAL(cur);
-        *rv = NS_OK;
-    } else {
-        *rv = NS_ERROR_XPC_BAD_CONVERT_JS;
-    }
-
-    return native;
-}
-
-JSBool
+bool
 xpc_qsUnwrapThisFromCcxImpl(XPCCallContext &ccx,
                             const nsIID &iid,
                             void **ppThis,
                             nsISupports **pThisRef,
-                            jsval *vp);
+                            JS::MutableHandleValue vp);
 
 /**
  * Alternate implementation of xpc_qsUnwrapThis using information already
  * present in the given XPCCallContext.
  */
 template <class T>
-inline JSBool
+inline bool
 xpc_qsUnwrapThisFromCcx(XPCCallContext &ccx,
                         T **ppThis,
                         nsISupports **pThisRef,
-                        jsval *pThisVal)
+                        JS::MutableHandleValue pThisVal)
 {
     return xpc_qsUnwrapThisFromCcxImpl(ccx,
                                        NS_GET_TEMPLATE_IID(T),
@@ -536,14 +478,14 @@ xpc_qsUnwrapObj(jsval v, nsISupports **ppArgRef, nsresult *rv)
 }
 
 nsresult
-xpc_qsUnwrapArgImpl(JSContext *cx, jsval v, const nsIID &iid, void **ppArg,
-                    nsISupports **ppArgRef, jsval *vp);
+xpc_qsUnwrapArgImpl(JSContext *cx, JS::HandleValue v, const nsIID &iid, void **ppArg,
+                    nsISupports **ppArgRef, JS::MutableHandleValue vp);
 
 /** Convert a jsval to an XPCOM pointer. */
 template <class Interface, class StrongRefType>
 inline nsresult
-xpc_qsUnwrapArg(JSContext *cx, jsval v, Interface **ppArg,
-                StrongRefType **ppArgRef, jsval *vp)
+xpc_qsUnwrapArg(JSContext *cx, JS::HandleValue v, Interface **ppArg,
+                StrongRefType **ppArgRef, JS::MutableHandleValue vp)
 {
     nsISupports* argRef = *ppArgRef;
     nsresult rv = xpc_qsUnwrapArgImpl(cx, v, NS_GET_TEMPLATE_IID(Interface),
@@ -560,15 +502,14 @@ castNativeArgFromWrapper(JSContext *cx,
                          uint32_t protoID,
                          int32_t protoDepth,
                          nsISupports **pArgRef,
-                         jsval *vp,
+                         JS::MutableHandleValue vp,
                          nsresult *rv)
 {
     JSObject *src = xpc_qsUnwrapObj(v, pArgRef, rv);
     if (!src)
         return nullptr;
 
-    return castNativeFromWrapper(cx, src, bit, protoID, protoDepth, pArgRef, vp,
-                                 nullptr, rv);
+    return castNativeFromWrapper(cx, src, bit, protoID, protoDepth, pArgRef, vp, rv);
 }
 
 inline nsWrapperCache*
@@ -593,20 +534,20 @@ xpc_qsGetWrapperCache(void *p)
  * aIdentity is a performance optimization. Set it to true,
  * only if p is the identity pointer.
  */
-JSBool
-xpc_qsXPCOMObjectToJsval(XPCLazyCallContext &lccx,
+bool
+xpc_qsXPCOMObjectToJsval(JSContext *aCx,
                          qsObjectHelper &aHelper,
                          const nsIID *iid,
                          XPCNativeInterface **iface,
-                         jsval *rval);
+                         JS::MutableHandleValue rval);
 
 /**
  * Convert a variant to jsval. Return true on success.
  */
-JSBool
-xpc_qsVariantToJsval(XPCLazyCallContext &ccx,
+bool
+xpc_qsVariantToJsval(JSContext *cx,
                      nsIVariant *p,
-                     jsval *rval);
+                     JS::MutableHandleValue rval);
 
 #ifdef DEBUG
 void
@@ -637,25 +578,27 @@ xpc_qsSameResult(int32_t result1, int32_t result2)
 
 // Apply |op| to |obj|, |id|, and |vp|. If |op| is a setter, treat the assignment as lenient.
 template<typename Op>
-inline JSBool ApplyPropertyOp(JSContext *cx, Op op, JSHandleObject obj, JSHandleId id, JSMutableHandleValue vp);
+inline bool ApplyPropertyOp(JSContext *cx, Op op, JS::HandleObject obj, JS::HandleId id,
+                              JS::MutableHandleValue vp);
 
 template<>
-inline JSBool
-ApplyPropertyOp<JSPropertyOp>(JSContext *cx, JSPropertyOp op, JSHandleObject obj, JSHandleId id, JSMutableHandleValue vp)
+inline bool
+ApplyPropertyOp<JSPropertyOp>(JSContext *cx, JSPropertyOp op, JS::HandleObject obj, JS::HandleId id,
+                              JS::MutableHandleValue vp)
 {
     return op(cx, obj, id, vp);
 }
 
 template<>
-inline JSBool
-ApplyPropertyOp<JSStrictPropertyOp>(JSContext *cx, JSStrictPropertyOp op, JSHandleObject obj,
-                                    JSHandleId id, JSMutableHandleValue vp)
+inline bool
+ApplyPropertyOp<JSStrictPropertyOp>(JSContext *cx, JSStrictPropertyOp op, JS::HandleObject obj,
+                                    JS::HandleId id, JS::MutableHandleValue vp)
 {
     return op(cx, obj, id, true, vp);
 }
 
 template<typename Op>
-JSBool
+bool
 PropertyOpForwarder(JSContext *cx, unsigned argc, jsval *vp)
 {
     // Layout:
@@ -685,7 +628,7 @@ PropertyOpForwarder(JSContext *cx, unsigned argc, jsval *vp)
     return ApplyPropertyOp<Op>(cx, *popp, obj, id, args.rval());
 }
 
-extern JSClass PointerHolderClass;
+extern const JSClass PointerHolderClass;
 
 template<typename Op>
 JSObject *
