@@ -12,7 +12,6 @@
 
 #include <cstdlib>  // srand
 
-#include "webrtc/modules/pacing/include/paced_sender.h"
 #include "webrtc/modules/rtp_rtcp/source/rtp_packet_history.h"
 #include "webrtc/modules/rtp_rtcp/source/rtp_sender_audio.h"
 #include "webrtc/modules/rtp_rtcp/source/rtp_sender_video.h"
@@ -22,6 +21,9 @@
 
 namespace webrtc {
 
+// Max in the RFC 3550 is 255 bytes, we limit it to be modulus 32 for SRTP.
+const int kMaxPaddingLength = 224;
+
 namespace {
 
 const char* FrameTypeToString(const FrameType frame_type) {
@@ -49,7 +51,7 @@ RTPSender::RTPSender(const int32_t id, const bool audio, Clock *clock,
       max_payload_length_(IP_PACKET_SIZE - 28),     // Default is IP-v4/UDP.
       target_send_bitrate_(0), packet_over_head_(28), payload_type_(-1),
       payload_type_map_(), rtp_header_extension_map_(),
-      transmission_time_offset_(0),
+      transmission_time_offset_(0), absolute_send_time_(0),
       // NACK.
       nack_byte_count_times_(), nack_byte_count_(), nack_bitrate_(clock),
       packet_history_(new RTPPacketHistory(clock)),
@@ -57,11 +59,12 @@ RTPSender::RTPSender(const int32_t id, const bool audio, Clock *clock,
       packets_sent_(0), payload_bytes_sent_(0), start_time_stamp_forced_(false),
       start_time_stamp_(0), ssrc_db_(*SSRCDatabase::GetSSRCDatabase()),
       remote_ssrc_(0), sequence_number_forced_(false), ssrc_forced_(false),
-      time_stamp_(0), csrcs_(0), csrc_(), include_csrcs_(true),
+      timestamp_(0), capture_time_ms_(0), last_packet_marker_bit_(false),
+      num_csrcs_(0), csrcs_(), include_csrcs_(true),
       rtx_(kRtxOff), payload_type_rtx_(-1) {
   memset(nack_byte_count_times_, 0, sizeof(nack_byte_count_times_));
   memset(nack_byte_count_, 0, sizeof(nack_byte_count_));
-  memset(csrc_, 0, sizeof(csrc_));
+  memset(csrcs_, 0, sizeof(csrcs_));
   // We need to seed the random generator.
   srand(static_cast<uint32_t>(clock_->TimeInMilliseconds()));
   ssrc_ = ssrc_db_.CreateSSRC();  // Can't be 0.
@@ -137,6 +140,16 @@ int32_t RTPSender::SetTransmissionTimeOffset(
   return 0;
 }
 
+int32_t RTPSender::SetAbsoluteSendTime(
+    const uint32_t absolute_send_time) {
+  if (absolute_send_time > 0xffffff) {  // UWord24.
+    return -1;
+  }
+  CriticalSectionScoped cs(send_critsect_);
+  absolute_send_time_ = absolute_send_time;
+  return 0;
+}
+
 int32_t RTPSender::RegisterRtpHeaderExtension(const RTPExtensionType type,
                                               const uint8_t id) {
   CriticalSectionScoped cs(send_critsect_);
@@ -330,9 +343,6 @@ int32_t RTPSender::SendOutgoingData(
     const uint8_t *payload_data, const uint32_t payload_size,
     const RTPFragmentationHeader *fragmentation,
     VideoCodecInformation *codec_info, const RTPVideoTypeHeader *rtp_type_hdr) {
-  TRACE_EVENT2("webrtc_rtp", "RTPSender::SendOutgoingData",
-               "timestsamp", capture_timestamp,
-               "frame_type", FrameTypeToString(frame_type));
   {
     // Drop this packet if we're not sending media packets.
     CriticalSectionScoped cs(send_critsect_);
@@ -348,6 +358,15 @@ int32_t RTPSender::SendOutgoingData(
     return -1;
   }
 
+  if (frame_type == kVideoFrameKey) {
+    TRACE_EVENT_INSTANT1("webrtc_rtp", "SendKeyFrame",
+                         "timestamp", capture_timestamp);
+  } else {
+    TRACE_EVENT_INSTANT2("webrtc_rtp", "SendFrame",
+                         "timestamp", capture_timestamp,
+                         "frame_type", FrameTypeToString(frame_type));
+  }
+
   if (audio_configured_) {
     assert(frame_type == kAudioFrameSpeech || frame_type == kAudioFrameCN ||
            frame_type == kFrameEmpty);
@@ -358,8 +377,12 @@ int32_t RTPSender::SendOutgoingData(
     assert(frame_type != kAudioFrameSpeech && frame_type != kAudioFrameCN);
 
     if (frame_type == kFrameEmpty) {
+      if (paced_sender_->Enabled()) {
+        // Padding is driven by the pacer and not by the encoder.
+        return 0;
+      }
       return SendPaddingAccordingToBitrate(payload_type, capture_timestamp,
-                                           capture_time_ms);
+                                           capture_time_ms) ? 0 : -1;
     }
     return video_->SendVideo(video_type, frame_type, payload_type,
                              capture_timestamp, capture_time_ms, payload_data,
@@ -368,7 +391,7 @@ int32_t RTPSender::SendOutgoingData(
   }
 }
 
-int32_t RTPSender::SendPaddingAccordingToBitrate(
+bool RTPSender::SendPaddingAccordingToBitrate(
     int8_t payload_type, uint32_t capture_timestamp,
     int64_t capture_time_ms) {
   // Current bitrate since last estimate(1 second) averaged with the
@@ -376,7 +399,7 @@ int32_t RTPSender::SendPaddingAccordingToBitrate(
   uint32_t current_bitrate = BitrateNow();
   int bitrate_diff = target_send_bitrate_ * 1000 - current_bitrate;
   if (bitrate_diff <= 0) {
-    return 0;
+    return true;
   }
   int bytes = 0;
   if (current_bitrate == 0) {
@@ -390,57 +413,89 @@ int32_t RTPSender::SendPaddingAccordingToBitrate(
       bytes = bytes_cap;
     }
   }
-  return SendPadData(payload_type, capture_timestamp, capture_time_ms, bytes);
+  int bytes_sent = SendPadData(payload_type, capture_timestamp, capture_time_ms,
+                               bytes, kDontRetransmit, false);
+  // We did not manage to send all bytes. Comparing with 31 due to modulus 32.
+  return bytes - bytes_sent < 31;
 }
 
-int32_t RTPSender::SendPadData(
-    int8_t payload_type, uint32_t capture_timestamp,
-    int64_t capture_time_ms, int32_t bytes) {
+int RTPSender::BuildPaddingPacket(uint8_t* packet, int header_length,
+                                  int32_t bytes) {
+  int padding_bytes_in_packet = kMaxPaddingLength;
+  if (bytes < kMaxPaddingLength) {
+    padding_bytes_in_packet = bytes;
+  }
+  packet[0] |= 0x20;  // Set padding bit.
+  int32_t *data =
+      reinterpret_cast<int32_t *>(&(packet[header_length]));
+
+  // Fill data buffer with random data.
+  for (int j = 0; j < (padding_bytes_in_packet >> 2); ++j) {
+    data[j] = rand();  // NOLINT
+  }
+  // Set number of padding bytes in the last byte of the packet.
+  packet[header_length + padding_bytes_in_packet - 1] = padding_bytes_in_packet;
+  return padding_bytes_in_packet;
+}
+
+int RTPSender::SendPadData(int payload_type, uint32_t timestamp,
+                           int64_t capture_time_ms, int32_t bytes,
+                           StorageType store, bool force_full_size_packets) {
   // Drop this packet if we're not sending media packets.
   if (!sending_media_) {
-    return 0;
+    return bytes;
   }
-  // Max in the RFC 3550 is 255 bytes, we limit it to be modulus 32 for SRTP.
-  int max_length = 224;
-  uint8_t data_buffer[IP_PACKET_SIZE];
-
-  for (; bytes > 0; bytes -= max_length) {
-    int padding_bytes_in_packet = max_length;
-    if (bytes < max_length) {
-      padding_bytes_in_packet = (bytes + 16) & 0xffe0;  // Keep our modulus 32.
+  int padding_bytes_in_packet = 0;
+  int bytes_sent = 0;
+  for (; bytes > 0; bytes -= padding_bytes_in_packet) {
+    // Always send full padding packets.
+    if (force_full_size_packets && bytes < kMaxPaddingLength)
+      bytes = kMaxPaddingLength;
+    if (bytes < kMaxPaddingLength) {
+      if (force_full_size_packets) {
+        bytes = kMaxPaddingLength;
+      } else {
+        // Round to the nearest multiple of 32.
+        bytes = (bytes + 16) & 0xffe0;
+      }
     }
-    if (padding_bytes_in_packet < 32) {
+    if (bytes < 32) {
       // Sanity don't send empty packets.
       break;
     }
-    // Correct seq num, timestamp and payload type.
-    int header_length = BuildRTPheader(
-                            data_buffer, payload_type, false,  // No markerbit.
-                            capture_timestamp, true,  // Timestamp provided.
-                            true);  // Increment sequence number.
-    data_buffer[0] |= 0x20;  // Set padding bit.
-    int32_t *data =
-        reinterpret_cast<int32_t *>(&(data_buffer[header_length]));
-
-    // Fill data buffer with random data.
-    for (int j = 0; j < (padding_bytes_in_packet >> 2); ++j) {
-      data[j] = rand();  // NOLINT
+    uint32_t ssrc;
+    uint16_t sequence_number;
+    {
+      CriticalSectionScoped cs(send_critsect_);
+      // Only send padding packets following the last packet of a frame,
+      // indicated by the marker bit.
+      if (!last_packet_marker_bit_)
+        return bytes_sent;
+      if (rtx_ == kRtxOff) {
+        ssrc = ssrc_;
+        sequence_number = sequence_number_;
+        ++sequence_number_;
+      } else {
+        ssrc = ssrc_rtx_;
+        sequence_number = sequence_number_rtx_;
+        ++sequence_number_rtx_;
+      }
     }
-    // Set number of padding bytes in the last byte of the packet.
-    data_buffer[header_length + padding_bytes_in_packet - 1] =
-        padding_bytes_in_packet;
-    // Send the packet.
-    if (0 > SendToNetwork(data_buffer, padding_bytes_in_packet, header_length,
-                          capture_time_ms, kDontRetransmit)) {
+    uint8_t padding_packet[IP_PACKET_SIZE];
+    int header_length = CreateRTPHeader(padding_packet, payload_type, ssrc,
+                                        false, timestamp, sequence_number, NULL,
+                                        0);
+    padding_bytes_in_packet = BuildPaddingPacket(padding_packet, header_length,
+                                                 bytes);
+    if (0 > SendToNetwork(padding_packet, padding_bytes_in_packet,
+                          header_length, capture_time_ms, store,
+                          PacedSender::kLowPriority)) {
       // Error sending the packet.
       break;
     }
+    bytes_sent += padding_bytes_in_packet;
   }
-  if (bytes > 31) {  // 31 due to our modulus 32.
-    // We did not manage to send all bytes.
-    return -1;
-  }
-  return 0;
+  return bytes_sent;
 }
 
 void RTPSender::SetStorePacketsStatus(const bool enable,
@@ -476,8 +531,8 @@ int32_t RTPSender::ReSendPacket(uint16_t packet_id, uint32_t min_resend_time) {
   }
 
   ModuleRTPUtility::RTPHeaderParser rtp_parser(data_buffer, length);
-  WebRtcRTPHeader rtp_header;
-  rtp_parser.Parse(rtp_header);
+  RTPHeader header;
+  rtp_parser.Parse(header);
 
   // Store the time when the packet was last sent or added to pacer.
   packet_history_->UpdateResendTime(packet_id);
@@ -492,23 +547,23 @@ int32_t RTPSender::ReSendPacket(uint16_t packet_id, uint32_t min_resend_time) {
   }
 
   TRACE_EVENT_INSTANT2("webrtc_rtp", "RTPSender::ReSendPacket",
-                       "timestamp", rtp_header.header.timestamp,
-                       "seqnum", rtp_header.header.sequenceNumber);
+                       "timestamp", header.timestamp,
+                       "seqnum", header.sequenceNumber);
 
   if (paced_sender_) {
     if (!paced_sender_->SendPacket(PacedSender::kHighPriority,
-                                   rtp_header.header.ssrc,
-                                   rtp_header.header.sequenceNumber,
+                                   header.ssrc,
+                                   header.sequenceNumber,
                                    capture_time_ms,
-                                   length)) {
+                                   length - header.headerLength)) {
       // We can't send the packet right now.
       // We will be called when it is time.
-      return 0;
+      return length;
     }
   }
 
   if (SendPacketToNetwork(buffer_to_send_ptr, length)) {
-    return 0;
+    return length;
   }
   return -1;
 }
@@ -646,7 +701,7 @@ void RTPSender::UpdateNACKBitRate(const uint32_t bytes,
 }
 
 // Called from pacer when we can send the packet.
-void RTPSender::TimeToSendPacket(uint16_t sequence_number,
+bool RTPSender::TimeToSendPacket(uint16_t sequence_number,
                                  int64_t capture_time_ms) {
   StorageType type;
   uint16_t length = IP_PACKET_SIZE;
@@ -654,48 +709,78 @@ void RTPSender::TimeToSendPacket(uint16_t sequence_number,
   int64_t stored_time_ms;
 
   if (packet_history_ == NULL) {
-    return;
+    // Packet cannot be found. Allow sending to continue.
+    return true;
   }
   if (!packet_history_->GetRTPPacket(sequence_number, 0, data_buffer, &length,
                                      &stored_time_ms, &type)) {
-    return;
+    // Packet cannot be found. Allow sending to continue.
+    return true;
   }
   assert(length > 0);
 
   ModuleRTPUtility::RTPHeaderParser rtp_parser(data_buffer, length);
-  WebRtcRTPHeader rtp_header;
+  RTPHeader rtp_header;
   rtp_parser.Parse(rtp_header);
   TRACE_EVENT_INSTANT2("webrtc_rtp", "RTPSender::TimeToSendPacket",
-                       "timestamp", rtp_header.header.timestamp,
+                       "timestamp", rtp_header.timestamp,
                        "seqnum", sequence_number);
 
-  int64_t diff_ms = clock_->TimeInMilliseconds() - capture_time_ms;
-  if (UpdateTransmissionTimeOffset(data_buffer, length, rtp_header, diff_ms)) {
+  int64_t now_ms = clock_->TimeInMilliseconds();
+  int64_t diff_ms = now_ms - capture_time_ms;
+  bool updated_transmission_time_offset =
+      UpdateTransmissionTimeOffset(data_buffer, length, rtp_header, diff_ms);
+  bool updated_abs_send_time =
+      UpdateAbsoluteSendTime(data_buffer, length, rtp_header, now_ms);
+  if (updated_transmission_time_offset || updated_abs_send_time) {
     // Update stored packet in case of receiving a re-transmission request.
     packet_history_->ReplaceRTPHeader(data_buffer,
-                                      rtp_header.header.sequenceNumber,
-                                      rtp_header.header.headerLength);
+                                      rtp_header.sequenceNumber,
+                                      rtp_header.headerLength);
   }
-  SendPacketToNetwork(data_buffer, length);
+  return SendPacketToNetwork(data_buffer, length);
+}
+
+int RTPSender::TimeToSendPadding(int bytes) {
+  if (!sending_media_) {
+    return 0;
+  }
+  int payload_type;
+  int64_t capture_time_ms;
+  uint32_t timestamp;
+  {
+    CriticalSectionScoped cs(send_critsect_);
+    payload_type = (rtx_ == kRtxOff) ? payload_type_ : payload_type_rtx_;
+    timestamp = timestamp_;
+    capture_time_ms = capture_time_ms_;
+  }
+  return SendPadData(payload_type, timestamp, capture_time_ms, bytes,
+                     kDontStore, true);
 }
 
 // TODO(pwestin): send in the RTPHeaderParser to avoid parsing it again.
 int32_t RTPSender::SendToNetwork(
     uint8_t *buffer, int payload_length, int rtp_header_length,
-    int64_t capture_time_ms, StorageType storage) {
+    int64_t capture_time_ms, StorageType storage,
+    PacedSender::Priority priority) {
   ModuleRTPUtility::RTPHeaderParser rtp_parser(
       buffer, payload_length + rtp_header_length);
-  WebRtcRTPHeader rtp_header;
+  RTPHeader rtp_header;
   rtp_parser.Parse(rtp_header);
 
+  int64_t now_ms = clock_->TimeInMilliseconds();
+
   // |capture_time_ms| <= 0 is considered invalid.
   // TODO(holmer): This should be changed all over Video Engine so that negative
   // time is consider invalid, while 0 is considered a valid time.
   if (capture_time_ms > 0) {
-    int64_t time_now = clock_->TimeInMilliseconds();
     UpdateTransmissionTimeOffset(buffer, payload_length + rtp_header_length,
-                                 rtp_header, time_now - capture_time_ms);
+                                 rtp_header, now_ms - capture_time_ms);
   }
+
+  UpdateAbsoluteSendTime(buffer, payload_length + rtp_header_length,
+                         rtp_header, now_ms);
+
   // Used for NACK and to spread out the transmission of packets.
   if (packet_history_->PutRTPPacket(buffer, rtp_header_length + payload_length,
                                     max_payload_length_, capture_time_ms,
@@ -727,10 +812,9 @@ int32_t RTPSender::SendToNetwork(
   }
 
   if (paced_sender_ && storage != kDontStore) {
-    if (!paced_sender_->SendPacket(
-        PacedSender::kNormalPriority, rtp_header.header.ssrc,
-        rtp_header.header.sequenceNumber, capture_time_ms,
-        payload_length + rtp_header_length)) {
+    if (!paced_sender_->SendPacket(priority, rtp_header.ssrc,
+                                   rtp_header.sequenceNumber, capture_time_ms,
+                                   payload_length)) {
       // We can't send the packet right now.
       // We will be called when it is time.
       return 0;
@@ -755,7 +839,7 @@ void RTPSender::ProcessBitrate() {
 uint16_t RTPSender::RTPHeaderLength() const {
   uint16_t rtp_header_length = 12;
   if (include_csrcs_) {
-    rtp_header_length += sizeof(uint32_t) * csrcs_;
+    rtp_header_length += sizeof(uint32_t) * num_csrcs_;
   }
   rtp_header_length += RtpHeaderExtensionTotalLength();
   return rtp_header_length;
@@ -782,60 +866,73 @@ uint32_t RTPSender::Bytes() const {
   return payload_bytes_sent_;
 }
 
-int32_t RTPSender::BuildRTPheader(
-    uint8_t *data_buffer, const int8_t payload_type,
-    const bool marker_bit, const uint32_t capture_time_stamp,
-    const bool time_stamp_provided, const bool inc_sequence_number) {
-  assert(payload_type >= 0);
-  CriticalSectionScoped cs(send_critsect_);
-
-  data_buffer[0] = static_cast<uint8_t>(0x80);  // version 2.
-  data_buffer[1] = static_cast<uint8_t>(payload_type);
+int RTPSender::CreateRTPHeader(
+    uint8_t* header, int8_t payload_type, uint32_t ssrc, bool marker_bit,
+    uint32_t timestamp, uint16_t sequence_number, const uint32_t* csrcs,
+    uint8_t num_csrcs) const {
+  header[0] = 0x80;  // version 2.
+  header[1] = static_cast<uint8_t>(payload_type);
   if (marker_bit) {
-    data_buffer[1] |= kRtpMarkerBitMask;  // Marker bit is set.
+    header[1] |= kRtpMarkerBitMask;  // Marker bit is set.
   }
-  if (time_stamp_provided) {
-    time_stamp_ = start_time_stamp_ + capture_time_stamp;
-  } else {
-    // Make a unique time stamp.
-    // We can't inc by the actual time, since then we increase the risk of back
-    // timing.
-    time_stamp_++;
-  }
-  ModuleRTPUtility::AssignUWord16ToBuffer(data_buffer + 2, sequence_number_);
-  ModuleRTPUtility::AssignUWord32ToBuffer(data_buffer + 4, time_stamp_);
-  ModuleRTPUtility::AssignUWord32ToBuffer(data_buffer + 8, ssrc_);
+  ModuleRTPUtility::AssignUWord16ToBuffer(header + 2, sequence_number);
+  ModuleRTPUtility::AssignUWord32ToBuffer(header + 4, timestamp);
+  ModuleRTPUtility::AssignUWord32ToBuffer(header + 8, ssrc);
   int32_t rtp_header_length = 12;
 
   // Add the CSRCs if any.
-  if (include_csrcs_ && csrcs_ > 0) {
-    if (csrcs_ > kRtpCsrcSize) {
+  if (num_csrcs > 0) {
+    if (num_csrcs > kRtpCsrcSize) {
       // error
       assert(false);
       return -1;
     }
-    uint8_t *ptr = &data_buffer[rtp_header_length];
-    for (uint32_t i = 0; i < csrcs_; ++i) {
-      ModuleRTPUtility::AssignUWord32ToBuffer(ptr, csrc_[i]);
+    uint8_t *ptr = &header[rtp_header_length];
+    for (int i = 0; i < num_csrcs; ++i) {
+      ModuleRTPUtility::AssignUWord32ToBuffer(ptr, csrcs[i]);
       ptr += 4;
     }
-    data_buffer[0] = (data_buffer[0] & 0xf0) | csrcs_;
+    header[0] = (header[0] & 0xf0) | num_csrcs;
 
     // Update length of header.
-    rtp_header_length += sizeof(uint32_t) * csrcs_;
+    rtp_header_length += sizeof(uint32_t) * num_csrcs;
   }
-  sequence_number_++;  // Prepare for next packet.
 
-  uint16_t len = BuildRTPHeaderExtension(data_buffer + rtp_header_length);
-  if (len) {
-    data_buffer[0] |= 0x10;  // Set extension bit.
+  uint16_t len = BuildRTPHeaderExtension(header + rtp_header_length);
+  if (len > 0) {
+    header[0] |= 0x10;  // Set extension bit.
     rtp_header_length += len;
   }
   return rtp_header_length;
 }
 
-uint16_t RTPSender::BuildRTPHeaderExtension(
-    uint8_t *data_buffer) const {
+int32_t RTPSender::BuildRTPheader(
+    uint8_t *data_buffer, const int8_t payload_type,
+    const bool marker_bit, const uint32_t capture_timestamp,
+    int64_t capture_time_ms, const bool time_stamp_provided,
+    const bool inc_sequence_number) {
+  assert(payload_type >= 0);
+  CriticalSectionScoped cs(send_critsect_);
+
+  if (time_stamp_provided) {
+    timestamp_ = start_time_stamp_ + capture_timestamp;
+  } else {
+    // Make a unique time stamp.
+    // We can't inc by the actual time, since then we increase the risk of back
+    // timing.
+    timestamp_++;
+  }
+  uint32_t sequence_number = sequence_number_++;
+  capture_time_ms_ = capture_time_ms;
+  last_packet_marker_bit_ = marker_bit;
+  int csrcs_length = 0;
+  if (include_csrcs_)
+    csrcs_length = num_csrcs_;
+  return CreateRTPHeader(data_buffer, payload_type, ssrc_, marker_bit,
+                         timestamp_, sequence_number, csrcs_, csrcs_length);
+}
+
+uint16_t RTPSender::BuildRTPHeaderExtension(uint8_t* data_buffer) const {
   if (rtp_header_extension_map_.Size() <= 0) {
     return 0;
   }
@@ -849,11 +946,11 @@ uint16_t RTPSender::BuildRTPHeaderExtension(
   //  |                             ....                              |
   //
   const uint32_t kPosLength = 2;
-  const uint32_t kHeaderLength = RTP_ONE_BYTE_HEADER_LENGTH_IN_BYTES;
+  const uint32_t kHeaderLength = kRtpOneByteHeaderLength;
 
   // Add extension ID (0xBEDE).
   ModuleRTPUtility::AssignUWord16ToBuffer(data_buffer,
-                                          RTP_ONE_BYTE_HEADER_EXTENSION);
+                                          kRtpOneByteHeaderExtensionId);
 
   // Add extensions.
   uint16_t total_block_length = 0;
@@ -861,9 +958,22 @@ uint16_t RTPSender::BuildRTPHeaderExtension(
   RTPExtensionType type = rtp_header_extension_map_.First();
   while (type != kRtpExtensionNone) {
     uint8_t block_length = 0;
-    if (type == kRtpExtensionTransmissionTimeOffset) {
-      block_length = BuildTransmissionTimeOffsetExtension(
-                         data_buffer + kHeaderLength + total_block_length);
+    switch (type) {
+      case kRtpExtensionTransmissionTimeOffset:
+        block_length = BuildTransmissionTimeOffsetExtension(
+            data_buffer + kHeaderLength + total_block_length);
+        break;
+      case kRtpExtensionAudioLevel:
+        // Because AudioLevel is handled specially by RTPSenderAudio, we pretend
+        // we don't have to care about it here, which is true until we wan't to
+        // use it together with any of the other extensions we support.
+        break;
+      case kRtpExtensionAbsoluteSendTime:
+        block_length = BuildAbsoluteSendTimeExtension(
+            data_buffer + kHeaderLength + total_block_length);
+        break;
+      default:
+        assert(false);
     }
     total_block_length += block_length;
     type = rtp_header_extension_map_.Next(type);
@@ -906,40 +1016,76 @@ uint8_t RTPSender::BuildTransmissionTimeOffsetExtension(
     // Not registered.
     return 0;
   }
-  int pos = 0;
+  size_t pos = 0;
   const uint8_t len = 2;
   data_buffer[pos++] = (id << 4) + len;
   ModuleRTPUtility::AssignUWord24ToBuffer(data_buffer + pos,
                                           transmission_time_offset_);
   pos += 3;
-  assert(pos == TRANSMISSION_TIME_OFFSET_LENGTH_IN_BYTES);
-  return TRANSMISSION_TIME_OFFSET_LENGTH_IN_BYTES;
+  assert(pos == kTransmissionTimeOffsetLength);
+  return kTransmissionTimeOffsetLength;
+}
+
+uint8_t RTPSender::BuildAbsoluteSendTimeExtension(
+    uint8_t* data_buffer) const {
+  // Absolute send time in RTP streams.
+  //
+  // The absolute send time is signaled to the receiver in-band using the
+  // general mechanism for RTP header extensions [RFC5285]. The payload
+  // of this extension (the transmitted value) is a 24-bit unsigned integer
+  // containing the sender's current time in seconds as a fixed point number
+  // with 18 bits fractional part.
+  //
+  // The form of the absolute send time extension block:
+  //
+  //    0                   1                   2                   3
+  //    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+  //   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+  //   |  ID   | len=2 |              absolute send time               |
+  //   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+  // Get id defined by user.
+  uint8_t id;
+  if (rtp_header_extension_map_.GetId(kRtpExtensionAbsoluteSendTime,
+                                      &id) != 0) {
+    // Not registered.
+    return 0;
+  }
+  size_t pos = 0;
+  const uint8_t len = 2;
+  data_buffer[pos++] = (id << 4) + len;
+  ModuleRTPUtility::AssignUWord24ToBuffer(data_buffer + pos,
+                                          absolute_send_time_);
+  pos += 3;
+  assert(pos == kAbsoluteSendTimeLength);
+  return kAbsoluteSendTimeLength;
 }
 
 bool RTPSender::UpdateTransmissionTimeOffset(
     uint8_t *rtp_packet, const uint16_t rtp_packet_length,
-    const WebRtcRTPHeader &rtp_header, const int64_t time_diff_ms) const {
+    const RTPHeader &rtp_header, const int64_t time_diff_ms) const {
   CriticalSectionScoped cs(send_critsect_);
 
-  // Get length until start of transmission block.
-  int transmission_block_pos =
+  // Get length until start of header extension block.
+  int extension_block_pos =
       rtp_header_extension_map_.GetLengthUntilBlockStartInBytes(
           kRtpExtensionTransmissionTimeOffset);
-  if (transmission_block_pos < 0) {
+  if (extension_block_pos < 0) {
     WEBRTC_TRACE(kTraceStream, kTraceRtpRtcp, id_,
                  "Failed to update transmission time offset, not registered.");
     return false;
   }
-  int block_pos = 12 + rtp_header.header.numCSRCs + transmission_block_pos;
-  if (rtp_packet_length < block_pos + 4 ||
-      rtp_header.header.headerLength < block_pos + 4) {
+  int block_pos = 12 + rtp_header.numCSRCs + extension_block_pos;
+  if (rtp_packet_length < block_pos + kTransmissionTimeOffsetLength ||
+      rtp_header.headerLength <
+          block_pos + kTransmissionTimeOffsetLength) {
     WEBRTC_TRACE(kTraceStream, kTraceRtpRtcp, id_,
                  "Failed to update transmission time offset, invalid length.");
     return false;
   }
   // Verify that header contains extension.
-  if (!((rtp_packet[12 + rtp_header.header.numCSRCs] == 0xBE) &&
-        (rtp_packet[12 + rtp_header.header.numCSRCs + 1] == 0xDE))) {
+  if (!((rtp_packet[12 + rtp_header.numCSRCs] == 0xBE) &&
+        (rtp_packet[12 + rtp_header.numCSRCs + 1] == 0xDE))) {
     WEBRTC_TRACE(
         kTraceStream, kTraceRtpRtcp, id_,
         "Failed to update transmission time offset, hdr extension not found.");
@@ -960,12 +1106,63 @@ bool RTPSender::UpdateTransmissionTimeOffset(
                  "Failed to update transmission time offset.");
     return false;
   }
-  // Update transmission offset field.
+  // Update transmission offset field (converting to a 90 kHz timestamp).
   ModuleRTPUtility::AssignUWord24ToBuffer(rtp_packet + block_pos + 1,
                                           time_diff_ms * 90);  // RTP timestamp.
   return true;
 }
 
+bool RTPSender::UpdateAbsoluteSendTime(
+    uint8_t *rtp_packet, const uint16_t rtp_packet_length,
+    const RTPHeader &rtp_header, const int64_t now_ms) const {
+  CriticalSectionScoped cs(send_critsect_);
+
+  // Get length until start of header extension block.
+  int extension_block_pos =
+      rtp_header_extension_map_.GetLengthUntilBlockStartInBytes(
+          kRtpExtensionAbsoluteSendTime);
+  if (extension_block_pos < 0) {
+    WEBRTC_TRACE(kTraceStream, kTraceRtpRtcp, id_,
+                 "Failed to update absolute send time, not registered.");
+    return false;
+  }
+  int block_pos = 12 + rtp_header.numCSRCs + extension_block_pos;
+  if (rtp_packet_length < block_pos + kAbsoluteSendTimeLength ||
+      rtp_header.headerLength < block_pos + kAbsoluteSendTimeLength) {
+    WEBRTC_TRACE(kTraceStream, kTraceRtpRtcp, id_,
+                 "Failed to update absolute send time, invalid length.");
+    return false;
+  }
+  // Verify that header contains extension.
+  if (!((rtp_packet[12 + rtp_header.numCSRCs] == 0xBE) &&
+        (rtp_packet[12 + rtp_header.numCSRCs + 1] == 0xDE))) {
+    WEBRTC_TRACE(
+        kTraceStream, kTraceRtpRtcp, id_,
+        "Failed to update absolute send time, hdr extension not found.");
+    return false;
+  }
+  // Get id.
+  uint8_t id = 0;
+  if (rtp_header_extension_map_.GetId(kRtpExtensionAbsoluteSendTime,
+                                      &id) != 0) {
+    WEBRTC_TRACE(kTraceStream, kTraceRtpRtcp, id_,
+                 "Failed to update absolute send time, no id.");
+    return false;
+  }
+  // Verify first byte in block.
+  const uint8_t first_block_byte = (id << 4) + 2;
+  if (rtp_packet[block_pos] != first_block_byte) {
+    WEBRTC_TRACE(kTraceStream, kTraceRtpRtcp, id_,
+                 "Failed to update absolute send time.");
+    return false;
+  }
+  // Update absolute send time field (convert ms to 24-bit unsigned with 18 bit
+  // fractional part).
+  ModuleRTPUtility::AssignUWord24ToBuffer(rtp_packet + block_pos + 1,
+                                          ((now_ms << 18) / 1000) & 0x00ffffff);
+  return true;
+}
+
 void RTPSender::SetSendingStatus(const bool enabled) {
   if (enabled) {
     uint32_t frequency_hz;
@@ -1019,7 +1216,7 @@ bool RTPSender::SendingMedia() const {
 
 uint32_t RTPSender::Timestamp() const {
   CriticalSectionScoped cs(send_critsect_);
-  return time_stamp_;
+  return timestamp_;
 }
 
 void RTPSender::SetStartTimestamp(uint32_t timestamp, bool force) {
@@ -1082,18 +1279,18 @@ void RTPSender::SetCSRCs(const uint32_t arr_of_csrc[kRtpCsrcSize],
   CriticalSectionScoped cs(send_critsect_);
 
   for (int i = 0; i < arr_length; i++) {
-    csrc_[i] = arr_of_csrc[i];
+    csrcs_[i] = arr_of_csrc[i];
   }
-  csrcs_ = arr_length;
+  num_csrcs_ = arr_length;
 }
 
 int32_t RTPSender::CSRCs(uint32_t arr_of_csrc[kRtpCsrcSize]) const {
   assert(arr_of_csrc);
   CriticalSectionScoped cs(send_critsect_);
-  for (int i = 0; i < csrcs_ && i < kRtpCsrcSize; i++) {
-    arr_of_csrc[i] = csrc_[i];
+  for (int i = 0; i < num_csrcs_ && i < kRtpCsrcSize; i++) {
+    arr_of_csrc[i] = csrcs_[i];
   }
-  return csrcs_;
+  return num_csrcs_;
 }
 
 void RTPSender::SetSequenceNumber(uint16_t seq) {
@@ -1227,16 +1424,16 @@ void RTPSender::BuildRtxPacket(uint8_t* buffer, uint16_t* length,
   ModuleRTPUtility::RTPHeaderParser rtp_parser(
       reinterpret_cast<const uint8_t *>(buffer), *length);
 
-  WebRtcRTPHeader rtp_header;
+  RTPHeader rtp_header;
   rtp_parser.Parse(rtp_header);
 
   // Add original RTP header.
-  memcpy(data_buffer_rtx, buffer, rtp_header.header.headerLength);
+  memcpy(data_buffer_rtx, buffer, rtp_header.headerLength);
 
   // Replace payload type, if a specific type is set for RTX.
   if (payload_type_rtx_ != -1) {
     data_buffer_rtx[1] = static_cast<uint8_t>(payload_type_rtx_);
-    if (rtp_header.header.markerBit)
+    if (rtp_header.markerBit)
       data_buffer_rtx[1] |= kRtpMarkerBitMask;
   }
 
@@ -1249,14 +1446,13 @@ void RTPSender::BuildRtxPacket(uint8_t* buffer, uint16_t* length,
   ModuleRTPUtility::AssignUWord32ToBuffer(ptr, ssrc_rtx_);
 
   // Add OSN (original sequence number).
-  ptr = data_buffer_rtx + rtp_header.header.headerLength;
-  ModuleRTPUtility::AssignUWord16ToBuffer(ptr,
-                                          rtp_header.header.sequenceNumber);
+  ptr = data_buffer_rtx + rtp_header.headerLength;
+  ModuleRTPUtility::AssignUWord16ToBuffer(ptr, rtp_header.sequenceNumber);
   ptr += 2;
 
   // Add original payload data.
-  memcpy(ptr, buffer + rtp_header.header.headerLength,
-         *length - rtp_header.header.headerLength);
+  memcpy(ptr, buffer + rtp_header.headerLength,
+         *length - rtp_header.headerLength);
   *length += 2;
 }
 
