@@ -1,8 +1,10 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/MemoryReporting.h"
 #include "mozilla/dom/ContentChild.h"
 
 #include "mozilla/Attributes.h"
@@ -13,6 +15,7 @@
 
 #include "mozilla/Preferences.h"
 #include "nsAppDirectoryServiceDefs.h"
+#include "nsDataHashtable.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsICategoryManager.h"
 #include "nsCategoryManagerUtils.h"
@@ -28,6 +31,7 @@
 #include "nsCOMArray.h"
 #include "nsXPCOMCID.h"
 #include "nsAutoPtr.h"
+#include "nsPrintfCString.h"
 
 #include "nsQuickSort.h"
 #include "pldhash.h"
@@ -42,6 +46,9 @@
 #include "nsTArray.h"
 #include "nsRefPtrHashtable.h"
 #include "nsIMemoryReporter.h"
+#include "nsThreadUtils.h"
+
+class PrefCallback;
 
 namespace mozilla {
 
@@ -159,12 +166,10 @@ static nsTArray<nsAutoPtr<CacheData> >* gCacheData = nullptr;
 static nsRefPtrHashtable<ValueObserverHashKey,
                          ValueObserver>* gObserverTable = nullptr;
 
-NS_MEMORY_REPORTER_MALLOC_SIZEOF_FUN(PreferencesMallocSizeOf)
-
 static size_t
 SizeOfObserverEntryExcludingThis(ValueObserverHashKey* aKey,
                                  const nsRefPtr<ValueObserver>& aData,
-                                 nsMallocSizeOfFun aMallocSizeOf,
+                                 mozilla::MallocSizeOf aMallocSizeOf,
                                  void*)
 {
   size_t n = 0;
@@ -173,52 +178,176 @@ SizeOfObserverEntryExcludingThis(ValueObserverHashKey* aKey,
   return n;
 }
 
-// static
-int64_t
-Preferences::GetPreferencesMemoryUsed()
+// Although this is a member of Preferences, it measures sPreferences and
+// several other global structures.
+/* static */ int64_t
+Preferences::SizeOfIncludingThisAndOtherStuff(mozilla::MallocSizeOf aMallocSizeOf)
 {
   NS_ENSURE_TRUE(InitStaticMembers(), 0);
 
-  size_t n = 0;
-  n += PreferencesMallocSizeOf(sPreferences);
+  size_t n = aMallocSizeOf(sPreferences);
   if (gHashTable.ops) {
     // pref keys are allocated in a private arena, which we count elsewhere.
     // pref stringvals are allocated out of the same private arena.
-    n += PL_DHashTableSizeOfExcludingThis(&gHashTable, nullptr,
-                                          PreferencesMallocSizeOf);
+    n += PL_DHashTableSizeOfExcludingThis(&gHashTable, nullptr, aMallocSizeOf);
   }
   if (gCacheData) {
-    n += gCacheData->SizeOfIncludingThis(PreferencesMallocSizeOf);
+    n += gCacheData->SizeOfIncludingThis(aMallocSizeOf);
     for (uint32_t i = 0, count = gCacheData->Length(); i < count; ++i) {
-      n += PreferencesMallocSizeOf((*gCacheData)[i]);
+      n += aMallocSizeOf((*gCacheData)[i]);
     }
   }
   if (gObserverTable) {
-    n += PreferencesMallocSizeOf(gObserverTable);
+    n += aMallocSizeOf(gObserverTable);
     n += gObserverTable->SizeOfExcludingThis(SizeOfObserverEntryExcludingThis,
-                                             PreferencesMallocSizeOf);
+                                             aMallocSizeOf);
   }
   // We don't measure sRootBranch and sDefaultRootBranch here because
   // DMD indicates they are not significant.
-  n += pref_SizeOfPrivateData(PreferencesMallocSizeOf);
+  n += pref_SizeOfPrivateData(aMallocSizeOf);
   return n;
 }
 
-NS_MEMORY_REPORTER_IMPLEMENT(Preferences,
-  "explicit/preferences",
-  KIND_HEAP,
-  UNITS_BYTES,
-  Preferences::GetPreferencesMemoryUsed,
-  "Memory used by the preferences system.")
+NS_MEMORY_REPORTER_MALLOC_SIZEOF_FUN(PreferencesMallocSizeOf)
+
+class PreferencesReporter MOZ_FINAL : public nsIMemoryReporter
+{
+public:
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIMEMORYREPORTER
+protected:
+  static const uint32_t kSuspectReferentCount = 1000;
+  static PLDHashOperator CountReferents(PrefCallback* aKey,
+                                        nsAutoPtr<PrefCallback>& aCallback,
+                                        void* aClosure);
+};
+
+NS_IMPL_ISUPPORTS1(PreferencesReporter, nsIMemoryReporter)
+
+NS_IMETHODIMP
+PreferencesReporter::GetName(nsACString& aName)
+{
+  aName.AssignLiteral("preference-service");
+  return NS_OK;
+}
+
+struct PreferencesReferentCount {
+  PreferencesReferentCount() : numStrong(0), numWeakAlive(0), numWeakDead(0) {}
+  size_t numStrong;
+  size_t numWeakAlive;
+  size_t numWeakDead;
+  nsTArray<nsCString> suspectPreferences;
+  // Count of the number of referents for each preference.
+  nsDataHashtable<nsCStringHashKey, uint32_t> prefCounter;
+};
+
+PLDHashOperator
+PreferencesReporter::CountReferents(PrefCallback* aKey,
+                                    nsAutoPtr<PrefCallback>& aCallback,
+                                    void* aClosure)
+{
+  PreferencesReferentCount* referentCount =
+    static_cast<PreferencesReferentCount*>(aClosure);
+
+  nsPrefBranch* prefBranch = aCallback->GetPrefBranch();
+  const char* pref = prefBranch->getPrefName(aCallback->GetDomain().get());
+
+  if (aCallback->IsWeak()) {
+    nsCOMPtr<nsIObserver> callbackRef = do_QueryReferent(aCallback->mWeakRef);
+    if (callbackRef) {
+      referentCount->numWeakAlive++;
+    } else {
+      referentCount->numWeakDead++;
+    }
+  } else {
+    referentCount->numStrong++;
+  }
+
+  nsDependentCString prefString(pref);
+  uint32_t oldCount = 0;
+  referentCount->prefCounter.Get(prefString, &oldCount);
+  uint32_t currentCount = oldCount + 1;
+  referentCount->prefCounter.Put(prefString, currentCount);
+
+  // Keep track of preferences that have a suspiciously large
+  // number of referents (symptom of leak).
+  if (currentCount == kSuspectReferentCount) {
+    referentCount->suspectPreferences.AppendElement(prefString);
+  }
+
+  return PL_DHASH_NEXT;
+}
+
+NS_IMETHODIMP
+PreferencesReporter::CollectReports(nsIMemoryReporterCallback* aCb,
+                                    nsISupports* aClosure)
+{
+#define REPORT(_path, _kind, _units, _amount, _desc)                          \
+    do {                                                                      \
+      nsresult rv;                                                            \
+      rv = aCb->Callback(EmptyCString(), _path, _kind,                        \
+                         _units, _amount, NS_LITERAL_CSTRING(_desc),          \
+                         aClosure);                                           \
+      NS_ENSURE_SUCCESS(rv, rv);                                              \
+    } while (0)
+
+  REPORT(NS_LITERAL_CSTRING("explicit/preferences"),
+         nsIMemoryReporter::KIND_HEAP, nsIMemoryReporter::UNITS_BYTES,
+         Preferences::SizeOfIncludingThisAndOtherStuff(PreferencesMallocSizeOf),
+         "Memory used by the preferences system.");
+
+  nsPrefBranch* rootBranch =
+    static_cast<nsPrefBranch*>(Preferences::GetRootBranch());
+  if (!rootBranch) {
+    return NS_OK;
+  }
+
+  PreferencesReferentCount referentCount;
+  rootBranch->mObservers.Enumerate(&CountReferents, &referentCount);
+
+  for (uint32_t i = 0; i < referentCount.suspectPreferences.Length(); i++) {
+    nsCString& suspect = referentCount.suspectPreferences[i];
+    uint32_t totalReferentCount = 0;
+    referentCount.prefCounter.Get(suspect, &totalReferentCount);
+
+    nsPrintfCString suspectPath("preference-service-suspect/"
+                                "referent(pref=%s)", suspect.get());
+
+    REPORT(suspectPath,
+           nsIMemoryReporter::KIND_OTHER, nsIMemoryReporter::UNITS_COUNT,
+           totalReferentCount,
+           "A preference with a suspiciously large number "
+           "referents (symptom of a leak).");
+  }
+
+  REPORT(NS_LITERAL_CSTRING("preference-service/referent/strong"),
+         nsIMemoryReporter::KIND_OTHER, nsIMemoryReporter::UNITS_COUNT,
+         referentCount.numStrong,
+         "The number of strong referents held by the preference service.");
+
+  REPORT(NS_LITERAL_CSTRING("preference-service/referent/weak/alive"),
+         nsIMemoryReporter::KIND_OTHER, nsIMemoryReporter::UNITS_COUNT,
+         referentCount.numWeakAlive,
+         "The number of weak referents held by the preference service "
+         "that are still alive.");
+
+  REPORT(NS_LITERAL_CSTRING("preference-service/referent/weak/dead"),
+         nsIMemoryReporter::KIND_OTHER, nsIMemoryReporter::UNITS_COUNT,
+         referentCount.numWeakDead,
+         "The number of weak referents held by the preference service "
+         "that are dead.");
+
+#undef REPORT
+
+  return NS_OK;
+}
 
 namespace {
 class AddPreferencesMemoryReporterRunnable : public nsRunnable
 {
   NS_IMETHOD Run()
   {
-    nsCOMPtr<nsIMemoryReporter> reporter =
-      new NS_MEMORY_REPORTER_NAME(Preferences);
-    return NS_RegisterMemoryReporter(reporter);
+    return NS_RegisterMemoryReporter(new PreferencesReporter());
   }
 };
 } // anonymous namespace
@@ -251,7 +380,6 @@ Preferences::GetInstanceForService()
   gCacheData = new nsTArray<nsAutoPtr<CacheData> >();
 
   gObserverTable = new nsRefPtrHashtable<ValueObserverHashKey, ValueObserver>();
-  gObserverTable->Init();
 
   // Preferences::GetInstanceForService() can be called from GetService(), and
   // NS_RegisterMemoryReporter calls GetService(nsIMemoryReporter).  To avoid a
@@ -284,7 +412,7 @@ Preferences::Shutdown()
   if (!sShutdown) {
     sShutdown = true; // Don't create the singleton instance after here.
 
-    // Don't set NULL to sPreferences here.  The instance may be grabbed by
+    // Don't set sPreferences to nullptr here.  The instance may be grabbed by
     // other modules.  The utility methods of Preferences should be available
     // until the singleton instance actually released.
     if (sPreferences) {
@@ -326,8 +454,8 @@ Preferences::~Preferences()
  * nsISupports Implementation
  */
 
-NS_IMPL_THREADSAFE_ADDREF(Preferences)
-NS_IMPL_THREADSAFE_RELEASE(Preferences)
+NS_IMPL_ADDREF(Preferences)
+NS_IMPL_RELEASE(Preferences)
 
 NS_INTERFACE_MAP_BEGIN(Preferences)
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIPrefService)
@@ -390,6 +518,7 @@ Preferences::Init()
   rv = observerService->AddObserver(this, "profile-before-change", true);
 
   observerService->AddObserver(this, "load-extension-defaults", true);
+  observerService->AddObserver(this, "suspend_process_notification", true);
 
   return(rv);
 }
@@ -425,6 +554,11 @@ Preferences::Observe(nsISupports *aSubject, const char *aTopic,
   } else if (!nsCRT::strcmp(aTopic, "reload-default-prefs")) {
     // Reload the default prefs from file.
     pref_InitInitialObjects();
+  } else if (!nsCRT::strcmp(aTopic, "suspend_process_notification")) {
+    // Our process is being suspended. The OS may wake our process later,
+    // or it may kill the process. In case our process is going to be killed
+    // from the suspended state, we save preferences before suspending.
+    rv = SavePrefFile(nullptr);
   }
   return rv;
 }
@@ -525,7 +659,7 @@ ReadExtensionPrefs(nsIFile *aFile)
     uint32_t read;
 
     PrefParseState ps;
-    PREF_InitParseState(&ps, PREF_ReaderCallback, NULL);
+    PREF_InitParseState(&ps, PREF_ReaderCallback, nullptr);
     while (NS_SUCCEEDED(rv = stream->Available(&avail)) && avail) {
       rv = stream->Read(buffer, 4096, &read);
       if (NS_FAILED(rv)) {
@@ -782,7 +916,7 @@ Preferences::WritePrefFile(nsIFile* aFile)
   PL_DHashTableEnumerate(&gHashTable, pref_savePref, &saveArgs);
     
   /* Sort the preferences to make a readable file on disk */
-  NS_QuickSort(valueArray, gHashTable.entryCount, sizeof(char *), pref_CompareStrings, NULL);
+  NS_QuickSort(valueArray, gHashTable.entryCount, sizeof(char *), pref_CompareStrings, nullptr);
   
   // write out the file header
   outStream->Write(outHeader, sizeof(outHeader) - 1, &writeAmount);
@@ -832,7 +966,7 @@ static nsresult openPrefFile(nsIFile* aFile)
     return NS_ERROR_OUT_OF_MEMORY;
 
   PrefParseState ps;
-  PREF_InitParseState(&ps, PREF_ReaderCallback, NULL);
+  PREF_InitParseState(&ps, PREF_ReaderCallback, nullptr);
 
   // Read is not guaranteed to return a buf the size of fileSize,
   // but usually will.
@@ -1007,7 +1141,7 @@ static nsresult pref_ReadPrefFromJar(nsZipArchive* jarReader, const char *name)
   NS_ENSURE_TRUE(manifest.Buffer(), NS_ERROR_NOT_AVAILABLE);
 
   PrefParseState ps;
-  PREF_InitParseState(&ps, PREF_ReaderCallback, NULL);
+  PREF_InitParseState(&ps, PREF_ReaderCallback, nullptr);
   PREF_ParseBuf(&ps, manifest, manifest.Length());
   PREF_FinalizeParseState(&ps);
 
