@@ -9,7 +9,6 @@
 #include "nsIDNSListener.h"
 #include "nsIDNSRecord.h"
 #include "nsIDNSService.h"
-#include "nsNetUtil.h"
 #include "nsThreadUtils.h"
 #include "nsIConsoleService.h"
 #include "nsJSUtils.h"
@@ -17,6 +16,8 @@
 #include "prnetdb.h"
 #include "nsITimer.h"
 #include "mozilla/net/DNS.h"
+#include "nsServiceManagerUtils.h"
+#include "nsNetCID.h"
 
 namespace mozilla {
 namespace net {
@@ -315,8 +316,8 @@ PACErrorReporter(JSContext *cx, const char *message, JSErrorReport *report)
 // timeout of 0 means the normal necko timeout strategy, otherwise the dns request
 // will be canceled after aTimeout milliseconds
 static
-JSBool PACResolve(const nsCString &aHostName, NetAddr *aNetAddr,
-                  unsigned int aTimeout)
+bool PACResolve(const nsCString &aHostName, NetAddr *aNetAddr,
+                unsigned int aTimeout)
 {
   if (!sRunning) {
     NS_WARNING("PACResolve without a running ProxyAutoConfig object");
@@ -326,6 +327,14 @@ JSBool PACResolve(const nsCString &aHostName, NetAddr *aNetAddr,
   return sRunning->ResolveAddress(aHostName, aNetAddr, aTimeout);
 }
 
+ProxyAutoConfig::ProxyAutoConfig()
+  : mJSRuntime(nullptr)
+  , mJSNeedsSetup(false)
+  , mShutdown(false)
+{
+  MOZ_COUNT_CTOR(ProxyAutoConfig);
+}
+
 bool
 ProxyAutoConfig::ResolveAddress(const nsCString &aHostName,
                                 NetAddr *aNetAddr,
@@ -337,7 +346,9 @@ ProxyAutoConfig::ResolveAddress(const nsCString &aHostName,
 
   nsRefPtr<PACResolver> helper = new PACResolver();
 
-  if (NS_FAILED(dns->AsyncResolve(aHostName, 0, helper,
+  if (NS_FAILED(dns->AsyncResolve(aHostName,
+                                  nsIDNSService::RESOLVE_PRIORITY_MEDIUM,
+                                  helper,
                                   NS_GetCurrentThread(),
                                   getter_AddRefs(helper->mRequest))))
     return false;
@@ -382,7 +393,7 @@ bool PACResolveToString(const nsCString &aHostName,
 
 // dnsResolve(host) javascript implementation
 static
-JSBool PACDnsResolve(JSContext *cx, unsigned int argc, JS::Value *vp)
+bool PACDnsResolve(JSContext *cx, unsigned int argc, JS::Value *vp)
 {
   if (NS_IsMainThread()) {
     NS_WARNING("DNS Resolution From PAC on Main Thread. How did that happen?");
@@ -405,13 +416,13 @@ JSBool PACDnsResolve(JSContext *cx, unsigned int argc, JS::Value *vp)
   else {
     JS_SET_RVAL(cx, vp, JSVAL_NULL);
   }
-  
+
   return true;
 }
 
 // myIpAddress() javascript implementation
 static
-JSBool PACMyIpAddress(JSContext *cx, unsigned int argc, JS::Value *vp)
+bool PACMyIpAddress(JSContext *cx, unsigned int argc, JS::Value *vp)
 {
   if (NS_IsMainThread()) {
     NS_WARNING("DNS Resolution From PAC on Main Thread. How did that happen?");
@@ -420,7 +431,7 @@ JSBool PACMyIpAddress(JSContext *cx, unsigned int argc, JS::Value *vp)
 
   if (!sRunning) {
     NS_WARNING("PAC myIPAddress without a running ProxyAutoConfig object");
-    return JS_FALSE;
+    return false;
   }
 
   return sRunning->MyIPAddress(vp);
@@ -428,7 +439,7 @@ JSBool PACMyIpAddress(JSContext *cx, unsigned int argc, JS::Value *vp)
 
 // proxyAlert(msg) javascript implementation
 static
-JSBool PACProxyAlert(JSContext *cx, unsigned int argc, JS::Value *vp)
+bool PACProxyAlert(JSContext *cx, unsigned int argc, JS::Value *vp)
 {
   JS::Rooted<JSString*> arg1(cx);
   if (!JS_ConvertArguments(cx, argc, JS_ARGV(cx, vp), "S", arg1.address()))
@@ -513,7 +524,7 @@ private:
   JSObject  *mGlobal;
   bool      mOK;
 
-  static JSClass sGlobalClass;
+  static const JSClass sGlobalClass;
 
   JSRuntimeWrapper()
     : mRuntime(nullptr), mContext(nullptr), mGlobal(nullptr), mOK(false)
@@ -560,7 +571,7 @@ private:
   }
 };
 
-JSClass JSRuntimeWrapper::sGlobalClass = {
+const JSClass JSRuntimeWrapper::sGlobalClass = {
   "PACResolutionThreadGlobal",
   JSCLASS_GLOBAL_FLAGS,
   JS_PropertyStub, JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
@@ -601,6 +612,11 @@ ProxyAutoConfig::SetupJS()
   JSAutoRequest ar(mJSRuntime->Context());
   JSAutoCompartment ac(mJSRuntime->Context(), mJSRuntime->Global());
 
+  // check if this is a data: uri so that we don't spam the js console with
+  // huge meaningless strings. this is not on the main thread, so it can't
+  // use nsIRUI scheme methods
+  bool isDataURI = nsDependentCSubstring(mPACURI, 0, 5).LowerCaseEqualsASCII("data:", 5);
+
   sRunning = this;
   JSScript *script = JS_CompileScript(mJSRuntime->Context(),
                                       mJSRuntime->Global(),
@@ -609,7 +625,12 @@ ProxyAutoConfig::SetupJS()
   if (!script ||
       !JS_ExecuteScript(mJSRuntime->Context(), mJSRuntime->Global(), script, nullptr)) {
     nsString alertMessage(NS_LITERAL_STRING("PAC file failed to install from "));
-    alertMessage += NS_ConvertUTF8toUTF16(mPACURI);
+    if (isDataURI) {
+      alertMessage += NS_LITERAL_STRING("data: URI");
+    }
+    else {
+      alertMessage += NS_ConvertUTF8toUTF16(mPACURI);
+    }
     PACLogToConsole(alertMessage);
     sRunning = nullptr;
     return NS_ERROR_FAILURE;
@@ -618,7 +639,12 @@ ProxyAutoConfig::SetupJS()
 
   mJSRuntime->SetOK();
   nsString alertMessage(NS_LITERAL_STRING("PAC file installed from "));
-  alertMessage += NS_ConvertUTF8toUTF16(mPACURI);
+  if (isDataURI) {
+    alertMessage += NS_LITERAL_STRING("data: URI");
+  }
+  else {
+    alertMessage += NS_ConvertUTF8toUTF16(mPACURI);
+  }
   PACLogToConsole(alertMessage);
 
   // we don't need these now
@@ -658,7 +684,7 @@ ProxyAutoConfig::GetProxyForURI(const nsCString &aTestURI,
 
     JS::Value argv[2] = { uriValue, hostValue };
     JS::Rooted<JS::Value> rval(cx);
-    JSBool ok = JS_CallFunctionName(cx, mJSRuntime->Global(),
+    bool ok = JS_CallFunctionName(cx, mJSRuntime->Global(),
                                     "FindProxyForURL", 2, argv, rval.address());
 
     if (ok && rval.isString()) {
