@@ -8,34 +8,32 @@
 
 #include "nsHttp.h"
 #include "nsHttpAuthCache.h"
-#include "nsHttpConnection.h"
 #include "nsHttpConnectionMgr.h"
 #include "ASpdySession.h"
 
-#include "nsXPIDLString.h"
 #include "nsString.h"
 #include "nsCOMPtr.h"
 #include "nsWeakReference.h"
 
+#include "nsIHttpDataUsage.h"
 #include "nsIHttpProtocolHandler.h"
-#include "nsIProtocolProxyService.h"
-#include "nsIIOService.h"
 #include "nsIObserver.h"
-#include "nsIObserverService.h"
-#include "nsIStreamConverterService.h"
-#include "nsICacheSession.h"
-#include "nsICookieService.h"
-#include "nsITimer.h"
-#include "nsIStrictTransportSecurityService.h"
 #include "nsISpeculativeConnect.h"
+#include "nsICache.h"
 
+class nsHttpConnection;
 class nsHttpConnectionInfo;
 class nsHttpHeaderArray;
 class nsHttpTransaction;
-class nsAHttpTransaction;
 class nsIHttpChannel;
 class nsIPrefBranch;
 class nsICancelable;
+class nsICookieService;
+class nsIIOService;
+class nsIObserverService;
+class nsISiteSecurityService;
+class nsIStreamConverterService;
+class nsITimer;
 
 namespace mozilla {
 namespace net {
@@ -53,6 +51,7 @@ class nsHttpHandler : public nsIHttpProtocolHandler
                     , public nsIObserver
                     , public nsSupportsWeakReference
                     , public nsISpeculativeConnect
+                    , public nsIHttpDataUsage
 {
 public:
     NS_DECL_THREADSAFE_ISUPPORTS
@@ -61,6 +60,7 @@ public:
     NS_DECL_NSIHTTPPROTOCOLHANDLER
     NS_DECL_NSIOBSERVER
     NS_DECL_NSISPECULATIVECONNECT
+    NS_DECL_NSIHTTPDATAUSAGE
 
     nsHttpHandler();
     virtual ~nsHttpHandler();
@@ -97,6 +97,7 @@ public:
     bool           IsSpdyEnabled() { return mEnableSpdy; }
     bool           IsSpdyV2Enabled() { return mSpdyV2; }
     bool           IsSpdyV3Enabled() { return mSpdyV3; }
+    bool           IsSpdyV31Enabled() { return mSpdyV31; }
     bool           CoalesceSpdy() { return mCoalesceSpdy; }
     bool           UseSpdyPersistentSettings() { return mSpdyPersistentSettings; }
     uint32_t       SpdySendingChunkSize() { return mSpdySendingChunkSize; }
@@ -196,7 +197,7 @@ public:
     nsresult GetStreamConverterService(nsIStreamConverterService **);
     nsresult GetIOService(nsIIOService** service);
     nsICookieService * GetCookieService(); // not addrefed
-    nsIStrictTransportSecurityService * GetSTSService();
+    nsISiteSecurityService * GetSSService();
 
     // callable from socket thread only
     uint32_t Get32BitsOfPseudoRandom();
@@ -308,7 +309,7 @@ private:
     nsMainThreadPtrHandle<nsIStreamConverterService> mStreamConvSvc;
     nsMainThreadPtrHandle<nsIObserverService>        mObserverService;
     nsMainThreadPtrHandle<nsICookieService>          mCookieService;
-    nsMainThreadPtrHandle<nsIStrictTransportSecurityService> mSTSService;
+    nsMainThreadPtrHandle<nsISiteSecurityService>    mSSService;
 
     // the authentication credentials cache
     nsHttpAuthCache mAuthCache;
@@ -417,6 +418,7 @@ private:
     bool           mEnableSpdy;
     bool           mSpdyV2;
     bool           mSpdyV3;
+    bool           mSpdyV31;
     bool           mCoalesceSpdy;
     bool           mSpdyPersistentSettings;
     bool           mAllowSpdyPush;
@@ -476,8 +478,30 @@ public:
     }
 
 private:
+    // for nsIHttpDataUsage
+    uint64_t mEthernetBytesRead;
+    uint64_t mEthernetBytesWritten;
+    uint64_t mCellBytesRead;
+    uint64_t mCellBytesWritten;
+    bool     mNetworkTypeKnown;
+    bool     mNetworkTypeWasEthernet;
+
     nsRefPtr<mozilla::net::Tickler> mWifiTickler;
+    nsresult GetNetworkEthernetInfo(nsIInterfaceRequestor *cb,
+                                    bool *ethernet);
+    nsresult GetNetworkEthernetInfoInner(nsIInterfaceRequestor *cb,
+                                         bool *ethernet);
+    nsresult GetNetworkInfo(nsIInterfaceRequestor *cb,
+                            bool *ethernet, uint32_t *gw);
+    nsresult GetNetworkInfoInner(nsIInterfaceRequestor *cb,
+                                 bool *ethernet, uint32_t *gw);
     void TickleWifi(nsIInterfaceRequestor *cb);
+
+public:
+    // this is called to update the member variables used for nsIHttpDataUsage
+    // it can be called from any thread
+    void UpdateDataUsage(nsIInterfaceRequestor *cb,
+                         uint64_t bytesRead, uint64_t bytesWritten);
 };
 
 extern nsHttpHandler *gHttpHandler;
